@@ -18,8 +18,7 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 // THE SOFTWARE.
 
-#ifndef CPLUSPLUS_TOKEN_H
-#define CPLUSPLUS_TOKEN_H
+#pragma once
 
 #include "CPlusPlusForwardDeclarations.h"
 
@@ -49,16 +48,34 @@ enum Kind {
     T_UTF8_STRING_LITERAL,
     T_UTF16_STRING_LITERAL,
     T_UTF32_STRING_LITERAL,
-    T_RAW_STRING_LITERAL,
+    T_FIRST_RAW_STRING_LITERAL,
+    T_RAW_STRING_LITERAL = T_FIRST_RAW_STRING_LITERAL,
     T_RAW_WIDE_STRING_LITERAL,
     T_RAW_UTF8_STRING_LITERAL,
     T_RAW_UTF16_STRING_LITERAL,
     T_RAW_UTF32_STRING_LITERAL,
+    T_LAST_RAW_STRING_LITERAL = T_RAW_UTF32_STRING_LITERAL,
     T_AT_STRING_LITERAL,
     T_ANGLE_STRING_LITERAL,
     T_LAST_STRING_LITERAL = T_ANGLE_STRING_LITERAL,
     T_LAST_LITERAL = T_ANGLE_STRING_LITERAL,
 
+    T_FIRST_PUNCTUATION_OR_OPERATOR,
+    T_FIRST_PUNCTUATION = T_FIRST_PUNCTUATION_OR_OPERATOR,
+    T_COLON = T_FIRST_PUNCTUATION_OR_OPERATOR,
+    T_COLON_COLON,
+    T_COMMA,
+    T_GREATER,
+    T_LESS,
+    T_LBRACE,
+    T_LBRACKET,
+    T_LPAREN,
+    T_RBRACE,
+    T_RBRACKET,
+    T_RPAREN,
+    T_SEMICOLON,
+    T_LAST_PUNCTUATION = T_SEMICOLON,
+
     T_FIRST_OPERATOR,
     T_AMPER = T_FIRST_OPERATOR,
     T_AMPER_AMPER,
@@ -67,9 +84,6 @@ enum Kind {
     T_ARROW_STAR,
     T_CARET,
     T_CARET_EQUAL,
-    T_COLON,
-    T_COLON_COLON,
-    T_COMMA,
     T_SLASH,
     T_SLASH_EQUAL,
     T_DOT,
@@ -79,17 +93,12 @@ enum Kind {
     T_EQUAL_EQUAL,
     T_EXCLAIM,
     T_EXCLAIM_EQUAL,
-    T_GREATER,
     T_GREATER_EQUAL,
     T_GREATER_GREATER,
     T_GREATER_GREATER_EQUAL,
-    T_LBRACE,
-    T_LBRACKET,
-    T_LESS,
     T_LESS_EQUAL,
     T_LESS_LESS,
     T_LESS_LESS_EQUAL,
-    T_LPAREN,
     T_MINUS,
     T_MINUS_EQUAL,
     T_MINUS_MINUS,
@@ -104,15 +113,12 @@ enum Kind {
     T_POUND,
     T_POUND_POUND,
     T_QUESTION,
-    T_RBRACE,
-    T_RBRACKET,
-    T_RPAREN,
-    T_SEMICOLON,
     T_STAR,
     T_STAR_EQUAL,
     T_TILDE,
     T_TILDE_EQUAL,
     T_LAST_OPERATOR = T_TILDE_EQUAL,
+    T_LAST_PUNCTUATION_OR_OPERATOR = T_LAST_OPERATOR,
 
     T_FIRST_KEYWORD,
     T_ALIGNAS = T_FIRST_KEYWORD,
@@ -253,6 +259,8 @@ enum Kind {
     T_LAST_QT_KEYWORD = T_Q_GADGET,
     T_LAST_KEYWORD = T_LAST_QT_KEYWORD,
 
+    T_LAST_TOKEN = T_LAST_KEYWORD,  // keep this before the aliases below
+
     // aliases
     T_OR = T_PIPE_PIPE,
     T_AND = T_AMPER_AMPER,
@@ -284,7 +292,7 @@ enum Kind {
     T___VOLATILE__ = T_VOLATILE,
 
     T___ATTRIBUTE = T___ATTRIBUTE__,
-    T_LAST_TOKEN
+    T___ALIGNOF__ = T_ALIGNOF,
 };
 
 class CPLUSPLUS_EXPORT Token
@@ -325,6 +333,12 @@ public:
     inline bool isOperator() const
     { return f.kind >= T_FIRST_OPERATOR && f.kind <= T_LAST_OPERATOR; }
 
+    inline bool isPunctuation() const
+    { return f.kind >= T_FIRST_PUNCTUATION && f.kind <= T_LAST_PUNCTUATION; }
+
+    inline bool isPunctuationOrOperator() const
+    { return f.kind >= T_FIRST_PUNCTUATION_OR_OPERATOR && f.kind <= T_LAST_PUNCTUATION_OR_OPERATOR; }
+
     inline bool isKeyword() const
     { return f.kind >= T_FIRST_KEYWORD && f.kind < T_FIRST_PRIMITIVE; }
 
@@ -368,8 +382,10 @@ public:
         // The token is C++11 user-defined literal such as:
         // 12_km, 0.5_Pa, 'c'_X, "abd"_L, u16"xyz"_M
         unsigned userDefinedLiteral : 1;
+        // Indicates the token is a trigraph
+        unsigned trigraph      : 1;
         // Unused...
-        unsigned pad           : 2;
+        unsigned pad           : 1;
         // The token length in bytes and UTF16 chars.
         unsigned bytes         : 16;
         unsigned utf16chars    : 16;
@@ -403,12 +419,23 @@ struct LanguageFeatures
         return features;
     }
 
+    bool operator==(const LanguageFeatures &other) const
+    {
+        return flags == other.flags;
+    }
+
+    bool operator!=(const LanguageFeatures &other) const
+    {
+        return flags != other.flags;
+    }
+
     union {
         unsigned int flags;
         struct {
             unsigned int qtEnabled : 1; // If Qt is used.
             unsigned int qtMocRunEnabled : 1;
             unsigned int qtKeywordsEnabled : 1; // If Qt is used but QT_NO_KEYWORDS defined
+            unsigned int cxxEnabled : 1;
             unsigned int cxx11Enabled : 1;
             unsigned int objCEnabled : 1;
             unsigned int c99Enabled : 1;
@@ -417,5 +444,3 @@ struct LanguageFeatures
 };
 
 } // namespace CPlusPlus
-
-#endif // CPLUSPLUS_TOKEN_H
