@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,28 +9,22 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "contextpanewidgetimage.h"
 
-#include <utils/hostosinfo.h>
-
 #include "ui_contextpanewidgetimage.h"
 #include "ui_contextpanewidgetborderimage.h"
 #include <qmljs/qmljspropertyreader.h>
@@ -43,6 +37,7 @@
 #include <QMouseEvent>
 #include <QScrollArea>
 #include <QSlider>
+#include <QStyle>
 #include <QToolButton>
 #include <QDebug>
 
@@ -73,7 +68,7 @@ bool WheelFilter::eventFilter(QObject *obj, QEvent *event)
 
 ContextPaneWidgetImage::ContextPaneWidgetImage(QWidget *parent, bool borderImage) :
     QWidget(parent),
-    ui(0), uiBorderImage(0), previewWasVisible(false)
+    ui(nullptr), uiBorderImage(nullptr), previewWasVisible(false)
 {
     LabelFilter *labelFilter = new LabelFilter(this);
 
@@ -88,17 +83,28 @@ ContextPaneWidgetImage::ContextPaneWidgetImage(QWidget *parent, bool borderImage
         uiBorderImage->label->installEventFilter(labelFilter);
 
 
-        connect(uiBorderImage->verticalTileRadioButton,  SIGNAL(toggled(bool)), this, SLOT(onVerticalStretchChanged()));
-        connect(uiBorderImage->verticalStretchRadioButton,  SIGNAL(toggled(bool)), this, SLOT(onVerticalStretchChanged()));
-        connect(uiBorderImage->verticalTileRadioButtonNoCrop,  SIGNAL(toggled(bool)), this, SLOT(onVerticalStretchChanged()));
+        connect(uiBorderImage->verticalTileRadioButton, &QRadioButton::toggled,
+                this, &ContextPaneWidgetImage::onVerticalStretchChanged);
+        connect(uiBorderImage->verticalStretchRadioButton, &QRadioButton::toggled,
+                this, &ContextPaneWidgetImage::onVerticalStretchChanged);
+        connect(uiBorderImage->verticalTileRadioButtonNoCrop, &QRadioButton::toggled,
+                this, &ContextPaneWidgetImage::onVerticalStretchChanged);
 
-        connect(uiBorderImage->horizontalTileRadioButton,  SIGNAL(toggled(bool)), this, SLOT(onHorizontalStretchChanged()));
-        connect(uiBorderImage->horizontalStretchRadioButton,  SIGNAL(toggled(bool)), this, SLOT(onHorizontalStretchChanged()));
-        connect(uiBorderImage->horizontalTileRadioButtonNoCrop, SIGNAL(toggled(bool)), this, SLOT(onHorizontalStretchChanged()));
-        connect(previewDialog()->previewLabel(), SIGNAL(leftMarginChanged()), this, SLOT(onLeftMarginsChanged()));
-        connect(previewDialog()->previewLabel(), SIGNAL(rightMarginChanged()), this, SLOT(onRightMarginsChanged()));
-        connect(previewDialog()->previewLabel(), SIGNAL(topMarginChanged()), this, SLOT(onTopMarginsChanged()));
-        connect(previewDialog()->previewLabel(), SIGNAL(bottomMarginChanged()), this, SLOT(onBottomMarginsChanged()));
+        connect(uiBorderImage->horizontalTileRadioButton, &QRadioButton::toggled,
+                this, &ContextPaneWidgetImage::onHorizontalStretchChanged);
+        connect(uiBorderImage->horizontalStretchRadioButton, &QRadioButton::toggled,
+                this, &ContextPaneWidgetImage::onHorizontalStretchChanged);
+        connect(uiBorderImage->horizontalTileRadioButtonNoCrop, &QRadioButton::toggled,
+                this, &ContextPaneWidgetImage::onHorizontalStretchChanged);
+        PreviewLabel *previewLabel = previewDialog()->previewLabel();
+        connect(previewLabel, &PreviewLabel::leftMarginChanged,
+                this, &ContextPaneWidgetImage::onLeftMarginsChanged);
+        connect(previewLabel, &PreviewLabel::rightMarginChanged,
+                this, &ContextPaneWidgetImage::onRightMarginsChanged);
+        connect(previewLabel, &PreviewLabel::topMarginChanged,
+                this, &ContextPaneWidgetImage::onTopMarginsChanged);
+        connect(previewLabel, &PreviewLabel::bottomMarginChanged,
+                this, &ContextPaneWidgetImage::onBottomMarginsChanged);
 
     } else {
         ui = new Ui::ContextPaneWidgetImage;
@@ -108,19 +114,27 @@ ContextPaneWidgetImage::ContextPaneWidgetImage(QWidget *parent, bool borderImage
         m_fileWidget = ui->fileWidget;
         m_sizeLabel = ui->sizeLabel;
 
-        connect(ui->stretchRadioButton, SIGNAL(toggled(bool)), this, SLOT(onStretchChanged()));
-        connect(ui->tileRadioButton, SIGNAL(toggled(bool)), this, SLOT(onStretchChanged()));
-        connect(ui->horizontalStretchRadioButton, SIGNAL(toggled(bool)), this, SLOT(onStretchChanged()));
-        connect(ui->verticalStretchRadioButton, SIGNAL(toggled(bool)), this, SLOT(onStretchChanged()));
-        connect(ui->preserveAspectFitRadioButton, SIGNAL(toggled(bool)), this, SLOT(onStretchChanged()));
-        connect(ui->cropAspectFitRadioButton, SIGNAL(toggled(bool)), this, SLOT(onStretchChanged()));
+        connect(ui->stretchRadioButton, &QRadioButton::toggled,
+                this, &ContextPaneWidgetImage::onStretchChanged);
+        connect(ui->tileRadioButton, &QRadioButton::toggled,
+                this, &ContextPaneWidgetImage::onStretchChanged);
+        connect(ui->horizontalStretchRadioButton, &QRadioButton::toggled,
+                this, &ContextPaneWidgetImage::onStretchChanged);
+        connect(ui->verticalStretchRadioButton, &QRadioButton::toggled,
+                this, &ContextPaneWidgetImage::onStretchChanged);
+        connect(ui->preserveAspectFitRadioButton, &QRadioButton::toggled,
+                this, &ContextPaneWidgetImage::onStretchChanged);
+        connect(ui->cropAspectFitRadioButton, &QRadioButton::toggled,
+                this, &ContextPaneWidgetImage::onStretchChanged);
     }
     previewDialog();
     m_fileWidget->setShowComboBox(true);
     m_fileWidget->setFilter(QLatin1String("*.png *.gif *.jpg"));
 
-    connect(m_fileWidget, SIGNAL(fileNameChanged(QUrl)), this, SLOT(onFileNameChanged()));
-    connect(labelFilter, SIGNAL(doubleClicked()), this, SLOT(onPixmapDoubleClicked()));
+    connect(m_fileWidget, &FileWidget::fileNameChanged,
+            this, &ContextPaneWidgetImage::onFileNameChanged);
+    connect(labelFilter, &LabelFilter::doubleClicked,
+            this, &ContextPaneWidgetImage::onPixmapDoubleClicked);
 
 }
 
@@ -366,7 +380,7 @@ void ContextPaneWidgetImage::onPixmapDoubleClicked()
 void ContextPaneWidgetImage::onLeftMarginsChanged()
 {
     if (previewDialog()->previewLabel()->leftMarging())
-        propertyChanged(QLatin1String("border.left"), previewDialog()->previewLabel()->leftMarging());
+        emit propertyChanged("border.left", previewDialog()->previewLabel()->leftMarging());
     else
         emit removeProperty(QLatin1String("border.left"));
 }
@@ -374,7 +388,7 @@ void ContextPaneWidgetImage::onLeftMarginsChanged()
 void ContextPaneWidgetImage::onRightMarginsChanged()
 {
     if (previewDialog()->previewLabel()->rightMarging())
-        propertyChanged(QLatin1String("border.right"), previewDialog()->previewLabel()->rightMarging());
+        emit propertyChanged("border.right", previewDialog()->previewLabel()->rightMarging());
     else
         emit removeProperty(QLatin1String("border.right"));
 
@@ -384,7 +398,7 @@ void ContextPaneWidgetImage::onRightMarginsChanged()
 void ContextPaneWidgetImage::onTopMarginsChanged()
 {
     if (previewDialog()->previewLabel()->topMarging())
-        propertyChanged(QLatin1String("border.top"), previewDialog()->previewLabel()->topMarging());
+        emit propertyChanged("border.top", previewDialog()->previewLabel()->topMarging());
     else
         emit removeProperty(QLatin1String("border.top"));
 }
@@ -392,7 +406,7 @@ void ContextPaneWidgetImage::onTopMarginsChanged()
 void ContextPaneWidgetImage::onBottomMarginsChanged()
 {
     if (previewDialog()->previewLabel()->bottomMarging())
-        propertyChanged(QLatin1String("border.bottom"), previewDialog()->previewLabel()->bottomMarging());
+        emit propertyChanged("border.bottom", previewDialog()->previewLabel()->bottomMarging());
     else
         emit removeProperty(QLatin1String("border.bottom"));
 
@@ -469,7 +483,7 @@ void ContextPaneWidgetImage::setPixmap(const QString &fileName)
 
     if (m_borderImage) {
         QString localFileName(fileName);
-        if (QFile(fileName).exists()) {
+        if (QFileInfo::exists(fileName)) {
             if (fileName.endsWith(QLatin1String("sci"))) {
                 QString pixmapFileName;
                 int left = 0;
@@ -505,7 +519,8 @@ void ContextPaneWidgetImage::setPixmap(const QString &fileName)
             if (uiBorderImage->verticalTileRadioButtonNoCrop->isChecked())
                 verticalTileMode =Qt::RoundTile;
             QTileRules rules(horizontalTileMode, verticalTileMode);
-            QMargins margins(previewDialog()->previewLabel()->leftMarging() ,previewDialog()->previewLabel()->topMarging() ,previewDialog()->previewLabel()->rightMarging(), previewDialog()->previewLabel()->bottomMarging());
+            PreviewLabel *previewLabel = previewDialog()->previewLabel();
+            QMargins margins(previewLabel->leftMarging() ,previewLabel->topMarging() ,previewLabel->rightMarging(), previewLabel->bottomMarging());
             qDrawBorderPixmap(&p, QRect(0, 0, 76, 76), margins, source, source.rect(), margins, rules);
             //p.drawPixmap(0,0,76,76, source);
         } else {
@@ -513,7 +528,7 @@ void ContextPaneWidgetImage::setPixmap(const QString &fileName)
         }
         uiBorderImage->label->setPixmap(pix);
     } else {
-        if (QFile(fileName).exists()) {
+        if (QFileInfo::exists(fileName)) {
             QPixmap source(fileName);
             previewDialog()->setPixmap(source, 1);
             ui->sizeLabel->setText(QString::number(source.width()) + QLatin1Char('x') + QString::number(source.height()));
@@ -612,15 +627,10 @@ PreviewLabel::PreviewLabel(QWidget *parent)
 
     m_hooverInfo->setFrameShape(QFrame::StyledPanel);
     m_hooverInfo->setFrameShadow(QFrame::Sunken);
-
-    // TODO: The following code should be enabled for OSX
-    // when QTBUG-23205 is fixed
-    if (!Utils::HostOsInfo::isMacHost()) {
-        QGraphicsDropShadowEffect *dropShadowEffect = new QGraphicsDropShadowEffect;
-        dropShadowEffect->setBlurRadius(4);
-        dropShadowEffect->setOffset(2, 2);
-        m_hooverInfo->setGraphicsEffect(dropShadowEffect);
-    }
+    QGraphicsDropShadowEffect *dropShadowEffect = new QGraphicsDropShadowEffect;
+    dropShadowEffect->setBlurRadius(4);
+    dropShadowEffect->setOffset(2, 2);
+    m_hooverInfo->setGraphicsEffect(dropShadowEffect);
     m_hooverInfo->setAutoFillBackground(true);
     m_hooverInfo->raise();
 }
@@ -655,7 +665,7 @@ void PreviewLabel::paintEvent(QPaintEvent *event)
         p.setBackgroundMode(Qt::TransparentMode);
         {
             QPen pen(Qt::SolidLine);
-            pen.setColor("#F0F0F0");
+            pen.setColor(QColor(0xf0, 0xf0, 0xf0));
             p.setPen(pen);
 
             if (m_left >= 0)
@@ -670,9 +680,9 @@ void PreviewLabel::paintEvent(QPaintEvent *event)
 
         {
             QBrush brush(Qt::Dense4Pattern);
-            brush.setColor("#101010");
+            brush.setColor(QColor(0x10, 0x10, 0x10));
             QPen pen(brush, 1, Qt::DotLine);
-            pen.setColor("#101010");
+            pen.setColor(QColor(0x10, 0x10, 0x10));
             p.setPen(pen);
 
             if (m_left >= 0)
@@ -694,8 +704,10 @@ static inline bool rangeCheck(int target, int pos)
 
 void PreviewLabel::mousePressEvent(QMouseEvent * event)
 {
-    if (!m_borderImage)
-        return QLabel::mouseMoveEvent(event);
+    if (!m_borderImage) {
+        QLabel::mouseMoveEvent(event);
+        return;
+    }
 
     bool bottom = false;
 
@@ -781,8 +793,10 @@ static inline int limitPositive(int i)
 
 void PreviewLabel::mouseMoveEvent(QMouseEvent * event)
 {
-    if (!m_borderImage)
-        return QLabel::mouseMoveEvent(event);
+    if (!m_borderImage) {
+        QLabel::mouseMoveEvent(event);
+        return;
+    }
 
     QPoint p = event->pos();
     bool bottom = false;
@@ -870,7 +884,6 @@ PreviewDialog::PreviewDialog(QWidget *parent) : DragWidget(parent)
     QVBoxLayout *layout = new QVBoxLayout(this);
     QHBoxLayout *horizontalLayout = new QHBoxLayout();
     QHBoxLayout *horizontalLayout2 = new QHBoxLayout();
-    layout->setMargin(0);
     layout->setContentsMargins(2, 2, 2, 16);
     layout->setSpacing(4);
     QToolButton *toolButton = new QToolButton(this);
@@ -878,7 +891,7 @@ PreviewDialog::PreviewDialog(QWidget *parent) : DragWidget(parent)
     toolButton->setIcon(icon);
     toolButton->setToolButtonStyle(Qt::ToolButtonIconOnly);
     toolButton->setFixedSize(icon.availableSizes().value(0) + QSize(4, 4));
-    connect(toolButton, SIGNAL(clicked()), this, SLOT(onTogglePane()));
+    connect(toolButton, &QToolButton::clicked, this, &PreviewDialog::onTogglePane);
 
     QScrollArea *scrollArea = new QScrollArea(this);
     WheelFilter *wheelFilter = new WheelFilter(scrollArea);
@@ -903,7 +916,7 @@ PreviewDialog::PreviewDialog(QWidget *parent) : DragWidget(parent)
 
     wheelFilter->setTarget(this);
 
-    connect(m_slider, SIGNAL(valueChanged(int)), this, SLOT(onSliderMoved(int)));
+    connect(m_slider, &QSlider::valueChanged, this, &PreviewDialog::onSliderMoved);
 
     foreach (QWidget *childWidget, findChildren<QWidget*>()) {
         childWidget->installEventFilter(wheelFilter);
