@@ -84,15 +84,6 @@ bool writeFile(const QString &filePath, const QByteArray &contents)
     return true;
 }
 
-void insertTextAtTopOfEditor(TextEditor::BaseTextEditor *editor, const QByteArray &text)
-{
-    QTC_ASSERT(editor, return);
-    ::Utils::ChangeSet cs;
-    cs.insert(0, QString::fromUtf8(text));
-    QTextCursor textCursor = editor->textCursor();
-    cs.apply(&textCursor);
-}
-
 class ChangeDocumentReloadSetting
 {
 public:
@@ -114,8 +105,7 @@ private:
 class TestDocument
 {
 public:
-    TestDocument(const QByteArray &fileName, CppTools::Tests::TemporaryDir *temporaryDir = 0)
-        : cursorPosition(-1)
+    TestDocument(const QByteArray &fileName, CppTools::Tests::TemporaryDir *temporaryDir = nullptr)
     {
         QTC_ASSERT(!fileName.isEmpty(), return);
         const QResource resource(qrcPath(fileName));
@@ -153,10 +143,10 @@ public:
     { return isCreated() && hasValidCursorPosition(); }
 
     QString filePath;
-    int cursorPosition;
+    int cursorPosition = -1;
 
 private:
-    TestDocument() : cursorPosition(-1) {}
+    TestDocument() = default;
     QSharedPointer<CppTools::Tests::TemporaryDir> m_temporaryDir;
 };
 
@@ -227,7 +217,7 @@ bool OpenEditorAtCursorPosition::waitUntilProjectPartChanged(const QString &newP
 
 bool OpenEditorAtCursorPosition::waitUntil(const std::function<bool ()> &condition, int timeout)
 {
-    QTime time;
+    QElapsedTimer time;
     time.start();
 
     forever {
@@ -278,10 +268,10 @@ public:
     ProjectLoader(const QStringList &projectFiles,
                   const ProjectExplorer::Macros &projectMacros,
                   bool testOnlyForCleanedProjects = false)
-        : m_project(0)
+        : m_project(nullptr)
         , m_projectFiles(projectFiles)
         , m_projectMacros(projectMacros)
-        , m_helper(0, testOnlyForCleanedProjects)
+        , m_helper(nullptr, testOnlyForCleanedProjects)
     {
     }
 
@@ -361,6 +351,20 @@ bool hasItem(TextEditor::ProposalModelPtr model, const QByteArray &text)
     return indexOfItemWithText(model, text) != -1;
 }
 
+int itemsWithText(TextEditor::ProposalModelPtr model, const QByteArray &text)
+{
+    if (!model)
+        return 0;
+
+    int amount = 0;
+    for (int i = 0, size = model->size(); i < size; ++i) {
+        if (model->text(i) == QString::fromUtf8(text))
+            ++amount;
+    }
+
+    return amount;
+}
+
 bool hasItem(TextEditor::ProposalModelPtr model, const QByteArray &text, const QByteArray &detail)
 {
     const int index = indexOfItemWithText(model, text);
@@ -419,7 +423,7 @@ bool MonitorGeneratedUiFile::waitUntilGenerated(int timeout) const
     if (m_isGenerated)
         return true;
 
-    QTime time;
+    QElapsedTimer time;
     time.start();
 
     forever {
@@ -459,10 +463,10 @@ public:
     {
         QTC_ASSERT(writeFile(m_filePath, m_fileContents), return false);
 
-        QTime totalTime;
+        QElapsedTimer totalTime;
         totalTime.start();
 
-        QTime writeFileAgainTime;
+        QElapsedTimer writeFileAgainTime;
         writeFileAgainTime.start();
 
         forever {
@@ -571,6 +575,13 @@ void ClangCodeCompletionTest::testCompleteConstructor()
     QVERIFY(hasItem(t.proposal, "Foo(int, double)"));
 }
 
+void ClangCodeCompletionTest::testCompleteClassAndConstructor()
+{
+    ProjectLessCompletionTest t("classAndConstructorCompletion.cpp");
+
+    QCOMPARE(itemsWithText(t.proposal, "Foo"), 2);
+}
+
 // Explicitly Inserting The Dot
 // ----------------------------
 // Inserting the dot for is important since it will send the editor
@@ -635,6 +646,7 @@ void ClangCodeCompletionTest::testCompleteProjectDependingCodeAfterChangingProje
 
         // Check completion with project configuration 2
         QVERIFY(projectLoader.updateProject({{"PROJECT_CONFIGURATION_2"}}));
+        openEditor.waitUntilBackendIsNotified();
         proposal = completionResults(openEditor.editor());
 
         QVERIFY(!hasItem(proposal, "projectConfiguration1"));
@@ -676,126 +688,6 @@ void ClangCodeCompletionTest::testCompleteProjectDependingCodeInGeneratedUiFile(
     QVERIFY(hasItem(proposal, "setupUi"));
 }
 
-void ClangCodeCompletionTest::testCompleteAfterModifyingIncludedHeaderInOtherEditor()
-{
-    QSKIP("We don't reparse anymore before a code completion so we get wrong completion results.");
-
-    CppTools::Tests::TemporaryDir temporaryDir;
-    const TestDocument sourceDocument("mysource.cpp", &temporaryDir);
-    QVERIFY(sourceDocument.isCreatedAndHasValidCursorPosition());
-    const TestDocument headerDocument("myheader.h", &temporaryDir);
-    QVERIFY(headerDocument.isCreated());
-
-    // Test that declarations from header file are visible in source file
-    OpenEditorAtCursorPosition openSource(sourceDocument);
-    QVERIFY(openSource.succeeded());
-    TextEditor::ProposalModelPtr proposal = completionResults(openSource.editor());
-    QVERIFY(hasItem(proposal, "globalFromHeader"));
-
-    // Open header and insert a new declaration
-    OpenEditorAtCursorPosition openHeader(headerDocument);
-    QVERIFY(openHeader.succeeded());
-    insertTextAtTopOfEditor(openHeader.editor(), "int globalFromHeaderUnsaved;\n");
-
-    // Switch back to source file and check if modified header is reflected in completions.
-    Core::EditorManager::activateEditor(openSource.editor());
-    QCoreApplication::processEvents(); // connections are queued
-    proposal = completionResults(openSource.editor());
-    QVERIFY(hasItem(proposal, "globalFromHeader"));
-    QVERIFY(hasItem(proposal, "globalFromHeaderUnsaved"));
-}
-
-void ClangCodeCompletionTest::testCompleteAfterModifyingIncludedHeaderByRefactoringActions()
-{
-    QSKIP("We don't reparse anymore before a code completion so we get wrong completion results.");
-
-    CppTools::Tests::TemporaryDir temporaryDir;
-    const TestDocument sourceDocument("mysource.cpp", &temporaryDir);
-    QVERIFY(sourceDocument.isCreatedAndHasValidCursorPosition());
-    const TestDocument headerDocument("myheader.h", &temporaryDir);
-    QVERIFY(headerDocument.isCreated());
-
-    // Open header
-    OpenEditorAtCursorPosition openHeader(headerDocument);
-    QVERIFY(openHeader.succeeded());
-
-    // Open source and test that declaration from header file is visible in source file
-    OpenEditorAtCursorPosition openSource(sourceDocument);
-    QVERIFY(openSource.succeeded());
-    TextEditor::ProposalModelPtr proposal = completionResults(openSource.editor());
-    QVERIFY(hasItem(proposal, "globalFromHeader"));
-
-    // Modify header document without switching to its editor.
-    // This simulates e.g. changes from refactoring actions.
-    ::Utils::ChangeSet cs;
-    cs.insert(0, QLatin1String("int globalFromHeaderUnsaved;\n"));
-    QTextCursor textCursor = openHeader.editor()->textCursor();
-    cs.apply(&textCursor);
-
-    // Check whether modified header is reflected in the completions.
-    proposal = completionResults(openSource.editor());
-    QVERIFY(hasItem(proposal, "globalFromHeader"));
-    QVERIFY(hasItem(proposal, "globalFromHeaderUnsaved"));
-}
-
-void ClangCodeCompletionTest::testCompleteAfterChangingIncludedAndOpenHeaderExternally()
-{
-    QSKIP("The file system watcher is doing it in backend process but we wait not long enough");
-
-    ChangeDocumentReloadSetting reloadSettingsChanger(Core::IDocument::ReloadUnmodified);
-
-    CppTools::Tests::TemporaryDir temporaryDir;
-    const TestDocument sourceDocument("mysource.cpp", &temporaryDir);
-    QVERIFY(sourceDocument.isCreatedAndHasValidCursorPosition());
-    const TestDocument headerDocument("myheader.h", &temporaryDir);
-    QVERIFY(headerDocument.isCreated());
-
-    // Open header
-    OpenEditorAtCursorPosition openHeader(headerDocument);
-    QVERIFY(openHeader.succeeded());
-
-    // Open source and test completions
-    OpenEditorAtCursorPosition openSource(sourceDocument);
-    QVERIFY(openSource.succeeded());
-    TextEditor::ProposalModelPtr proposal = completionResults(openSource.editor());
-    QVERIFY(hasItem(proposal, "globalFromHeader"));
-
-    // Simulate external modification and wait for reload
-    WriteFileAndWaitForReloadedDocument waitForReloadedDocument(
-                headerDocument.filePath,
-                "int globalFromHeaderReloaded;\n",
-                openHeader.editor()->document());
-    QVERIFY(waitForReloadedDocument.wait());
-
-    // Retrigger completion and check if its updated
-    proposal = completionResults(openSource.editor());
-    QVERIFY(hasItem(proposal, "globalFromHeaderReloaded"));
-}
-
-void ClangCodeCompletionTest::testCompleteAfterChangingIncludedAndNotOpenHeaderExternally()
-{
-    QSKIP("The file system watcher is doing it in backend process but we wait not long enough");
-
-    CppTools::Tests::TemporaryDir temporaryDir;
-    const TestDocument sourceDocument("mysource.cpp", &temporaryDir);
-    QVERIFY(sourceDocument.isCreatedAndHasValidCursorPosition());
-    const TestDocument headerDocument("myheader.h", &temporaryDir);
-    QVERIFY(headerDocument.isCreated());
-
-    // Open source and test completions
-    OpenEditorAtCursorPosition openSource(sourceDocument);
-    QVERIFY(openSource.succeeded());
-    TextEditor::ProposalModelPtr proposal = completionResults(openSource.editor());
-    QVERIFY(hasItem(proposal, "globalFromHeader"));
-
-    // Simulate external modification, e.g version control checkout
-    QVERIFY(writeFile(headerDocument.filePath, "int globalFromHeaderReloaded;\n"));
-
-    // Retrigger completion and check if its updated
-    proposal = completionResults(openSource.editor());
-    QVERIFY(hasItem(proposal, "globalFromHeaderReloaded"));
-}
-
 } // namespace Tests
 } // namespace Internal
 } // namespace ClangCodeModel
