@@ -111,7 +111,7 @@ CMakeBuildSettingsWidget::CMakeBuildSettingsWidget(CMakeBuildConfiguration *bc)
     connect(buildDirChooser, &Utils::PathChooser::rawPathChanged, this,
             [this](const QString &path) {
                 m_configModel->flush(); // clear out config cache...
-                m_buildConfiguration->setBuildDirectory(Utils::FileName::fromString(path));
+                m_buildConfiguration->setBuildDirectory(Utils::FilePath::fromString(path));
             });
 
     int row = 0;
@@ -385,7 +385,7 @@ void CMakeBuildSettingsWidget::updateAdvancedCheckBox()
 void CMakeBuildSettingsWidget::updateFromKit()
 {
     const ProjectExplorer::Kit *k = m_buildConfiguration->target()->kit();
-    const CMakeConfig config = CMakeConfigurationKitInformation::configuration(k);
+    const CMakeConfig config = CMakeConfigurationKitAspect::configuration(k);
 
     QHash<QString, QString> configHash;
     for (const CMakeConfigItem &i : config)
