@@ -37,6 +37,7 @@
 #include <projectexplorer/deployconfiguration.h>
 #include <projectexplorer/gnumakeparser.h>
 #include <projectexplorer/kitinformation.h>
+#include <projectexplorer/processparameters.h>
 #include <projectexplorer/projectexplorerconstants.h>
 #include <projectexplorer/projectexplorer.h>
 #include <projectexplorer/target.h>
@@ -90,14 +91,8 @@ CMakeBuildStep::CMakeBuildStep(BuildStepList *bsl) :
     }
 
     // Set a good default build target:
-    if (m_buildTarget.isEmpty()) {
-        if (bsl->id() == ProjectExplorer::Constants::BUILDSTEPS_CLEAN)
-            setBuildTarget(cleanTarget());
-        else if (bsl->id() == ProjectExplorer::Constants::BUILDSTEPS_DEPLOY)
-            setBuildTarget(installTarget());
-        else
-            setBuildTarget(allTarget());
-    }
+    if (m_buildTarget.isEmpty())
+        setBuildTarget(defaultBuildTarget());
 
     connect(target(), &Target::kitChanged, this, &CMakeBuildStep::cmakeCommandChanged);
     connect(project(), &Project::parsingFinished,
@@ -114,12 +109,14 @@ CMakeRunConfiguration *CMakeBuildStep::targetsActiveRunConfiguration() const
     return qobject_cast<CMakeRunConfiguration *>(target()->activeRunConfiguration());
 }
 
-void CMakeBuildStep::handleBuildTargetChanges()
+void CMakeBuildStep::handleBuildTargetChanges(bool success)
 {
-    if (isCurrentExecutableTarget(m_buildTarget))
-        return; // Do not change just because a different set of build targets is there...
-    if (!static_cast<CMakeProject *>(project())->buildTargetTitles().contains(m_buildTarget))
-        setBuildTarget(allTarget());
+    if (!success)
+        return; // Do not change when parsing failed.
+    if (!isCurrentExecutableTarget(m_buildTarget)
+        && !static_cast<CMakeProject *>(project())->buildTargetTitles().contains(m_buildTarget)) {
+        setBuildTarget(defaultBuildTarget());
+    }
     emit buildTargetsChanged();
 }
 
@@ -145,7 +142,7 @@ bool CMakeBuildStep::fromMap(const QVariantMap &map)
 }
 
 
-bool CMakeBuildStep::init(QList<const BuildStep *> &earlierSteps)
+bool CMakeBuildStep::init()
 {
     bool canInit = true;
     CMakeBuildConfiguration *bc = cmakeBuildConfiguration();
@@ -157,16 +154,16 @@ bool CMakeBuildStep::init(QList<const BuildStep *> &earlierSteps)
         emit addTask(Task(Task::Error,
                           QCoreApplication::translate("CMakeProjectManager::CMakeBuildStep",
                                                       "The build configuration is currently disabled."),
-                          Utils::FileName(), -1, ProjectExplorer::Constants::TASK_CATEGORY_BUILDSYSTEM));
+                          Utils::FilePath(), -1, ProjectExplorer::Constants::TASK_CATEGORY_BUILDSYSTEM));
         canInit = false;
     }
 
-    CMakeTool *tool = CMakeKitInformation::cmakeTool(target()->kit());
+    CMakeTool *tool = CMakeKitAspect::cmakeTool(target()->kit());
     if (!tool || !tool->isValid()) {
         emit addTask(Task(Task::Error,
                           tr("A CMake tool must be set up for building. "
                              "Configure a CMake tool in the kit options."),
-                          Utils::FileName(), -1,
+                          Utils::FilePath(), -1,
                           ProjectExplorer::Constants::TASK_CATEGORY_BUILDSYSTEM));
         canInit = false;
     }
@@ -178,7 +175,7 @@ bool CMakeBuildStep::init(QList<const BuildStep *> &earlierSteps)
                                     "You asked to build the current Run Configuration's build target only, "
                                     "but it is not associated with a build target. "
                                     "Update the Make Step in your build settings."),
-                        Utils::FileName(), -1,
+                        Utils::FilePath(), -1,
                         ProjectExplorer::Constants::TASK_CATEGORY_BUILDSYSTEM));
         canInit = false;
     }
@@ -189,23 +186,19 @@ bool CMakeBuildStep::init(QList<const BuildStep *> &earlierSteps)
     }
 
     // Warn if doing out-of-source builds with a CMakeCache.txt is the source directory
-    const Utils::FileName projectDirectory = bc->target()->project()->projectDirectory();
+    const Utils::FilePath projectDirectory = bc->target()->project()->projectDirectory();
     if (bc->buildDirectory() != projectDirectory) {
-        Utils::FileName cmc = projectDirectory;
-        cmc.appendPath("CMakeCache.txt");
-        if (cmc.exists()) {
+        if (projectDirectory.pathAppended("CMakeCache.txt").exists()) {
             emit addTask(Task(Task::Warning,
                               tr("There is a CMakeCache.txt file in \"%1\", which suggest an "
                                  "in-source build was done before. You are now building in \"%2\", "
                                  "and the CMakeCache.txt file might confuse CMake.")
                               .arg(projectDirectory.toUserOutput(), bc->buildDirectory().toUserOutput()),
-                              Utils::FileName(), -1,
+                              Utils::FilePath(), -1,
                               ProjectExplorer::Constants::TASK_CATEGORY_BUILDSYSTEM));
         }
     }
 
-    QString arguments = allArguments(rc);
-
     setIgnoreReturnValue(m_buildTarget == CMakeBuildStep::cleanTarget());
 
     ProcessParameters *pp = processParameters();
@@ -215,9 +208,8 @@ bool CMakeBuildStep::init(QList<const BuildStep *> &earlierSteps)
     if (!env.value("NINJA_STATUS").startsWith(m_ninjaProgressString))
         env.set("NINJA_STATUS", m_ninjaProgressString + "%o/sec] ");
     pp->setEnvironment(env);
-    pp->setWorkingDirectory(bc->buildDirectory().toString());
-    pp->setCommand(cmakeCommand());
-    pp->setArguments(arguments);
+    pp->setWorkingDirectory(bc->buildDirectory());
+    pp->setCommandLine(cmakeCommand(rc));
     pp->resolveAll();
 
     setOutputParser(new CMakeParser);
@@ -227,49 +219,50 @@ bool CMakeBuildStep::init(QList<const BuildStep *> &earlierSteps)
         appendOutputParser(parser);
     outputParser()->setWorkingDirectory(pp->effectiveWorkingDirectory());
 
-    return AbstractProcessStep::init(earlierSteps);
+    return AbstractProcessStep::init();
 }
 
-void CMakeBuildStep::run(QFutureInterface<bool> &fi)
+void CMakeBuildStep::doRun()
 {
     // Make sure CMake state was written to disk before trying to build:
     CMakeBuildConfiguration *bc = cmakeBuildConfiguration();
     QTC_ASSERT(bc, return);
 
-    bool mustDelay = false;
+    m_waiting = false;
     auto p = static_cast<CMakeProject *>(bc->project());
     if (p->persistCMakeState()) {
         emit addOutput(tr("Persisting CMake state..."), BuildStep::OutputFormat::NormalMessage);
-        mustDelay = true;
+        m_waiting = true;
     } else if (p->mustUpdateCMakeStateBeforeBuild()) {
         emit addOutput(tr("Running CMake in preparation to build..."), BuildStep::OutputFormat::NormalMessage);
-        mustDelay = true;
-    } else {
-        mustDelay = false;
+        m_waiting = true;
     }
 
-    if (mustDelay) {
+    if (m_waiting) {
         m_runTrigger = connect(project(), &Project::parsingFinished,
-                               this, [this, &fi](bool success) { handleProjectWasParsed(fi, success); });
+                               this, [this](bool success) { handleProjectWasParsed(success); });
     } else {
-        runImpl(fi);
+        runImpl();
     }
 }
 
-void CMakeBuildStep::runImpl(QFutureInterface<bool> &fi)
+void CMakeBuildStep::runImpl()
 {
     // Do the actual build:
-    AbstractProcessStep::run(fi);
+    AbstractProcessStep::doRun();
 }
 
-void CMakeBuildStep::handleProjectWasParsed(QFutureInterface<bool> &fi, bool success)
+void CMakeBuildStep::handleProjectWasParsed(bool success)
 {
+    m_waiting = false;
     disconnect(m_runTrigger);
-    if (success) {
-        runImpl(fi);
+    if (isCanceled()) {
+        emit finished(false);
+    } else if (success) {
+        runImpl();
     } else {
         AbstractProcessStep::stdError(tr("Project did not parse successfully, cannot build."));
-        reportRunResult(fi, false);
+        emit finished(false);
     }
 }
 
@@ -278,9 +271,15 @@ BuildStepConfigWidget *CMakeBuildStep::createConfigWidget()
     return new CMakeBuildStepConfigWidget(this);
 }
 
-bool CMakeBuildStep::immutable() const
+QString CMakeBuildStep::defaultBuildTarget() const
 {
-    return false;
+    const ProjectConfiguration *const pc = qobject_cast<ProjectConfiguration *>(parent());
+    const Core::Id parentId = pc ? pc->id() : Core::Id();
+    if (parentId == ProjectExplorer::Constants::BUILDSTEPS_CLEAN)
+        return cleanTarget();
+    if (parentId == ProjectExplorer::Constants::BUILDSTEPS_DEPLOY)
+        return installTarget();
+    return allTarget();
 }
 
 void CMakeBuildStep::stdOutput(const QString &line)
@@ -290,7 +289,7 @@ void CMakeBuildStep::stdOutput(const QString &line)
         bool ok = false;
         int percent = m_percentProgress.cap(1).toInt(&ok);
         if (ok)
-            futureInterface()->setProgressValue(percent);
+            emit progress(percent, QString());
         return;
     } else if (m_ninjaProgress.indexIn(line) != -1) {
         AbstractProcessStep::stdOutput(line);
@@ -301,7 +300,7 @@ void CMakeBuildStep::stdOutput(const QString &line)
             int all = m_ninjaProgress.cap(2).toInt(&ok);
             if (ok && all != 0) {
                 const int percent = static_cast<int>(100.0 * done/all);
-                futureInterface()->setProgressValue(percent);
+                emit progress(percent, QString());
             }
         }
         return;
@@ -345,39 +344,37 @@ void CMakeBuildStep::setToolArguments(const QString &list)
     m_toolArguments = list;
 }
 
-QString CMakeBuildStep::allArguments(const CMakeRunConfiguration *rc) const
+Utils::CommandLine CMakeBuildStep::cmakeCommand(CMakeRunConfiguration *rc) const
 {
-    QString arguments;
+    CMakeTool *tool = CMakeKitAspect::cmakeTool(target()->kit());
 
-    Utils::QtcProcess::addArg(&arguments, "--build");
-    Utils::QtcProcess::addArg(&arguments, ".");
+    Utils::CommandLine cmd(tool ? tool->cmakeExecutable() : Utils::FilePath(), {});
+    cmd.addArgs({"--build", "."});
 
     QString target;
 
     if (isCurrentExecutableTarget(m_buildTarget)) {
-        if (rc)
-            target = rc->buildKey().section('\n', 0, 0);
-        else
+        if (rc) {
+            target = rc->buildKey();
+            const int pos = target.indexOf("///::///");
+            if (pos >= 0) {
+                target = target.mid(pos + 8);
+            }
+        } else {
             target = "<i>&lt;" + tr(ADD_RUNCONFIGURATION_TEXT) + "&gt;</i>";
+        }
     } else {
         target = m_buildTarget;
     }
 
-    Utils::QtcProcess::addArg(&arguments, "--target");
-    Utils::QtcProcess::addArg(&arguments, target);
+    cmd.addArgs({"--target", target});
 
     if (!m_toolArguments.isEmpty()) {
-        Utils::QtcProcess::addArg(&arguments, "--");
-        arguments += ' ' + m_toolArguments;
+        cmd.addArg("--");
+        cmd.addArgs(m_toolArguments);
     }
 
-    return arguments;
-}
-
-QString CMakeBuildStep::cmakeCommand() const
-{
-    CMakeTool *tool = CMakeKitInformation::cmakeTool(target()->kit());
-    return tool ? tool->cmakeExecutable().toString() : QString();
+    return cmd;
 }
 
 QString CMakeBuildStep::cleanTarget()
@@ -410,10 +407,13 @@ QStringList CMakeBuildStep::specialTargets()
 //
 
 CMakeBuildStepConfigWidget::CMakeBuildStepConfigWidget(CMakeBuildStep *buildStep) :
+    BuildStepConfigWidget(buildStep),
     m_buildStep(buildStep),
     m_toolArguments(new QLineEdit),
     m_buildTargetsList(new QListWidget)
 {
+    setDisplayName(tr("Build", "CMakeProjectManager::CMakeBuildStepConfigWidget display name."));
+
     auto fl = new QFormLayout(this);
     fl->setMargin(0);
     fl->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
@@ -469,11 +469,6 @@ void CMakeBuildStepConfigWidget::itemChanged(QListWidgetItem *item)
     updateDetails();
 }
 
-QString CMakeBuildStepConfigWidget::displayName() const
-{
-    return tr("Build", "CMakeProjectManager::CMakeBuildStepConfigWidget display name.");
-}
-
 void CMakeBuildStepConfigWidget::buildTargetsChanged()
 {
     {
@@ -527,25 +522,17 @@ void CMakeBuildStepConfigWidget::updateDetails()
 {
     BuildConfiguration *bc = m_buildStep->buildConfiguration();
     if (!bc) {
-        m_summaryText = tr("<b>No build configuration found on this kit.</b>");
-        emit updateSummary();
+        setSummaryText(tr("<b>No build configuration found on this kit.</b>"));
         return;
     }
 
     ProcessParameters param;
     param.setMacroExpander(bc->macroExpander());
     param.setEnvironment(bc->environment());
-    param.setWorkingDirectory(bc->buildDirectory().toString());
-    param.setCommand(m_buildStep->cmakeCommand());
-    param.setArguments(m_buildStep->allArguments(0));
-    m_summaryText = param.summary(displayName());
+    param.setWorkingDirectory(bc->buildDirectory());
+    param.setCommandLine(m_buildStep->cmakeCommand(nullptr));
 
-    emit updateSummary();
-}
-
-QString CMakeBuildStepConfigWidget::summaryText() const
-{
-    return m_summaryText;
+    setSummaryText(param.summary(displayName()));
 }
 
 //
@@ -562,12 +549,11 @@ CMakeBuildStepFactory::CMakeBuildStepFactory()
 void CMakeBuildStep::processStarted()
 {
     m_useNinja = false;
-    futureInterface()->setProgressRange(0, 100);
     AbstractProcessStep::processStarted();
 }
 
 void CMakeBuildStep::processFinished(int exitCode, QProcess::ExitStatus status)
 {
     AbstractProcessStep::processFinished(exitCode, status);
-    futureInterface()->setProgressValue(100);
+    emit progress(100, QString());
 }
