@@ -23,17 +23,18 @@
 **
 ****************************************************************************/
 
-#ifndef EXTERNALTOOLCONFIG_H
-#define EXTERNALTOOLCONFIG_H
+#pragma once
 
-#include "coreplugin/externaltool.h"
+#include "../externaltool.h"
 
-#include <QWidget>
 #include <QAbstractItemModel>
 #include <QDialog>
+#include <QWidget>
 
 QT_FORWARD_DECLARE_CLASS(QPlainTextEdit)
 
+namespace Utils { class EnvironmentItem; }
+
 namespace Core {
 namespace Internal {
 
@@ -45,23 +46,23 @@ class ExternalToolModel : public QAbstractItemModel
 
 public:
     explicit ExternalToolModel(QObject *parent);
-    ~ExternalToolModel();
+    ~ExternalToolModel() override;
 
-    int columnCount(const QModelIndex &parent = QModelIndex()) const;
-    QVariant data(const QModelIndex &modelIndex, int role = Qt::DisplayRole) const;
-    QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const;
-    QModelIndex parent(const QModelIndex &child) const;
-    int rowCount(const QModelIndex &parent = QModelIndex()) const;
-    Qt::ItemFlags flags(const QModelIndex &modelIndex) const;
-    bool setData(const QModelIndex &modelIndex, const QVariant &value, int role = Qt::EditRole);
+    int columnCount(const QModelIndex &parent = QModelIndex()) const override;
+    QVariant data(const QModelIndex &modelIndex, int role = Qt::DisplayRole) const override;
+    QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const override;
+    QModelIndex parent(const QModelIndex &child) const override;
+    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
+    Qt::ItemFlags flags(const QModelIndex &modelIndex) const override;
+    bool setData(const QModelIndex &modelIndex, const QVariant &value, int role = Qt::EditRole) override;
 
-    QMimeData *mimeData(const QModelIndexList &indexes) const;
+    QMimeData *mimeData(const QModelIndexList &indexes) const override;
     bool dropMimeData(const QMimeData *data,
                       Qt::DropAction action,
                       int row,
                       int column,
-                      const QModelIndex &parent);
-    QStringList mimeTypes() const;
+                      const QModelIndex &parent) override;
+    QStringList mimeTypes() const override;
 
     void setTools(const QMap<QString, QList<ExternalTool *> > &tools);
     QMap<QString, QList<ExternalTool *> > tools() const;
@@ -72,7 +73,7 @@ public:
     QModelIndex addCategory();
     QModelIndex addTool(const QModelIndex &atIndex);
     void removeTool(const QModelIndex &modelIndex);
-    Qt::DropActions supportedDropActions() const;
+    Qt::DropActions supportedDropActions() const override;
 private:
     QVariant data(ExternalTool *tool, int role = Qt::DisplayRole) const;
     QVariant data(const QString &category, int role = Qt::DisplayRole) const;
@@ -80,25 +81,13 @@ private:
     QMap<QString, QList<ExternalTool *> > m_tools;
 };
 
-class EnvironmentChangesDialog : public QDialog
-{
-    Q_OBJECT
-public:
-    explicit EnvironmentChangesDialog(QWidget *parent = 0);
-
-    QStringList changes() const;
-    void setChanges(const QStringList &changes);
-private:
-    QPlainTextEdit *m_editor;
-};
-
 class ExternalToolConfig : public QWidget
 {
     Q_OBJECT
 
 public:
-    explicit ExternalToolConfig(QWidget *parent = 0);
-    ~ExternalToolConfig();
+    explicit ExternalToolConfig(QWidget *parent = nullptr);
+    ~ExternalToolConfig() override;
 
     void setTools(const QMap<QString, QList<ExternalTool *> > &tools);
     QMap<QString, QList<ExternalTool *> > tools() const;
@@ -119,12 +108,9 @@ private:
     void updateEnvironmentLabel();
 
     Ui::ExternalToolConfig *ui;
-    QStringList m_environment;
+    QList<Utils::EnvironmentItem> m_environment;
     ExternalToolModel *m_model;
 };
 
 } // Internal
 } // Core
-
-
-#endif // EXTERNALTOOLCONFIG_H
