@@ -26,6 +26,7 @@
 #include "cppcodestylesettingspage.h"
 
 #include "cppcodestylepreferences.h"
+#include "cppcodestylesnippets.h"
 #include "cpppointerdeclarationformatter.h"
 #include "cppqtstyleindenter.h"
 #include "cpptoolsconstants.h"
@@ -35,9 +36,10 @@
 #include <coreplugin/icore.h>
 #include <cppeditor/cppeditorconstants.h>
 #include <texteditor/codestyleeditor.h>
+#include <texteditor/icodestylepreferencesfactory.h>
 #include <texteditor/textdocument.h>
 #include <texteditor/displaysettings.h>
-#include <texteditor/snippets/isnippetprovider.h>
+#include <texteditor/snippets/snippetprovider.h>
 #include <texteditor/texteditorsettings.h>
 
 #include <cplusplus/Overview.h>
@@ -48,170 +50,6 @@
 #include <QTextBlock>
 #include <QTextStream>
 
-static const char *defaultCodeStyleSnippets[] = {
-    "#include <math.h>\n"
-    "\n"
-    "class Complex\n"
-    "    {\n"
-    "public:\n"
-    "    Complex(double re, double im)\n"
-    "        : _re(re), _im(im)\n"
-    "        {}\n"
-    "    double modulus() const\n"
-    "        {\n"
-    "        return sqrt(_re * _re + _im * _im);\n"
-    "        }\n"
-    "private:\n"
-    "    double _re;\n"
-    "    double _im;\n"
-    "    };\n"
-    "\n"
-    "void bar(int i)\n"
-    "    {\n"
-    "    static int counter = 0;\n"
-    "    counter += i;\n"
-    "    }\n"
-    "\n"
-    "namespace Foo\n"
-    "    {\n"
-    "    namespace Bar\n"
-    "        {\n"
-    "        void foo(int a, int b)\n"
-    "            {\n"
-    "            for (int i = 0; i < a; i++)\n"
-    "                {\n"
-    "                if (i < b)\n"
-    "                    bar(i);\n"
-    "                else\n"
-    "                    {\n"
-    "                    bar(i);\n"
-    "                    bar(b);\n"
-    "                    }\n"
-    "                }\n"
-    "            }\n"
-    "        } // namespace Bar\n"
-    "    } // namespace Foo\n"
-    ,
-    "#include <math.h>\n"
-    "\n"
-    "class Complex\n"
-    "    {\n"
-    "public:\n"
-    "    Complex(double re, double im)\n"
-    "        : _re(re), _im(im)\n"
-    "        {}\n"
-    "    double modulus() const\n"
-    "        {\n"
-    "        return sqrt(_re * _re + _im * _im);\n"
-    "        }\n"
-    "private:\n"
-    "    double _re;\n"
-    "    double _im;\n"
-    "    };\n"
-    "\n"
-    "void bar(int i)\n"
-    "    {\n"
-    "    static int counter = 0;\n"
-    "    counter += i;\n"
-    "    }\n"
-    "\n"
-    "namespace Foo\n"
-    "    {\n"
-    "    namespace Bar\n"
-    "        {\n"
-    "        void foo(int a, int b)\n"
-    "            {\n"
-    "            for (int i = 0; i < a; i++)\n"
-    "                {\n"
-    "                if (i < b)\n"
-    "                    bar(i);\n"
-    "                else\n"
-    "                    {\n"
-    "                    bar(i);\n"
-    "                    bar(b);\n"
-    "                    }\n"
-    "                }\n"
-    "            }\n"
-    "        } // namespace Bar\n"
-    "    } // namespace Foo\n"
-    ,
-    "namespace Foo\n"
-    "{\n"
-    "namespace Bar\n"
-    "{\n"
-    "class FooBar\n"
-    "    {\n"
-    "public:\n"
-    "    FooBar(int a)\n"
-    "        : _a(a)\n"
-    "        {}\n"
-    "    int calculate() const\n"
-    "        {\n"
-    "        if (a > 10)\n"
-    "            {\n"
-    "            int b = 2 * a;\n"
-    "            return a * b;\n"
-    "            }\n"
-    "        return -a;\n"
-    "        }\n"
-    "private:\n"
-    "    int _a;\n"
-    "    };\n"
-    "}\n"
-    "}\n"
-    ,
-    "#include \"bar.h\"\n"
-    "\n"
-    "int foo(int a)\n"
-    "    {\n"
-    "    switch (a)\n"
-    "        {\n"
-    "        case 1:\n"
-    "            bar(1);\n"
-    "            break;\n"
-    "        case 2:\n"
-    "            {\n"
-    "            bar(2);\n"
-    "            break;\n"
-    "            }\n"
-    "        case 3:\n"
-    "        default:\n"
-    "            bar(3);\n"
-    "            break;\n"
-    "        }\n"
-    "    return 0;\n"
-    "    }\n"
-    ,
-    "void foo() {\n"
-    "    if (a &&\n"
-    "        b)\n"
-    "        c;\n"
-    "\n"
-    "    while (a ||\n"
-    "           b)\n"
-    "        break;\n"
-    "    a = b +\n"
-    "        c;\n"
-    "    myInstance.longMemberName +=\n"
-    "            foo;\n"
-    "    myInstance.longMemberName += bar +\n"
-    "                                 foo;\n"
-    "}\n"
-    ,
-    "int *foo(const Bar &b1, Bar &&b2, int*, int *&rpi)\n"
-    "{\n"
-    "    int*pi = 0;\n"
-    "    int*const*const cpcpi = &pi;\n"
-    "    int*const*pcpi = &pi;\n"
-    "    int**const cppi = &pi;\n"
-    "\n"
-    "    void (*foo)(char *s) = 0;\n"
-    "    int (*bar)[] = 0;\n"
-    "\n"
-    "    return pi;\n"
-    "}\n"
-};
-
 using namespace TextEditor;
 
 namespace CppTools {
@@ -223,7 +61,7 @@ static void applyRefactorings(QTextDocument *textDocument, TextEditorWidget *edi
 {
     // Preprocess source
     Environment env;
-    Preprocessor preprocess(0, &env);
+    Preprocessor preprocess(nullptr, &env);
     const QByteArray preprocessedSource
         = preprocess.run(QLatin1String("<no-file>"), textDocument->toPlainText());
 
@@ -237,7 +75,7 @@ static void applyRefactorings(QTextDocument *textDocument, TextEditorWidget *edi
     // Run the formatter
     Overview overview;
     overview.showReturnTypes = true;
-    overview.starBindFlags = Overview::StarBindFlags(0);
+    overview.starBindFlags = Overview::StarBindFlags(nullptr);
 
     if (settings.bindStarToIdentifier)
         overview.starBindFlags |= Overview::BindToIdentifier;
@@ -260,9 +98,7 @@ static void applyRefactorings(QTextDocument *textDocument, TextEditorWidget *edi
 
 CppCodeStylePreferencesWidget::CppCodeStylePreferencesWidget(QWidget *parent)
     : QWidget(parent),
-      m_preferences(0),
-      m_ui(new Ui::CppCodeStyleSettingsPage),
-      m_blockUpdates(false)
+      m_ui(new Ui::CppCodeStyleSettingsPage)
 {
     m_ui->setupUi(this);
     m_ui->categoryTab->setProperty("_q_custom_style_disabled", true);
@@ -271,11 +107,11 @@ CppCodeStylePreferencesWidget::CppCodeStylePreferencesWidget(QWidget *parent)
                << m_ui->previewTextEditBraces << m_ui->previewTextEditSwitch
                << m_ui->previewTextEditPadding << m_ui->previewTextEditPointerReferences;
     for (int i = 0; i < m_previews.size(); ++i)
-        m_previews[i]->setPlainText(QLatin1String(defaultCodeStyleSnippets[i]));
+        m_previews[i]->setPlainText(QLatin1String(Constants::DEFAULT_CODE_STYLE_SNIPPETS[i]));
 
     decorateEditors(TextEditorSettings::fontSettings());
-    connect(TextEditorSettings::instance(), SIGNAL(fontSettingsChanged(TextEditor::FontSettings)),
-       this, SLOT(decorateEditors(TextEditor::FontSettings)));
+    connect(TextEditorSettings::instance(), &TextEditorSettings::fontSettingsChanged,
+            this, &CppCodeStylePreferencesWidget::decorateEditors);
 
     setVisualizeWhitespace(true);
 
@@ -321,6 +157,8 @@ CppCodeStylePreferencesWidget::CppCodeStylePreferencesWidget(QWidget *parent)
             this, &CppCodeStylePreferencesWidget::slotCodeStyleSettingsChanged);
     connect(m_ui->bindStarToRightSpecifier, &QCheckBox::toggled,
             this, &CppCodeStylePreferencesWidget::slotCodeStyleSettingsChanged);
+    connect(m_ui->preferGetterNamesWithoutGet, &QCheckBox::toggled,
+            this, &CppCodeStylePreferencesWidget::slotCodeStyleSettingsChanged);
 
     m_ui->categoryTab->setCurrentIndex(0);
 
@@ -339,10 +177,15 @@ void CppCodeStylePreferencesWidget::setCodeStyle(CppTools::CppCodeStylePreferenc
 
     connect(m_preferences, &CppCodeStylePreferences::currentTabSettingsChanged,
             this, &CppCodeStylePreferencesWidget::setTabSettings);
-    connect(m_preferences, SIGNAL(currentCodeStyleSettingsChanged(CppTools::CppCodeStyleSettings)),
-            this, SLOT(setCodeStyleSettings(CppTools::CppCodeStyleSettings)));
-    connect(m_preferences, SIGNAL(currentPreferencesChanged(TextEditor::ICodeStylePreferences*)),
-            this, SLOT(slotCurrentPreferencesChanged(TextEditor::ICodeStylePreferences*)));
+    connect(m_preferences, &CppCodeStylePreferences::currentCodeStyleSettingsChanged,
+            this, [this](const CppTools::CppCodeStyleSettings &codeStyleSettings) {
+        setCodeStyleSettings(codeStyleSettings);
+    });
+
+    connect(m_preferences, &ICodeStylePreferences::currentPreferencesChanged,
+            this, [this](TextEditor::ICodeStylePreferences *currentPreferences) {
+        slotCurrentPreferencesChanged(currentPreferences);
+    });
 
     setTabSettings(m_preferences->tabSettings());
     setCodeStyleSettings(m_preferences->codeStyleSettings(), false);
@@ -375,6 +218,7 @@ CppCodeStyleSettings CppCodeStylePreferencesWidget::cppCodeStyleSettings() const
     set.bindStarToRightSpecifier = m_ui->bindStarToRightSpecifier->isChecked();
     set.extraPaddingForConditionsIfConfusingAlign = m_ui->extraPaddingConditions->isChecked();
     set.alignAssignments = m_ui->alignAssignments->isChecked();
+    set.preferGetterNameWithoutGetPrefix = m_ui->preferGetterNamesWithoutGet->isChecked();
 
     return set;
 }
@@ -408,6 +252,7 @@ void CppCodeStylePreferencesWidget::setCodeStyleSettings(const CppCodeStyleSetti
     m_ui->bindStarToRightSpecifier->setChecked(s.bindStarToRightSpecifier);
     m_ui->extraPaddingConditions->setChecked(s.extraPaddingForConditionsIfConfusingAlign);
     m_ui->alignAssignments->setChecked(s.alignAssignments);
+    m_ui->preferGetterNamesWithoutGet->setChecked(s.preferGetterNameWithoutGetPrefix);
     m_blockUpdates = wasBlocked;
     if (preview)
         updatePreview();
@@ -432,7 +277,7 @@ void CppCodeStylePreferencesWidget::slotCodeStyleSettingsChanged()
         return;
 
     if (m_preferences) {
-        CppCodeStylePreferences *current = qobject_cast<CppCodeStylePreferences *>(m_preferences->currentPreferences());
+        auto current = qobject_cast<CppCodeStylePreferences *>(m_preferences->currentPreferences());
         if (current)
             current->setCodeStyleSettings(cppCodeStyleSettings());
     }
@@ -446,7 +291,7 @@ void CppCodeStylePreferencesWidget::slotTabSettingsChanged(const TabSettings &se
         return;
 
     if (m_preferences) {
-        CppCodeStylePreferences *current = qobject_cast<CppCodeStylePreferences *>(m_preferences->currentPreferences());
+        auto current = qobject_cast<CppCodeStylePreferences *>(m_preferences->currentPreferences());
         if (current)
             current->setTabSettings(settings);
     }
@@ -473,7 +318,7 @@ void CppCodeStylePreferencesWidget::updatePreview()
         QTextCursor tc = preview->textCursor();
         tc.beginEditBlock();
         while (block.isValid()) {
-            preview->textDocument()->indenter()->indentBlock(doc, block, QChar::Null, ts);
+            preview->textDocument()->indenter()->indentBlock(block, QChar::Null, ts);
 
             block = block.next();
         }
@@ -484,15 +329,9 @@ void CppCodeStylePreferencesWidget::updatePreview()
 
 void CppCodeStylePreferencesWidget::decorateEditors(const FontSettings &fontSettings)
 {
-    const ISnippetProvider *provider = ExtensionSystem::PluginManager::getObject<ISnippetProvider>(
-        [](ISnippetProvider *current) {
-            return current->groupId() == QLatin1String(CppEditor::Constants::CPP_SNIPPETS_GROUP_ID);
-        });
-
     foreach (SnippetEditorWidget *editor, m_previews) {
         editor->textDocument()->setFontSettings(fontSettings);
-        if (provider)
-            provider->decorateEditor(editor);
+        SnippetProvider::decorateEditor(editor, CppEditor::Constants::CPP_SNIPPETS_GROUP_ID);
     }
 }
 
@@ -508,30 +347,30 @@ void CppCodeStylePreferencesWidget::setVisualizeWhitespace(bool on)
 
 // ------------------ CppCodeStyleSettingsPage
 
-CppCodeStyleSettingsPage::CppCodeStyleSettingsPage(QWidget *parent) :
-    Core::IOptionsPage(parent),
-    m_pageCppCodeStylePreferences(0)
+CppCodeStyleSettingsPage::CppCodeStyleSettingsPage(QWidget *parent)
+    : Core::IOptionsPage(parent)
 {
     setId(Constants::CPP_CODE_STYLE_SETTINGS_ID);
     setDisplayName(QCoreApplication::translate("CppTools", Constants::CPP_CODE_STYLE_SETTINGS_NAME));
     setCategory(Constants::CPP_SETTINGS_CATEGORY);
-    setDisplayCategory(QCoreApplication::translate("CppTools", Constants::CPP_SETTINGS_TR_CATEGORY));
-    setCategoryIcon(QLatin1String(Constants::SETTINGS_CATEGORY_CPP_ICON));
 }
 
 QWidget *CppCodeStyleSettingsPage::widget()
 {
     if (!m_widget) {
-        CppCodeStylePreferences *originalCodeStylePreferences
-                = CppToolsSettings::instance()->cppCodeStyle();
-        m_pageCppCodeStylePreferences = new CppCodeStylePreferences(m_widget);
-        m_pageCppCodeStylePreferences->setDelegatingPool(originalCodeStylePreferences->delegatingPool());
-        m_pageCppCodeStylePreferences->setCodeStyleSettings(originalCodeStylePreferences->codeStyleSettings());
-        m_pageCppCodeStylePreferences->setCurrentDelegate(originalCodeStylePreferences->currentDelegate());
+        CppCodeStylePreferences *originalCodeStylePreferences = CppToolsSettings::instance()
+                                                                    ->cppCodeStyle();
+        m_pageCppCodeStylePreferences = new CppCodeStylePreferences();
+        m_pageCppCodeStylePreferences->setDelegatingPool(
+            originalCodeStylePreferences->delegatingPool());
+        m_pageCppCodeStylePreferences->setCodeStyleSettings(
+            originalCodeStylePreferences->codeStyleSettings());
+        m_pageCppCodeStylePreferences->setCurrentDelegate(
+            originalCodeStylePreferences->currentDelegate());
         // we set id so that it won't be possible to set delegate to the original prefs
         m_pageCppCodeStylePreferences->setId(originalCodeStylePreferences->id());
-        m_widget = new CodeStyleEditor(TextEditorSettings::codeStyleFactory(CppTools::Constants::CPP_SETTINGS_ID),
-                                       m_pageCppCodeStylePreferences);
+        m_widget = TextEditorSettings::codeStyleFactory(CppTools::Constants::CPP_SETTINGS_ID)
+                       ->createCodeStyleEditor(m_pageCppCodeStylePreferences);
     }
     return m_widget;
 }
@@ -554,6 +393,8 @@ void CppCodeStyleSettingsPage::apply()
             originalCppCodeStylePreferences->setCurrentDelegate(m_pageCppCodeStylePreferences->currentDelegate());
             originalCppCodeStylePreferences->toSettings(QLatin1String(CppTools::Constants::CPP_SETTINGS_ID), s);
         }
+
+        m_widget->apply();
     }
 }
 
