@@ -110,21 +110,21 @@ DebuggerKitChooser::DebuggerKitChooser(Mode mode, QWidget *parent)
 {
     setKitPredicate([this](const Kit *k) {
         // Match valid debuggers and restrict local debugging to compatible toolchains.
-        auto errors = DebuggerKitInformation::configurationErrors(k);
+        auto errors = DebuggerKitAspect::configurationErrors(k);
         // we do not care for mismatched ABI if we want *any* debugging
-        if (m_mode == AnyDebugging && errors == DebuggerKitInformation::DebuggerDoesNotMatch)
-            errors = DebuggerKitInformation::NoConfigurationError;
+        if (m_mode == AnyDebugging && errors == DebuggerKitAspect::DebuggerDoesNotMatch)
+            errors = DebuggerKitAspect::NoConfigurationError;
         if (errors)
             return false;
         if (m_mode == LocalDebugging)
-            return ToolChainKitInformation::targetAbi(k).os() == m_hostAbi.os();
+            return ToolChainKitAspect::targetAbi(k).os() == m_hostAbi.os();
         return true;
     });
 }
 
 QString DebuggerKitChooser::kitToolTip(Kit *k) const
 {
-    return DebuggerKitInformation::displayString(k);
+    return DebuggerKitAspect::displayString(k);
 }
 
 ///////////////////////////////////////////////////////////////////////
@@ -150,7 +150,7 @@ public:
     Runnable runnable;
     bool breakAtMain = false;
     bool runInTerminal = false;
-    QString serverStartScript;
+    FilePath serverStartScript;
     QString debugInfoLocation;
 };
 
@@ -172,7 +172,7 @@ QString StartApplicationParameters::displayName() const
 {
     const int maxLength = 60;
 
-    QString name = FileName::fromString(runnable.executable).fileName()
+    QString name = FilePath::fromString(runnable.executable).fileName()
             + ' ' + runnable.commandLineArguments;
     if (name.size() > 60) {
         int index = name.lastIndexOf(' ', maxLength);
@@ -198,7 +198,7 @@ void StartApplicationParameters::toSettings(QSettings *settings) const
     settings->setValue("LastExternalWorkingDirectory", runnable.workingDirectory);
     settings->setValue("LastExternalBreakAtMain", breakAtMain);
     settings->setValue("LastExternalRunInTerminal", runInTerminal);
-    settings->setValue("LastServerStartScript", serverStartScript);
+    settings->setValue("LastServerStartScript", serverStartScript.toVariant());
     settings->setValue("LastDebugInfoLocation", debugInfoLocation);
 }
 
@@ -212,7 +212,7 @@ void StartApplicationParameters::fromSettings(const QSettings *settings)
     runnable.workingDirectory = settings->value("LastExternalWorkingDirectory").toString();
     breakAtMain = settings->value("LastExternalBreakAtMain").toBool();
     runInTerminal = settings->value("LastExternalRunInTerminal").toBool();
-    serverStartScript = settings->value("LastServerStartScript").toString();
+    serverStartScript = FilePath::fromVariant(settings->value("LastServerStartScript"));
     debugInfoLocation = settings->value("LastDebugInfoLocation").toString();
 }
 
@@ -229,9 +229,7 @@ StartApplicationDialog::StartApplicationDialog(QWidget *parent)
     setWindowTitle(tr("Start Debugger"));
 
     d->kitChooser = new KitChooser(this);
-    d->kitChooser->setKitPredicate([](const Kit *k) {
-        return !DebuggerKitInformation::configurationErrors(k);
-    });
+    d->kitChooser->setShowIcons(true);
     d->kitChooser->populate();
 
     d->serverPortLabel = new QLabel(tr("Server port:"), this);
@@ -326,7 +324,7 @@ StartApplicationDialog::StartApplicationDialog(QWidget *parent)
             this, &StartApplicationDialog::updateState);
     connect(d->buttonBox, &QDialogButtonBox::accepted, this, &QDialog::accept);
     connect(d->buttonBox, &QDialogButtonBox::rejected, this, &QDialog::reject);
-    connect(d->historyComboBox, static_cast<void(QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
+    connect(d->historyComboBox, QOverload<int>::of(&QComboBox::currentIndexChanged),
             this, &StartApplicationDialog::historyIndexChanged);
 
     connect(d->channelOverrideEdit, &QLineEdit::textChanged,
@@ -408,10 +406,10 @@ void StartApplicationDialog::run(bool attachRemote)
         return;
 
     Kit *k = dialog.d->kitChooser->currentKit();
-    IDevice::ConstPtr dev = DeviceKitInformation::device(k);
 
-    auto runControl = new RunControl(nullptr, ProjectExplorer::Constants::DEBUG_RUN_MODE);
-    auto debugger = new DebuggerRunTool(runControl, k);
+    auto runControl = new RunControl(ProjectExplorer::Constants::DEBUG_RUN_MODE);
+    runControl->setKit(k);
+    auto debugger = new DebuggerRunTool(runControl);
 
     const StartApplicationParameters newParameters = dialog.parameters();
     if (newParameters != history.back()) {
@@ -428,6 +426,7 @@ void StartApplicationDialog::run(bool attachRemote)
         settings->endGroup();
     }
 
+    IDevice::ConstPtr dev = DeviceKitAspect::device(k);
     Runnable inferior = newParameters.runnable;
     const QString inputAddress = dialog.d->channelOverrideEdit->text();
     if (!inputAddress.isEmpty())
@@ -474,7 +473,7 @@ StartApplicationParameters StartApplicationDialog::parameters() const
     result.serverPort = d->serverPortSpinBox->value();
     result.serverAddress = d->channelOverrideEdit->text();
     result.runnable.executable = d->localExecutablePathChooser->path();
-    result.serverStartScript = d->serverStartScriptPathChooser->path();
+    result.serverStartScript = d->serverStartScriptPathChooser->fileName();
     result.kitId = d->kitChooser->currentKitId();
     result.debugInfoLocation = d->debuginfoPathChooser->path();
     result.runnable.commandLineArguments = d->arguments->text();
@@ -490,7 +489,7 @@ void StartApplicationDialog::setParameters(const StartApplicationParameters &p)
     d->serverPortSpinBox->setValue(p.serverPort);
     d->channelOverrideEdit->setText(p.serverAddress);
     d->localExecutablePathChooser->setPath(p.runnable.executable);
-    d->serverStartScriptPathChooser->setPath(p.serverStartScript);
+    d->serverStartScriptPathChooser->setFileName(p.serverStartScript);
     d->debuginfoPathChooser->setPath(p.debugInfoLocation);
     d->arguments->setText(p.runnable.commandLineArguments);
     d->workingDirectory->setPath(p.runnable.workingDirectory);
