@@ -32,6 +32,7 @@
 #include <QStringList>
 
 namespace ProjectExplorer {
+namespace Internal { class CompileOutputSettings; }
 
 class Task;
 class Project;
@@ -50,13 +51,13 @@ public:
     static bool isBuilding();
     static bool tasksAvailable();
 
-    static bool buildLists(QList<BuildStepList *> bsls, const QStringList &stepListNames,
-                    const QStringList &preambelMessage = QStringList());
-    static bool buildList(BuildStepList *bsl, const QString &stepListName);
+    static bool buildLists(QList<BuildStepList *> bsls,
+                           const QStringList &preambelMessage = QStringList());
+    static bool buildList(BuildStepList *bsl);
 
-    static bool isBuilding(Project *p);
-    static bool isBuilding(Target *t);
-    static bool isBuilding(ProjectConfiguration *p);
+    static bool isBuilding(const Project *p);
+    static bool isBuilding(const Target *t);
+    static bool isBuilding(const ProjectConfiguration *p);
     static bool isBuilding(BuildStep *step);
 
     // Append any build step to the list of build steps (currently only used to add the QMakeStep)
@@ -64,6 +65,9 @@ public:
 
     static int getErrorTaskCount();
 
+    static void setCompileOutputSettings(const Internal::CompileOutputSettings &settings);
+    static const Internal::CompileOutputSettings &compileOutputSettings();
+
 public slots:
     static void cancel();
     // Shows without focus
@@ -84,10 +88,8 @@ private:
     static void addToOutputWindow(const QString &string, BuildStep::OutputFormat format,
                            BuildStep::OutputNewlineSetting newlineSettings = BuildStep::DoAppendNewline);
 
-    void buildStepFinishedAsync();
     static void nextBuildQueue();
-    static void progressChanged();
-    static void progressTextChanged();
+    static void progressChanged(int percent, const QString &text);
     static void emitCancelMessage();
     static void showBuildResults();
     static void updateTaskCount();
@@ -96,7 +98,7 @@ private:
     static void startBuildQueue();
     static void nextStep();
     static void clearBuildQueue();
-    static bool buildQueueAppend(QList<BuildStep *> steps, QStringList names, const QStringList &preambleMessage = QStringList());
+    static bool buildQueueAppend(const QList<BuildStep *> &steps, QStringList names, const QStringList &preambleMessage = QStringList());
     static void incrementActiveBuildSteps(BuildStep *bs);
     static void decrementActiveBuildSteps(BuildStep *bs);
     static void disconnectOutput(BuildStep *bs);
