@@ -33,6 +33,8 @@
 #include <QSet>
 #include <QVariant>
 
+#include <memory>
+
 namespace Utils {
 class Environment;
 class MacroExpander;
@@ -59,6 +61,8 @@ public:
     using Predicate = std::function<bool(const Kit *)>;
 
     explicit Kit(Core::Id id = Core::Id());
+    explicit Kit(const QVariantMap &data);
+    ~Kit();
 
     // Do not trigger evaluations
     void blockNotification();
@@ -67,7 +71,7 @@ public:
 
     bool isValid() const;
     bool hasWarning() const;
-    QList<Task> validate() const;
+    Tasks validate() const;
     void fix(); // Fix the individual kit information: Make sure it contains a valid value.
                 // Fix will not look at other information in the kit!
     void setup(); // Apply advanced magic(TM). Used only once on each kit during initial setup.
@@ -86,9 +90,17 @@ public:
     bool isSdkProvided() const;
     Core::Id id() const;
 
-    QIcon icon() const;
-    Utils::FileName iconPath() const;
-    void setIconPath(const Utils::FileName &path);
+    // The higher the weight, the more aspects have sensible values for this kit.
+    // For instance, a kit where a matching debugger was found for the toolchain will have a
+    // higher weight than one whose toolchain does not match a known debugger, assuming
+    // all other aspects are equal.
+    int weight() const;
+
+    QIcon icon() const; // Raw device icon, independent of warning or error.
+    QIcon displayIcon() const; // Error or warning or device icon.
+    Utils::FilePath iconPath() const;
+    void setIconPath(const Utils::FilePath &path);
+    void setDeviceTypeForIcon(Core::Id deviceType);
 
     QList<Core::Id> allKeys() const;
     QVariant value(Core::Id key, const QVariant &unset = QVariant()) const;
@@ -105,7 +117,7 @@ public:
     void addToEnvironment(Utils::Environment &env) const;
     IOutputParser *createOutputParser() const;
 
-    QString toHtml(const QList<Task> &additional = QList<Task>()) const;
+    QString toHtml(const Tasks &additional = Tasks()) const;
     Kit *clone(bool keepName = false) const;
     void copyFrom(const Kit *k);
 
@@ -119,16 +131,20 @@ public:
     void setMutable(Core::Id id, bool b);
     bool isMutable(Core::Id id) const;
 
+    void setIrrelevantAspects(const QSet<Core::Id> &irrelevant);
+    QSet<Core::Id> irrelevantAspects() const;
+
     QSet<Core::Id> supportedPlatforms() const;
     QSet<Core::Id> availableFeatures() const;
     bool hasFeatures(const QSet<Core::Id> &features) const;
     Utils::MacroExpander *macroExpander() const;
 
-private:
-    void setSdkProvided(bool sdkProvided);
+    QString newKitName(const QList<Kit *> &allKits) const;
+    static QString newKitName(const QString &name, const QList<Kit *> &allKits);
 
-    ~Kit();
-    Kit(const QVariantMap &data);
+private:
+    static void copyKitCommon(Kit *target, const Kit *source);
+    void setSdkProvided(bool sdkProvided);
 
     // Unimplemented.
     Kit(const Kit &other);
@@ -139,9 +155,9 @@ private:
 
     QVariantMap toMap() const;
 
-    Internal::KitPrivate *const d;
+    const std::unique_ptr<Internal::KitPrivate> d;
 
-    friend class KitInformation;
+    friend class KitAspect;
     friend class KitManager;
     friend class Internal::KitManagerPrivate;
     friend class Internal::KitModel; // needed for setAutoDetected() when cloning kits
