@@ -25,6 +25,7 @@
 
 #include "projectconfiguration.h"
 
+#include <utils/algorithm.h>
 #include <utils/qtcassert.h>
 
 using namespace ProjectExplorer;
@@ -33,6 +34,53 @@ const char CONFIGURATION_ID_KEY[] = "ProjectExplorer.ProjectConfiguration.Id";
 const char DISPLAY_NAME_KEY[] = "ProjectExplorer.ProjectConfiguration.DisplayName";
 const char DEFAULT_DISPLAY_NAME_KEY[] = "ProjectExplorer.ProjectConfiguration.DefaultDisplayName";
 
+// ProjectConfigurationAspect
+
+ProjectConfigurationAspect::ProjectConfigurationAspect() = default;
+
+ProjectConfigurationAspect::~ProjectConfigurationAspect() = default;
+
+void ProjectConfigurationAspect::setConfigWidgetCreator
+    (const ConfigWidgetCreator &configWidgetCreator)
+{
+    m_configWidgetCreator = configWidgetCreator;
+}
+
+QWidget *ProjectConfigurationAspect::createConfigWidget() const
+{
+    return m_configWidgetCreator ? m_configWidgetCreator() : nullptr;
+}
+
+
+// ProjectConfigurationAspects
+
+ProjectConfigurationAspects::ProjectConfigurationAspects() = default;
+
+ProjectConfigurationAspects::~ProjectConfigurationAspects()
+{
+    qDeleteAll(base());
+}
+
+ProjectConfigurationAspect *ProjectConfigurationAspects::aspect(Core::Id id) const
+{
+    return Utils::findOrDefault(base(), Utils::equal(&ProjectConfigurationAspect::id, id));
+}
+
+void ProjectConfigurationAspects::fromMap(const QVariantMap &map) const
+{
+    for (ProjectConfigurationAspect *aspect : *this)
+        aspect->fromMap(map);
+}
+
+void ProjectConfigurationAspects::toMap(QVariantMap &map) const
+{
+    for (ProjectConfigurationAspect *aspect : *this)
+        aspect->toMap(map);
+}
+
+
+// ProjectConfiguration
+
 ProjectConfiguration::ProjectConfiguration(QObject *parent, Core::Id id)
     : QObject(parent), m_id(id)
 {
@@ -40,6 +88,8 @@ ProjectConfiguration::ProjectConfiguration(QObject *parent, Core::Id id)
     setObjectName(id.toString());
 }
 
+ProjectConfiguration::~ProjectConfiguration() = default;
+
 Core::Id ProjectConfiguration::id() const
 {
     return m_id;
@@ -103,6 +153,9 @@ QVariantMap ProjectConfiguration::toMap() const
     map.insert(QLatin1String(CONFIGURATION_ID_KEY), m_id.toSetting());
     map.insert(QLatin1String(DISPLAY_NAME_KEY), m_displayName);
     map.insert(QLatin1String(DEFAULT_DISPLAY_NAME_KEY), m_defaultDisplayName);
+
+    m_aspects.toMap(map);
+
     return map;
 }
 
@@ -117,27 +170,24 @@ bool ProjectConfiguration::fromMap(const QVariantMap &map)
     m_defaultDisplayName = map.value(QLatin1String(DEFAULT_DISPLAY_NAME_KEY),
                                      m_defaultDisplayName.isEmpty() ?
                                          m_displayName : m_defaultDisplayName).toString();
+
+    m_aspects.fromMap(map);
+
     return true;
 }
 
+ProjectConfigurationAspect *ProjectConfiguration::aspect(Core::Id id) const
+{
+    return m_aspects.aspect(id);
+}
+
+void ProjectConfiguration::acquaintAspects()
+{
+    for (ProjectConfigurationAspect *aspect : m_aspects)
+        aspect->acquaintSiblings(m_aspects);
+}
+
 Core::Id ProjectExplorer::idFromMap(const QVariantMap &map)
 {
     return Core::Id::fromSetting(map.value(QLatin1String(CONFIGURATION_ID_KEY)));
 }
-
-bool StatefulProjectConfiguration::isEnabled() const
-{
-    return m_isEnabled;
-}
-
-StatefulProjectConfiguration::StatefulProjectConfiguration(QObject *parent, Core::Id id) :
-    ProjectConfiguration(parent, id)
-{ }
-
-void StatefulProjectConfiguration::setEnabled(bool enabled)
-{
-    if (enabled == m_isEnabled)
-        return;
-    m_isEnabled = enabled;
-    emit enabledChanged();
-}
