@@ -39,6 +39,7 @@
 #include <utils/hostosinfo.h>
 #include <utils/mimetypes/mimedatabase.h>
 #include <utils/mimetypes/mimetype.h>
+#include <utils/pointeralgorithm.h>
 #include <utils/qtcassert.h>
 
 #include <QFileInfo>
@@ -51,23 +52,22 @@
 
 namespace ProjectExplorer {
 
-static FolderNode *folderNode(const FolderNode *folder, const Utils::FileName &directory)
+static FolderNode *folderNode(const FolderNode *folder, const Utils::FilePath &directory)
 {
-    return static_cast<FolderNode *>(Utils::findOrDefault(folder->nodes(),
-                                                          [&directory](const Node *n) {
-        const FolderNode *fn = n->asFolderNode();
+    return static_cast<FolderNode *>(Utils::findOrDefault(folder->folderNodes(),
+                                                          [&directory](const FolderNode *fn) {
         return fn && fn->filePath() == directory;
     }));
 }
 
 static FolderNode *recursiveFindOrCreateFolderNode(FolderNode *folder,
-                                                   const Utils::FileName &directory,
-                                                   const Utils::FileName &overrideBaseDir,
+                                                   const Utils::FilePath &directory,
+                                                   const Utils::FilePath &overrideBaseDir,
                                                    const FolderNode::FolderNodeFactory &factory)
 {
-    Utils::FileName path = overrideBaseDir.isEmpty() ? folder->filePath() : overrideBaseDir;
+    Utils::FilePath path = overrideBaseDir.isEmpty() ? folder->filePath() : overrideBaseDir;
 
-    Utils::FileName directoryWithoutPrefix;
+    Utils::FilePath directoryWithoutPrefix;
     bool isRelative = false;
 
     if (path.isEmpty() || path.toFileInfo().isRoot()) {
@@ -89,15 +89,15 @@ static FolderNode *recursiveFindOrCreateFolderNode(FolderNode *folder,
 
     ProjectExplorer::FolderNode *parent = folder;
     foreach (const QString &part, parts) {
-        path.appendPath(part);
+        path = path.pathAppended(part);
         // Find folder in subFolders
         FolderNode *next = folderNode(parent, path);
         if (!next) {
             // No FolderNode yet, so create it
             auto tmp = factory(path);
             tmp->setDisplayName(part);
-            parent->addNode(tmp);
-            next = tmp;
+            next = tmp.get();
+            parent->addNode(std::move(tmp));
         }
         parent = next;
     }
@@ -119,32 +119,40 @@ static FolderNode *recursiveFindOrCreateFolderNode(FolderNode *folder,
   \sa ProjectExplorer::NodesWatcher
 */
 
-Node::Node(NodeType nodeType, const Utils::FileName &filePath, int line, const QByteArray &id) :
-    m_filePath(filePath), m_nodeId(id), m_line(line), m_nodeType(nodeType)
-{ }
+Node::Node() = default;
 
 void Node::setPriority(int p)
 {
     m_priority = p;
 }
 
+void Node::setFilePath(const Utils::FilePath &filePath)
+{
+    m_filePath = filePath;
+}
+
+void Node::setLine(int line)
+{
+    m_line = line;
+}
+
 void Node::setListInProject(bool l)
 {
     if (l)
-        m_flags |= FlagListInProject;
+        m_flags = static_cast<NodeFlag>(m_flags | FlagListInProject);
     else
-        m_flags &= ~FlagListInProject;
+        m_flags = static_cast<NodeFlag>(m_flags & ~FlagListInProject);
 }
 
 void Node::setIsGenerated(bool g)
 {
     if (g)
-        m_flags |= FlagIsGenerated;
+        m_flags = static_cast<NodeFlag>(m_flags | FlagIsGenerated);
     else
-        m_flags &= ~FlagIsGenerated;
+        m_flags = static_cast<NodeFlag>(m_flags & ~FlagIsGenerated);
 }
 
-void Node::setAbsoluteFilePathAndLine(const Utils::FileName &path, int line)
+void Node::setAbsoluteFilePathAndLine(const Utils::FilePath &path, int line)
 {
     if (m_filePath == path && m_line == line)
         return;
@@ -155,11 +163,6 @@ void Node::setAbsoluteFilePathAndLine(const Utils::FileName &path, int line)
 
 Node::~Node() = default;
 
-NodeType Node::nodeType() const
-{
-    return m_nodeType;
-}
-
 int Node::priority() const
 {
     return m_priority;
@@ -170,7 +173,7 @@ int Node::priority() const
   */
 bool Node::listInProject() const
 {
-    return m_flags.testFlag(FlagListInProject);
+    return (m_flags & FlagListInProject) == FlagListInProject;
 }
 
 /*!
@@ -212,7 +215,7 @@ const ProjectNode *Node::managingProject() const
 /*!
   The path of the file or folder in the filesystem the node represents.
   */
-const Utils::FileName &Node::filePath() const
+const Utils::FilePath &Node::filePath() const
 {
     return m_filePath;
 }
@@ -222,11 +225,6 @@ int Node::line() const
     return m_line;
 }
 
-QByteArray Node::id() const
-{
-    return m_nodeId;
-}
-
 QString Node::displayName() const
 {
     return filePath().fileName();
@@ -239,7 +237,7 @@ QString Node::tooltip() const
 
 bool Node::isEnabled() const
 {
-    if (!m_flags.testFlag(FlagIsEnabled))
+    if ((m_flags & FlagIsEnabled) == 0)
         return false;
     FolderNode *parent = parentFolderNode();
     return parent ? parent->isEnabled() : true;
@@ -253,7 +251,7 @@ bool Node::isGenerated() const
     return (m_flags & FlagIsGenerated) == FlagIsGenerated;
 }
 
-bool Node::supportsAction(ProjectAction, Node *) const
+bool Node::supportsAction(ProjectAction, const Node *) const
 {
     return false;
 }
@@ -261,9 +259,9 @@ bool Node::supportsAction(ProjectAction, Node *) const
 void Node::setEnabled(bool enabled)
 {
     if (enabled)
-        m_flags |= FlagIsEnabled;
+        m_flags = static_cast<NodeFlag>(m_flags | FlagIsEnabled);
     else
-        m_flags &= ~FlagIsEnabled;
+        m_flags = static_cast<NodeFlag>(m_flags & ~FlagIsEnabled);
 }
 
 bool Node::sortByPath(const Node *a, const Node *b)
@@ -290,7 +288,8 @@ FileType Node::fileTypeForMimeType(const Utils::MimeType &mt)
             type = FileType::Resource;
         else if (mtName == Constants::SCXML_MIMETYPE)
             type = FileType::StateChart;
-        else if (mtName == Constants::QML_MIMETYPE)
+        else if (mtName == Constants::QML_MIMETYPE
+                 || mtName == Constants::QMLUI_MIMETYPE)
             type = FileType::QML;
     } else {
         type = FileType::Unknown;
@@ -298,7 +297,7 @@ FileType Node::fileTypeForMimeType(const Utils::MimeType &mt)
     return type;
 }
 
-FileType Node::fileTypeForFileName(const Utils::FileName &file)
+FileType Node::fileTypeForFileName(const Utils::FilePath &file)
 {
     return fileTypeForMimeType(Utils::mimeTypeForFile(file.toString(),
                                                       Utils::MimeMatchMode::MatchExtension));
@@ -314,13 +313,11 @@ FileType Node::fileTypeForFileName(const Utils::FileName &file)
   \sa ProjectExplorer::FolderNode, ProjectExplorer::ProjectNode
 */
 
-FileNode::FileNode(const Utils::FileName &filePath, const FileType fileType, bool generated,
-                   int line, const QByteArray &id) :
-    Node(NodeType::File, filePath, line, id),
+FileNode::FileNode(const Utils::FilePath &filePath, const FileType fileType) :
     m_fileType(fileType)
 {
+    setFilePath(filePath);
     setListInProject(true);
-    setIsGenerated(generated);
     if (fileType == FileType::Project)
         setPriority(DefaultProjectFilePriority);
     else
@@ -329,7 +326,9 @@ FileNode::FileNode(const Utils::FileName &filePath, const FileType fileType, boo
 
 FileNode *FileNode::clone() const
 {
-    auto fn = new FileNode(filePath(), fileType(), isGenerated(), line(), id());
+    auto fn = new FileNode(filePath(), fileType());
+    fn->setLine(line());
+    fn->setIsGenerated(isGenerated());
     fn->setEnabled(isEnabled());
     fn->setPriority(priority());
     fn->setListInProject(listInProject());
@@ -341,8 +340,8 @@ FileType FileNode::fileType() const
     return m_fileType;
 }
 
-static QList<FileNode *> scanForFilesRecursively(const Utils::FileName &directory,
-                                                 const std::function<FileNode *(const Utils::FileName &)> factory,
+static QList<FileNode *> scanForFilesRecursively(const Utils::FilePath &directory,
+                                                 const std::function<FileNode *(const Utils::FilePath &)> factory,
                                                  QSet<QString> &visited, QFutureInterface<QList<FileNode*>> *future,
                                                  double progressStart, double progressRange,
                                                  const QList<Core::IVersionControl*> &versionControls)
@@ -357,7 +356,7 @@ static QList<FileNode *> scanForFilesRecursively(const Utils::FileName &director
     if (visitedCount == visited.count())
         return result;
 
-    const QList<QFileInfo> entries = baseDir.entryInfoList(QStringList(), QDir::AllEntries|QDir::NoDotAndDotDot);
+    const QFileInfoList entries = baseDir.entryInfoList(QStringList(), QDir::AllEntries|QDir::NoDotAndDotDot);
     double progress = 0;
     const double progressIncrement = progressRange / static_cast<double>(entries.count());
     int lastIntProgress = 0;
@@ -365,7 +364,7 @@ static QList<FileNode *> scanForFilesRecursively(const Utils::FileName &director
         if (future && future->isCanceled())
             return result;
 
-        const Utils::FileName entryName = Utils::FileName::fromString(entry.absoluteFilePath());
+        const Utils::FilePath entryName = Utils::FilePath::fromString(entry.absoluteFilePath());
         if (!Utils::contains(versionControls, [&entryName](const Core::IVersionControl *vc) {
                              return vc->isVcsFileOrDirectory(entryName);
             })) {
@@ -388,27 +387,19 @@ static QList<FileNode *> scanForFilesRecursively(const Utils::FileName &director
     return result;
 }
 
-
-QList<FileNode *> FileNode::scanForFiles(const Utils::FileName &directory,
-                                         const std::function<FileNode *(const Utils::FileName &)> factory,
-                                         QFutureInterface<QList<FileNode *> > *future)
-{
-    return FileNode::scanForFilesWithVersionControls(directory, factory, QList<Core::IVersionControl *>(), future);
-}
-
 QList<FileNode *>
-FileNode::scanForFilesWithVersionControls(const Utils::FileName &directory,
-                                          const std::function<FileNode *(const Utils::FileName &)> factory,
-                                          const QList<Core::IVersionControl *> &versionControls,
-                                          QFutureInterface<QList<FileNode *>> *future)
+FileNode::scanForFiles(const Utils::FilePath &directory,
+                       const std::function<FileNode *(const Utils::FilePath &)> factory,
+                       QFutureInterface<QList<FileNode *>> *future)
 {
     QSet<QString> visited;
     if (future)
         future->setProgressRange(0, 1000000);
-    return scanForFilesRecursively(directory, factory, visited, future, 0.0, 1000000.0, versionControls);
+    return scanForFilesRecursively(directory, factory, visited, future, 0.0, 1000000.0,
+                                   Core::VcsManager::versionControls());
 }
 
-bool FileNode::supportsAction(ProjectAction action, Node *node) const
+bool FileNode::supportsAction(ProjectAction action, const Node *node) const
 {
     if (action == InheritedFromParent)
         return true;
@@ -416,6 +407,14 @@ bool FileNode::supportsAction(ProjectAction action, Node *node) const
     return parentFolder && parentFolder->supportsAction(action, node);
 }
 
+QString FileNode::displayName() const
+{
+    int l = line();
+    if (l < 0)
+        return Node::displayName();
+    return Node::displayName() + ':' + QString::number(l);
+}
+
 /*!
   \class ProjectExplorer::FolderNode
 
@@ -423,21 +422,13 @@ bool FileNode::supportsAction(ProjectAction action, Node *node) const
 
   \sa ProjectExplorer::FileNode, ProjectExplorer::ProjectNode
 */
-FolderNode::FolderNode(const Utils::FileName &folderPath, NodeType nodeType,
-                       const QString &displayName, const QByteArray &id) :
-    Node(nodeType, folderPath, -1, id),
-    m_displayName(displayName)
+FolderNode::FolderNode(const Utils::FilePath &folderPath)
 {
+    setFilePath(folderPath);
     setPriority(DefaultFolderPriority);
     setListInProject(false);
     setIsGenerated(false);
-    if (m_displayName.isEmpty())
-        m_displayName = folderPath.toUserOutput();
-}
-
-FolderNode::~FolderNode()
-{
-    qDeleteAll(m_nodes);
+    m_displayName = folderPath.toUserOutput();
 }
 
 /*!
@@ -468,9 +459,9 @@ Node *FolderNode::findNode(const std::function<bool(Node *)> &filter)
     if (filter(this))
         return this;
 
-    for (Node *n : m_nodes) {
-        if (n->asFileNode() && filter(n)) {
-            return n;
+    for (const std::unique_ptr<Node> &n : m_nodes) {
+        if (n->asFileNode() && filter(n.get())) {
+            return n.get();
         } else if (FolderNode *folder = n->asFolderNode()) {
             Node *result = folder->findNode(filter);
             if (result)
@@ -485,11 +476,11 @@ QList<Node *> FolderNode::findNodes(const std::function<bool(Node *)> &filter)
     QList<Node *> result;
     if (filter(this))
         result.append(this);
-    for (Node *n : m_nodes) {
-        if (n->asFileNode() && filter(n))
-            result.append(n);
+    for (const std::unique_ptr<Node> &n  : m_nodes) {
+        if (n->asFileNode() && filter(n.get()))
+            result.append(n.get());
         else if (FolderNode *folder = n->asFolderNode())
-            result.append(folder->findNode(filter));
+            result.append(folder->findNodes(filter));
     }
     return result;
 }
@@ -503,12 +494,12 @@ void FolderNode::forEachNode(const std::function<void(FileNode *)> &fileTask,
             return;
     }
     if (fileTask) {
-        for (Node *n : m_nodes) {
+        for (const std::unique_ptr<Node> &n : m_nodes) {
             if (FileNode *fn = n->asFileNode())
                 fileTask(fn);
         }
     }
-    for (Node *n : m_nodes) {
+    for (const std::unique_ptr<Node> &n : m_nodes) {
         if (FolderNode *fn = n->asFolderNode()) {
             if (folderTask)
                 folderTask(fn);
@@ -519,26 +510,59 @@ void FolderNode::forEachNode(const std::function<void(FileNode *)> &fileTask,
 
 void FolderNode::forEachGenericNode(const std::function<void(Node *)> &genericTask) const
 {
-    for (Node *n : m_nodes) {
-        genericTask(n);
+    for (const std::unique_ptr<Node> &n : m_nodes) {
+        genericTask(n.get());
         if (FolderNode *fn = n->asFolderNode())
             fn->forEachGenericNode(genericTask);
     }
 }
 
+void FolderNode::forEachProjectNode(const std::function<void(const ProjectNode *)> &task) const
+{
+    if (const ProjectNode *projectNode = asProjectNode())
+        task(projectNode);
+
+    for (const std::unique_ptr<Node> &n : m_nodes) {
+        if (FolderNode *fn = n->asFolderNode())
+            fn->forEachProjectNode(task);
+    }
+}
+
+ProjectNode *FolderNode::findProjectNode(const std::function<bool(const ProjectNode *)> &predicate)
+{
+    if (ProjectNode *projectNode = asProjectNode()) {
+        if (predicate(projectNode))
+            return projectNode;
+    }
+
+    for (const std::unique_ptr<Node> &n : m_nodes) {
+        if (FolderNode *fn = n->asFolderNode()) {
+            if (ProjectNode *pn = fn->findProjectNode(predicate))
+                return pn;
+        }
+    }
+    return nullptr;
+}
+
+const QList<Node *> FolderNode::nodes() const
+{
+    return Utils::toRawPointer<QList>(m_nodes);
+}
+
 QList<FileNode*> FolderNode::fileNodes() const
 {
     QList<FileNode *> result;
-    for (Node *n : m_nodes) {
+    for (const std::unique_ptr<Node> &n : m_nodes) {
         if (FileNode *fn = n->asFileNode())
             result.append(fn);
     }
     return result;
 }
 
-FileNode *FolderNode::fileNode(const Utils::FileName &file) const
+FileNode *FolderNode::fileNode(const Utils::FilePath &file) const
 {
-    return static_cast<FileNode *>(Utils::findOrDefault(m_nodes, [&file](const Node *n) {
+    return static_cast<FileNode *>(Utils::findOrDefault(m_nodes,
+                                                        [&file](const std::unique_ptr<Node> &n) {
         const FileNode *fn = n->asFileNode();
         return fn && fn->filePath() == file;
     }));
@@ -547,28 +571,28 @@ FileNode *FolderNode::fileNode(const Utils::FileName &file) const
 QList<FolderNode*> FolderNode::folderNodes() const
 {
     QList<FolderNode *> result;
-    for (Node *n : m_nodes) {
+    for (const std::unique_ptr<Node> &n : m_nodes) {
         if (FolderNode *fn = n->asFolderNode())
             result.append(fn);
     }
     return result;
 }
 
-void FolderNode::addNestedNode(FileNode *fileNode, const Utils::FileName &overrideBaseDir,
+void FolderNode::addNestedNode(std::unique_ptr<FileNode> &&fileNode,
+                               const Utils::FilePath &overrideBaseDir,
                                const FolderNodeFactory &factory)
 {
-    // Get relative path to rootNode
     FolderNode *folder = recursiveFindOrCreateFolderNode(this, fileNode->filePath().parentDir(),
                                                          overrideBaseDir, factory);
-    folder->addNode(fileNode);
-
+    folder->addNode(std::move(fileNode));
 }
 
-void FolderNode::addNestedNodes(const QList<FileNode *> &files, const Utils::FileName &overrideBaseDir,
-                                const FolderNodeFactory &factory)
+void FolderNode::addNestedNodes(std::vector<std::unique_ptr<FileNode> > &&files,
+                                const Utils::FilePath &overrideBaseDir,
+                                const FolderNode::FolderNodeFactory &factory)
 {
-    for (FileNode *fn : files)
-        addNestedNode(fn, overrideBaseDir, factory);
+    for (std::unique_ptr<FileNode> &f : files)
+        addNestedNode(std::move(f), overrideBaseDir, factory);
 }
 
 // "Compress" a tree of foldernodes such that foldernodes with exactly one foldernode as a child
@@ -577,21 +601,23 @@ void FolderNode::addNestedNodes(const QList<FileNode *> &files, const Utils::Fil
 // files.
 void FolderNode::compress()
 {
-    QList<Node *> children = nodes();
-    if (auto subFolder = children.count() == 1 ? children.at(0)->asFolderNode() : nullptr) {
-        if (subFolder->nodeType() != nodeType())
+    if (auto subFolder = m_nodes.size() == 1 ? m_nodes.at(0)->asFolderNode() : nullptr) {
+        const bool sameType = (isFolderNodeType() && subFolder->isFolderNodeType())
+                || (isProjectNodeType() && subFolder->isProjectNodeType())
+                || (isVirtualFolderType() && subFolder->isVirtualFolderType());
+        if (!sameType)
             return;
+
         // Only one subfolder: Compress!
         setDisplayName(QDir::toNativeSeparators(displayName() + "/" + subFolder->displayName()));
         for (Node *n : subFolder->nodes()) {
-            subFolder->removeNode(n);
-            n->setParentFolderNode(nullptr);
-            addNode(n);
+            std::unique_ptr<Node> toMove = subFolder->takeNode(n);
+            toMove->setParentFolderNode(nullptr);
+            addNode(std::move(toMove));
         }
         setAbsoluteFilePathAndLine(subFolder->filePath(), -1);
 
-        removeNode(subFolder);
-        delete subFolder;
+        takeNode(subFolder);
 
         compress();
     } else {
@@ -600,34 +626,26 @@ void FolderNode::compress()
     }
 }
 
-bool FolderNode::isAncesterOf(Node *n)
+bool FolderNode::replaceSubtree(Node *oldNode, std::unique_ptr<Node> &&newNode)
 {
-    if (n == this)
-        return true;
-    FolderNode *p = n->parentFolderNode();
-    while (p && p != this)
-        p = p->parentFolderNode();
-    return p == this;
-}
-
-bool FolderNode::replaceSubtree(Node *oldNode, Node *newNode)
-{
-    std::unique_ptr<Node> nn(newNode);
+    std::unique_ptr<Node> keepAlive;
     if (!oldNode) {
-        addNode(nn.release()); // Happens e.g. when a project is registered
+        addNode(std::move(newNode)); // Happens e.g. when a project is registered
     } else {
         auto it = std::find_if(m_nodes.begin(), m_nodes.end(),
-                               [oldNode](const Node *n) { return oldNode == n; });
+                               [oldNode](const std::unique_ptr<Node> &n) {
+            return oldNode == n.get();
+        });
         QTC_ASSERT(it != m_nodes.end(), return false);
-        if (nn) {
-            nn->setParentFolderNode(this);
-            *it = nn.release();
+        if (newNode) {
+            newNode->setParentFolderNode(this);
+            keepAlive = std::move(*it);
+            *it = std::move(newNode);
         } else {
-            removeNode(oldNode); // Happens e.g. when project is shutting down
+            keepAlive = takeNode(oldNode); // Happens e.g. when project is shutting down
         }
-        QTimer::singleShot(0, [oldNode]() { delete oldNode; });
     }
-    ProjectTree::emitSubtreeChanged(this);
+    handleSubTreeChanged(this);
     return true;
 }
 
@@ -643,13 +661,26 @@ void FolderNode::setIcon(const QIcon &icon)
     m_icon = icon;
 }
 
+void FolderNode::setLocationInfo(const QList<FolderNode::LocationInfo> &info)
+{
+    m_locations = info;
+}
+
+const QList<FolderNode::LocationInfo> FolderNode::locationInfo() const
+{
+    return m_locations;
+}
+
 QString FolderNode::addFileFilter() const
 {
+    if (!m_addFileFilter.isNull())
+        return m_addFileFilter;
+
     FolderNode *fn = parentFolderNode();
     return fn ? fn->addFileFilter() : QString();
 }
 
-bool FolderNode::supportsAction(ProjectAction action, Node *node) const
+bool FolderNode::supportsAction(ProjectAction action, const Node *node) const
 {
     if (action == InheritedFromParent)
         return true;
@@ -707,22 +738,21 @@ FolderNode::AddNewInformation FolderNode::addNewInformation(const QStringList &f
   Adds a node specified by \a node to the internal list of nodes.
 */
 
-void FolderNode::addNode(Node *node)
+void FolderNode::addNode(std::unique_ptr<Node> &&node)
 {
     QTC_ASSERT(node, return);
-    QTC_ASSERT(!node->parentFolderNode(), qDebug("File node has already a parent folder"));
+    QTC_ASSERT(!node->parentFolderNode(), qDebug("Node has already a parent folder"));
     node->setParentFolderNode(this);
-    m_nodes.append(node);
+    m_nodes.emplace_back(std::move(node));
 }
 
 /*!
-  Removes a node specified by \a node from the internal list of nodes.
-  The node object itself is not deleted.
+  Return a node specified by \a node from the internal list.
 */
 
-void FolderNode::removeNode(Node *node)
+std::unique_ptr<Node> FolderNode::takeNode(Node *node)
 {
-    m_nodes.removeOne(node);
+    return Utils::takeOrDefault(m_nodes, node);
 }
 
 bool FolderNode::showInSimpleTree() const
@@ -732,7 +762,7 @@ bool FolderNode::showInSimpleTree() const
 
 bool FolderNode::showWhenEmpty() const
 {
-    return false;
+    return m_showWhenEmpty;
 }
 
 /*!
@@ -746,18 +776,9 @@ bool FolderNode::showWhenEmpty() const
 
   \sa ProjectExplorer::FileNode, ProjectExplorer::ProjectNode
 */
-VirtualFolderNode::VirtualFolderNode(const Utils::FileName &folderPath, int priority,
-                                     const QByteArray &id) :
-    FolderNode(folderPath, NodeType::VirtualFolder, QString(), id)
+VirtualFolderNode::VirtualFolderNode(const Utils::FilePath &folderPath) :
+    FolderNode(folderPath)
 {
-    setPriority(priority);
-}
-
-QString VirtualFolderNode::addFileFilter() const
-{
-    if (!m_addFileFilter.isNull())
-        return m_addFileFilter;
-    return FolderNode::addFileFilter();
 }
 
 /*!
@@ -773,11 +794,12 @@ QString VirtualFolderNode::addFileFilter() const
 /*!
   Creates an uninitialized project node object.
   */
-ProjectNode::ProjectNode(const Utils::FileName &projectFilePath, const QByteArray &id) :
-    FolderNode(projectFilePath, NodeType::Project, projectFilePath.fileName(), id)
+ProjectNode::ProjectNode(const Utils::FilePath &projectFilePath) :
+    FolderNode(projectFilePath)
 {
     setPriority(DefaultProjectPriority);
     setListInProject(true);
+    setDisplayName(projectFilePath.fileName());
 }
 
 bool ProjectNode::canAddSubProject(const QString &proFilePath) const
@@ -792,6 +814,11 @@ bool ProjectNode::addSubProject(const QString &proFilePath)
     return false;
 }
 
+QStringList ProjectNode::subProjectFileNamePatterns() const
+{
+    return QStringList();
+}
+
 bool ProjectNode::removeSubProject(const QString &proFilePath)
 {
     Q_UNUSED(proFilePath)
@@ -832,7 +859,7 @@ bool ProjectNode::renameFile(const QString &filePath, const QString &newFilePath
     return false;
 }
 
-bool ProjectNode::supportsAction(ProjectAction, Node *) const
+bool ProjectNode::supportsAction(ProjectAction, const Node *) const
 {
     return false;
 }
@@ -843,34 +870,49 @@ bool ProjectNode::deploysFolder(const QString &folder) const
     return false;
 }
 
-/*!
-  \function bool ProjectNode::runConfigurations() const
-
-  Returns a list of \c RunConfiguration suitable for this node.
-  */
-QList<RunConfiguration *> ProjectNode::runConfigurations() const
+ProjectNode *ProjectNode::projectNode(const Utils::FilePath &file) const
 {
-    return QList<RunConfiguration *>();
-}
-
-ProjectNode *ProjectNode::projectNode(const Utils::FileName &file) const
-{
-    for (Node *node : m_nodes) {
-        if (ProjectNode *pnode = node->asProjectNode())
+    for (const std::unique_ptr<Node> &n: m_nodes) {
+        if (ProjectNode *pnode = n->asProjectNode())
             if (pnode->filePath() == file)
                 return pnode;
     }
     return nullptr;
 }
 
+QVariant ProjectNode::data(Core::Id role) const
+{
+    Q_UNUSED(role);
+    return QVariant();
+}
+
+bool ProjectNode::setData(Core::Id role, const QVariant &value) const
+{
+    Q_UNUSED(role);
+    Q_UNUSED(value);
+    return false;
+}
+
 bool FolderNode::isEmpty() const
 {
-    return m_nodes.isEmpty();
+    return m_nodes.size() == 0;
+}
+
+void FolderNode::handleSubTreeChanged(FolderNode *node)
+{
+    if (FolderNode *parent = parentFolderNode())
+        parent->handleSubTreeChanged(node);
+}
+
+void FolderNode::setShowWhenEmpty(bool showWhenEmpty)
+{
+    m_showWhenEmpty = showWhenEmpty;
 }
 
 ContainerNode::ContainerNode(Project *project)
-    : FolderNode(project->projectDirectory(), NodeType::Project), m_project(project)
-{}
+    : FolderNode(project->projectDirectory()), m_project(project)
+{
+}
 
 QString ContainerNode::displayName() const
 {
@@ -887,9 +929,9 @@ QString ContainerNode::displayName() const
     return name;
 }
 
-bool ContainerNode::supportsAction(ProjectAction action, Node *node) const
+bool ContainerNode::supportsAction(ProjectAction action, const Node *node) const
 {
-    Node *rootNode = m_project->rootProjectNode();
+    const Node *rootNode = m_project->rootProjectNode();
     return rootNode && rootNode->supportsAction(action, node);
 }
 
@@ -898,4 +940,14 @@ ProjectNode *ContainerNode::rootProjectNode() const
     return m_project->rootProjectNode();
 }
 
+void ContainerNode::removeAllChildren()
+{
+    m_nodes.clear();
+}
+
+void ContainerNode::handleSubTreeChanged(FolderNode *node)
+{
+    m_project->handleSubTreeChanged(node);
+}
+
 } // namespace ProjectExplorer
