@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,40 +9,37 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "runsettingspropertiespage.h"
 
+#include "addrunconfigdialog.h"
 #include "buildstepspage.h"
 #include "deployconfiguration.h"
-#include "deployconfigurationmodel.h"
-#include "runconfigurationmodel.h"
 #include "runconfiguration.h"
 #include "target.h"
-#include "project.h"
+#include "projectconfigurationmodel.h"
+#include "session.h"
 
 #include <extensionsystem/pluginmanager.h>
 #include <projectexplorer/projectexplorer.h>
 #include <projectexplorer/buildmanager.h>
 #include <utils/algorithm.h>
 #include <utils/qtcassert.h>
+#include <utils/stringutils.h>
+#include <utils/utilsicons.h>
 
 #include <QVariant>
 #include <QAction>
@@ -61,83 +58,27 @@ namespace Internal {
 
 struct FactoryAndId
 {
-    ProjectExplorer::IRunConfigurationFactory *factory;
+    RunConfigurationFactory *factory;
     Core::Id id;
 };
 
-class DeployFactoryAndId
-{
-public:
-    ProjectExplorer::DeployConfigurationFactory *factory;
-    Core::Id id;
-};
-
-
 } // namespace Internal
 } // namespace ProjectExplorer
 
 Q_DECLARE_METATYPE(ProjectExplorer::Internal::FactoryAndId)
-Q_DECLARE_METATYPE(ProjectExplorer::Internal::DeployFactoryAndId)
 
 using namespace ProjectExplorer;
 using namespace ProjectExplorer::Internal;
 using ExtensionSystem::PluginManager;
 
-///
-/// RunSettingsPanelFactory
-///
-
-QString RunSettingsPanelFactory::id() const
-{
-    return QLatin1String(RUNSETTINGS_PANEL_ID);
-}
-
-QString RunSettingsPanelFactory::displayName() const
-{
-    return RunSettingsWidget::tr("Run Settings");
-}
-
-int RunSettingsPanelFactory::priority() const
-{
-    return 20;
-}
-
-bool RunSettingsPanelFactory::supports(Target *target)
-{
-    Q_UNUSED(target);
-    return true;
-}
-
-PropertiesPanel *RunSettingsPanelFactory::createPanel(Target *target)
-{
-    PropertiesPanel *panel = new PropertiesPanel;
-    QWidget *w = new QWidget();
-    QVBoxLayout *l = new QVBoxLayout(w);
-    QWidget *b = new RunSettingsWidget(target);
-    l->addWidget(b);
-    l->addSpacerItem(new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding));
-    l->setContentsMargins(QMargins());
-    panel->setWidget(w);
-    panel->setIcon(QIcon(QLatin1String(":/projectexplorer/images/RunSettings.png")));
-    panel->setDisplayName(RunSettingsWidget::tr("Run Settings"));
-    return panel;
-}
-
 ///
 /// RunSettingsWidget
 ///
 
-RunSettingsWidget::RunSettingsWidget(Target *target)
-    : m_target(target),
-      m_runConfigurationsModel(new RunConfigurationModel(target, this)),
-      m_deployConfigurationModel(new DeployConfigurationModel(target, this)),
-      m_runConfigurationWidget(0),
-      m_runConfiguration(0),
-      m_runLayout(0),
-      m_deployConfigurationWidget(0),
-      m_deployLayout(0),
-      m_deploySteps(0),
-      m_ignoreChange(false)
+RunSettingsWidget::RunSettingsWidget(Target *target) :
+    m_target(target),
+    m_runConfigurationsModel(new RunConfigurationModel(target, this)),
+    m_deployConfigurationModel(new DeployConfigurationModel(target, this))
 {
     Q_ASSERT(m_target);
 
@@ -146,27 +87,26 @@ RunSettingsWidget::RunSettingsWidget(Target *target)
     m_removeDeployToolButton = new QPushButton(tr("Remove"), this);
     m_renameDeployButton = new QPushButton(tr("Rename..."), this);
 
-    QWidget *deployWidget = new QWidget(this);
+    auto deployWidget = new QWidget(this);
 
     m_runConfigurationCombo = new QComboBox(this);
     m_runConfigurationCombo->setSizeAdjustPolicy(QComboBox::AdjustToContents);
     m_runConfigurationCombo->setMinimumContentsLength(15);
 
-    m_addRunToolButton = new QPushButton(tr("Add"), this);
+    m_addRunToolButton = new QPushButton(tr("Add..."), this);
     m_removeRunToolButton = new QPushButton(tr("Remove"), this);
     m_renameRunButton = new QPushButton(tr("Rename..."), this);
+    m_cloneRunButton = new QPushButton(tr("Clone..."), this);
 
-    QSpacerItem *spacer1 =
-        new QSpacerItem(10, 10, QSizePolicy::Expanding, QSizePolicy::Minimum);
-    QSpacerItem *spacer2 =
-        new QSpacerItem(10, 10, QSizePolicy::Minimum, QSizePolicy::Expanding);
+    auto spacer1 = new QSpacerItem(10, 10, QSizePolicy::Expanding, QSizePolicy::Minimum);
+    auto spacer2 = new QSpacerItem(10, 10, QSizePolicy::Minimum, QSizePolicy::Expanding);
 
-    QWidget *runWidget = new QWidget(this);
+    auto runWidget = new QWidget(this);
 
-    QLabel *deployTitle = new QLabel(tr("Deployment"), this);
-    QLabel *deployLabel = new QLabel(tr("Method:"), this);
-    QLabel *runTitle = new QLabel(tr("Run"), this);
-    QLabel *runLabel = new QLabel(tr("Run configuration:"), this);
+    auto deployTitle = new QLabel(tr("Deployment"), this);
+    auto deployLabel = new QLabel(tr("Method:"), this);
+    auto runTitle = new QLabel(tr("Run"), this);
+    auto runLabel = new QLabel(tr("Run configuration:"), this);
 
     runLabel->setBuddy(m_runConfigurationCombo);
 
@@ -195,7 +135,8 @@ RunSettingsWidget::RunSettingsWidget(Target *target)
     m_gridLayout->addWidget(m_addRunToolButton, 4, 2, 1, 1);
     m_gridLayout->addWidget(m_removeRunToolButton, 4, 3, 1, 1);
     m_gridLayout->addWidget(m_renameRunButton, 4, 4, 1, 1);
-    m_gridLayout->addItem(spacer1, 4, 5, 1, 1);
+    m_gridLayout->addWidget(m_cloneRunButton, 4, 5, 1, 1);
+    m_gridLayout->addItem(spacer1, 4, 6, 1, 1);
     m_gridLayout->addWidget(runWidget, 5, 0, 1, 6);
     m_gridLayout->addItem(spacer2, 6, 0, 1, 1);
 
@@ -219,17 +160,17 @@ RunSettingsWidget::RunSettingsWidget(Target *target)
     m_removeDeployToolButton->setEnabled(m_target->deployConfigurations().count() > 1);
     m_renameDeployButton->setEnabled(m_target->activeDeployConfiguration());
 
-    connect(m_addDeployMenu, SIGNAL(aboutToShow()),
-            this, SLOT(aboutToShowDeployMenu()));
-    connect(m_deployConfigurationCombo, SIGNAL(currentIndexChanged(int)),
-            this, SLOT(currentDeployConfigurationChanged(int)));
-    connect(m_removeDeployToolButton, SIGNAL(clicked(bool)),
-            this, SLOT(removeDeployConfiguration()));
-    connect(m_renameDeployButton, SIGNAL(clicked()),
-            this, SLOT(renameDeployConfiguration()));
+    connect(m_addDeployMenu, &QMenu::aboutToShow,
+            this, &RunSettingsWidget::aboutToShowDeployMenu);
+    connect(m_deployConfigurationCombo, QOverload<int>::of(&QComboBox::currentIndexChanged),
+            this, &RunSettingsWidget::currentDeployConfigurationChanged);
+    connect(m_removeDeployToolButton, &QAbstractButton::clicked,
+            this, &RunSettingsWidget::removeDeployConfiguration);
+    connect(m_renameDeployButton, &QAbstractButton::clicked,
+            this, &RunSettingsWidget::renameDeployConfiguration);
 
-    connect(m_target, SIGNAL(activeDeployConfigurationChanged(ProjectExplorer::DeployConfiguration*)),
-            this, SLOT(activeDeployConfigurationChanged()));
+    connect(m_target, &Target::activeDeployConfigurationChanged,
+            this, &RunSettingsWidget::activeDeployConfigurationChanged);
 
     // run part
     runWidget->setContentsMargins(0, 10, 0, 25);
@@ -237,8 +178,16 @@ RunSettingsWidget::RunSettingsWidget(Target *target)
     m_runLayout->setMargin(0);
     m_runLayout->setSpacing(5);
 
-    m_addRunMenu = new QMenu(m_addRunToolButton);
-    m_addRunToolButton->setMenu(m_addRunMenu);
+    m_disabledIcon = new QLabel;
+    m_disabledIcon->setPixmap(Utils::Icons::WARNING.pixmap());
+    m_disabledText = new QLabel;
+    auto disabledHBox = new QHBoxLayout;
+    disabledHBox->addWidget(m_disabledIcon);
+    disabledHBox->addWidget(m_disabledText);
+    disabledHBox->addStretch(255);
+
+    m_runLayout->addLayout(disabledHBox);
+
     RunConfiguration *rc = m_target->activeRunConfiguration();
     m_runConfigurationCombo->setModel(m_runConfigurationsModel);
     m_runConfigurationCombo->setCurrentIndex(
@@ -246,80 +195,46 @@ RunSettingsWidget::RunSettingsWidget(Target *target)
 
     m_removeRunToolButton->setEnabled(m_target->runConfigurations().size() > 1);
     m_renameRunButton->setEnabled(rc);
+    m_cloneRunButton->setEnabled(rc);
 
     setConfigurationWidget(rc);
 
-    connect(m_addRunMenu, SIGNAL(aboutToShow()),
-            this, SLOT(aboutToShowAddMenu()));
-    connect(m_runConfigurationCombo, SIGNAL(currentIndexChanged(int)),
-            this, SLOT(currentRunConfigurationChanged(int)));
-    connect(m_removeRunToolButton, SIGNAL(clicked(bool)),
-            this, SLOT(removeRunConfiguration()));
-    connect(m_renameRunButton, SIGNAL(clicked()),
-            this, SLOT(renameRunConfiguration()));
+    connect(m_addRunToolButton, &QAbstractButton::clicked,
+            this, &RunSettingsWidget::showAddRunConfigDialog);
+    connect(m_runConfigurationCombo, QOverload<int>::of(&QComboBox::currentIndexChanged),
+            this, &RunSettingsWidget::currentRunConfigurationChanged);
+    connect(m_removeRunToolButton, &QAbstractButton::clicked,
+            this, &RunSettingsWidget::removeRunConfiguration);
+    connect(m_renameRunButton, &QAbstractButton::clicked,
+            this, &RunSettingsWidget::renameRunConfiguration);
+    connect(m_cloneRunButton, &QAbstractButton::clicked,
+            this, &RunSettingsWidget::cloneRunConfiguration);
 
-    connect(m_target, SIGNAL(addedRunConfiguration(ProjectExplorer::RunConfiguration*)),
-            this, SLOT(updateRemoveToolButton()));
-    connect(m_target, SIGNAL(removedRunConfiguration(ProjectExplorer::RunConfiguration*)),
-            this, SLOT(updateRemoveToolButton()));
+    connect(m_target, &Target::addedRunConfiguration,
+            this, &RunSettingsWidget::updateRemoveToolButton);
+    connect(m_target, &Target::removedRunConfiguration,
+            this, &RunSettingsWidget::updateRemoveToolButton);
 
-    connect(m_target, SIGNAL(addedDeployConfiguration(ProjectExplorer::DeployConfiguration*)),
-            this, SLOT(updateRemoveToolButton()));
-    connect(m_target, SIGNAL(removedDeployConfiguration(ProjectExplorer::DeployConfiguration*)),
-            this, SLOT(updateRemoveToolButton()));
+    connect(m_target, &Target::addedDeployConfiguration,
+            this, &RunSettingsWidget::updateRemoveToolButton);
+    connect(m_target, &Target::removedDeployConfiguration,
+            this, &RunSettingsWidget::updateRemoveToolButton);
 
-    connect(m_target, SIGNAL(activeRunConfigurationChanged(ProjectExplorer::RunConfiguration*)),
-            this, SLOT(activeRunConfigurationChanged()));
+    connect(m_target, &Target::activeRunConfigurationChanged,
+            this, &RunSettingsWidget::activeRunConfigurationChanged);
 }
 
-RunSettingsWidget::~RunSettingsWidget()
+void RunSettingsWidget::showAddRunConfigDialog()
 {
-}
-
-void RunSettingsWidget::aboutToShowAddMenu()
-{
-    m_addRunMenu->clear();
-    if (m_target->activeRunConfiguration()) {
-        m_addRunMenu->addAction(tr("&Clone Selected"),
-                                this, SLOT(cloneRunConfiguration()));
-    }
-    QList<IRunConfigurationFactory *> factories =
-        ExtensionSystem::PluginManager::getObjects<IRunConfigurationFactory>();
-
-    QList<QAction *> menuActions;
-    foreach (IRunConfigurationFactory *factory, factories) {
-        QList<Core::Id> ids = factory->availableCreationIds(m_target);
-        foreach (Core::Id id, ids) {
-            QAction *action = new QAction(factory->displayNameForId(id), m_addRunMenu);
-            FactoryAndId fai;
-            fai.factory = factory;
-            fai.id = id;
-            QVariant v;
-            v.setValue(fai);
-            action->setData(v);
-            connect(action, SIGNAL(triggered()),
-                    this, SLOT(addRunConfiguration()));
-            menuActions.append(action);
-        }
-    }
-
-    Utils::sort(menuActions, [](const QAction *l, const QAction *r) {
-        return l->text() < r->text();
-    });
-    foreach (QAction *action, menuActions)
-        m_addRunMenu->addAction(action);
-}
-
-void RunSettingsWidget::addRunConfiguration()
-{
-    QAction *act = qobject_cast<QAction *>(sender());
-    if (!act)
+    AddRunConfigDialog dlg(m_target, this);
+    if (dlg.exec() != QDialog::Accepted)
         return;
-    FactoryAndId fai = act->data().value<FactoryAndId>();
-    RunConfiguration *newRC = fai.factory->create(m_target, fai.id);
+    RunConfigurationCreationInfo rci = dlg.creationInfo();
+    QTC_ASSERT(rci.id.isValid(), return);
+    RunConfiguration *newRC = rci.create(m_target);
     if (!newRC)
         return;
-    QTC_CHECK(newRC->id() == fai.id);
+    QTC_CHECK(newRC->id() == rci.id);
     m_target->addRunConfiguration(newRC);
     m_target->setActiveRunConfiguration(newRC);
     m_removeRunToolButton->setEnabled(m_target->runConfigurations().size() > 1);
@@ -328,17 +243,18 @@ void RunSettingsWidget::addRunConfiguration()
 void RunSettingsWidget::cloneRunConfiguration()
 {
     RunConfiguration* activeRunConfiguration = m_target->activeRunConfiguration();
-    IRunConfigurationFactory *factory = IRunConfigurationFactory::find(m_target,
-                                                                       activeRunConfiguration);
-    if (!factory)
-        return;
 
     //: Title of a the cloned RunConfiguration window, text of the window
-    QString name = uniqueRCName(QInputDialog::getText(this, tr("Clone Configuration"), tr("New configuration name:")));
+    QString name = uniqueRCName(
+                        QInputDialog::getText(this,
+                                              tr("Clone Configuration"),
+                                              tr("New configuration name:"),
+                                              QLineEdit::Normal,
+                                              activeRunConfiguration->displayName()));
     if (name.isEmpty())
         return;
 
-    RunConfiguration *newRc = factory->clone(m_target, activeRunConfiguration);
+    RunConfiguration *newRc = RunConfigurationFactory::clone(m_target, activeRunConfiguration);
     if (!newRc)
         return;
 
@@ -361,6 +277,7 @@ void RunSettingsWidget::removeRunConfiguration()
     m_target->removeRunConfiguration(rc);
     m_removeRunToolButton->setEnabled(m_target->runConfigurations().size() > 1);
     m_renameRunButton->setEnabled(m_target->activeRunConfiguration());
+    m_cloneRunButton->setEnabled(m_target->activeRunConfiguration());
 }
 
 void RunSettingsWidget::activeRunConfigurationChanged()
@@ -370,9 +287,10 @@ void RunSettingsWidget::activeRunConfigurationChanged()
     QModelIndex actRc = m_runConfigurationsModel->indexFor(m_target->activeRunConfiguration());
     m_ignoreChange = true;
     m_runConfigurationCombo->setCurrentIndex(actRc.row());
-    setConfigurationWidget(m_runConfigurationsModel->runConfigurationAt(actRc.row()));
+    setConfigurationWidget(qobject_cast<RunConfiguration *>(m_runConfigurationsModel->projectConfigurationAt(actRc.row())));
     m_ignoreChange = false;
     m_renameRunButton->setEnabled(m_target->activeRunConfiguration());
+    m_cloneRunButton->setEnabled(m_target->activeRunConfiguration());
 }
 
 void RunSettingsWidget::renameRunConfiguration()
@@ -398,9 +316,9 @@ void RunSettingsWidget::currentRunConfigurationChanged(int index)
     if (m_ignoreChange)
         return;
 
-    RunConfiguration *selectedRunConfiguration = 0;
+    RunConfiguration *selectedRunConfiguration = nullptr;
     if (index >= 0)
-        selectedRunConfiguration = m_runConfigurationsModel->runConfigurationAt(index);
+        selectedRunConfiguration = qobject_cast<RunConfiguration *>(m_runConfigurationsModel->projectConfigurationAt(index));
 
     if (selectedRunConfiguration == m_runConfiguration)
         return;
@@ -418,47 +336,30 @@ void RunSettingsWidget::currentDeployConfigurationChanged(int index)
     if (m_ignoreChange)
         return;
     if (index == -1)
-        m_target->setActiveDeployConfiguration(0);
+        SessionManager::setActiveDeployConfiguration(m_target, nullptr, SetActive::Cascade);
     else
-        m_target->setActiveDeployConfiguration(m_deployConfigurationModel->deployConfigurationAt(index));
+        SessionManager::setActiveDeployConfiguration(m_target,
+                                                     qobject_cast<DeployConfiguration *>(m_deployConfigurationModel->projectConfigurationAt(index)),
+                                                     SetActive::Cascade);
 }
 
 void RunSettingsWidget::aboutToShowDeployMenu()
 {
     m_addDeployMenu->clear();
-    QList<DeployConfigurationFactory *> factories = DeployConfigurationFactory::find(m_target);
-    if (factories.isEmpty())
-        return;
 
-    foreach (DeployConfigurationFactory *factory, factories) {
-        QList<Core::Id> ids = factory->availableCreationIds(m_target);
-        foreach (Core::Id id, ids) {
-            QAction *action = m_addDeployMenu->addAction(factory->displayNameForId(id));
-            DeployFactoryAndId data = { factory, id };
-            action->setData(QVariant::fromValue(data));
-            connect(action, SIGNAL(triggered()),
-                    this, SLOT(addDeployConfiguration()));
-        }
+    for (DeployConfigurationFactory *factory : DeployConfigurationFactory::find(m_target)) {
+        QAction *action = m_addDeployMenu->addAction(factory->defaultDisplayName());
+        connect(action, &QAction::triggered, [factory, this]() {
+            DeployConfiguration *newDc = factory->create(m_target);
+            if (!newDc)
+                return;
+            m_target->addDeployConfiguration(newDc);
+            SessionManager::setActiveDeployConfiguration(m_target, newDc, SetActive::Cascade);
+            m_removeDeployToolButton->setEnabled(m_target->deployConfigurations().size() > 1);
+        });
     }
 }
 
-void RunSettingsWidget::addDeployConfiguration()
-{
-    QAction *act = qobject_cast<QAction *>(sender());
-    if (!act)
-        return;
-    DeployFactoryAndId data = act->data().value<DeployFactoryAndId>();
-    if (!data.factory->canCreate(m_target, data.id))
-        return;
-    DeployConfiguration *newDc = data.factory->create(m_target, data.id);
-    if (!newDc)
-        return;
-    QTC_CHECK(!newDc || newDc->id() == data.id);
-    m_target->addDeployConfiguration(newDc);
-    m_target->setActiveDeployConfiguration(newDc);
-    m_removeDeployToolButton->setEnabled(m_target->deployConfigurations().size() > 1);
-}
-
 void RunSettingsWidget::removeDeployConfiguration()
 {
     DeployConfiguration *dc = m_target->activeDeployConfiguration();
@@ -520,9 +421,9 @@ void RunSettingsWidget::updateRemoveToolButton()
 void RunSettingsWidget::updateDeployConfiguration(DeployConfiguration *dc)
 {
     delete m_deployConfigurationWidget;
-    m_deployConfigurationWidget = 0;
+    m_deployConfigurationWidget = nullptr;
     delete m_deploySteps;
-    m_deploySteps = 0;
+    m_deploySteps = nullptr;
 
     m_ignoreChange = true;
     m_deployConfigurationCombo->setCurrentIndex(-1);
@@ -553,15 +454,18 @@ void RunSettingsWidget::setConfigurationWidget(RunConfiguration *rc)
         return;
 
     delete m_runConfigurationWidget;
-    m_runConfigurationWidget = 0;
+    m_runConfigurationWidget = nullptr;
     removeSubWidgets();
     if (!rc)
         return;
     m_runConfigurationWidget = rc->createConfigurationWidget();
     m_runConfiguration = rc;
-    if (m_runConfigurationWidget)
+    if (m_runConfigurationWidget) {
         m_runLayout->addWidget(m_runConfigurationWidget);
-
+        updateEnabledState();
+        connect(m_runConfiguration, &RunConfiguration::enabledChanged,
+                m_runConfigurationWidget, [this]() { updateEnabledState(); });
+    }
     addRunControlWidgets();
 }
 
@@ -575,7 +479,7 @@ QString RunSettingsWidget::uniqueDCName(const QString &name)
                 continue;
             dcNames.append(dc->displayName());
         }
-        result = Project::makeUnique(result, dcNames);
+        result = Utils::makeUniquelyNumbered(result, dcNames);
     }
     return result;
 }
@@ -590,28 +494,29 @@ QString RunSettingsWidget::uniqueRCName(const QString &name)
                 continue;
             rcNames.append(rc->displayName());
         }
-        result = Project::makeUnique(result, rcNames);
+        result = Utils::makeUniquelyNumbered(result, rcNames);
     }
     return result;
 }
 
 void RunSettingsWidget::addRunControlWidgets()
 {
-    foreach (IRunConfigurationAspect *aspect, m_runConfiguration->extraAspects()) {
-        ProjectExplorer::RunConfigWidget *rcw = aspect->createConfigurationWidget();
-        if (rcw)
-            addSubWidget(rcw);
+    for (ProjectConfigurationAspect *aspect : m_runConfiguration->aspects()) {
+        if (QWidget *rcw = aspect->createConfigWidget()) {
+            auto label = new QLabel(this);
+            label->setText(aspect->displayName());
+            connect(aspect, &GlobalOrProjectAspect::changed, label, [label, aspect] {
+                label->setText(aspect->displayName());
+            });
+            addSubWidget(rcw, label);
+        }
     }
 }
 
-void RunSettingsWidget::addSubWidget(RunConfigWidget *widget)
+void RunSettingsWidget::addSubWidget(QWidget *widget, QLabel *label)
 {
     widget->setContentsMargins(0, 10, 0, 0);
 
-    QLabel *label = new QLabel(this);
-    label->setText(widget->displayName());
-    connect(widget, SIGNAL(displayNameChanged(QString)),
-            label, SLOT(setText(QString)));
     QFont f = label->font();
     f.setBold(true);
     f.setPointSizeF(f.pointSizeF() * 1.2);
@@ -628,10 +533,21 @@ void RunSettingsWidget::addSubWidget(RunConfigWidget *widget)
 
 void RunSettingsWidget::removeSubWidgets()
 {
-    // foreach does not like commas in types, it's only a macro after all
-    foreach (const RunConfigItem &item, m_subWidgets) {
+    for (const RunConfigItem &item : m_subWidgets) {
         delete item.first;
         delete item.second;
     }
     m_subWidgets.clear();
 }
+
+void RunSettingsWidget::updateEnabledState()
+{
+    const bool enable = m_runConfiguration ? m_runConfiguration->isEnabled() : false;
+    const QString reason = m_runConfiguration ? m_runConfiguration->disabledReason() : QString();
+
+    m_runConfigurationWidget->setEnabled(enable);
+
+    m_disabledIcon->setVisible(!enable && !reason.isEmpty());
+    m_disabledText->setVisible(!enable && !reason.isEmpty());
+    m_disabledText->setText(reason);
+}
