@@ -28,21 +28,31 @@
 
 #include <coreplugin/coreicons.h>
 #include <coreplugin/ioutputpane.h>
+#include <texteditor/textmark.h>
 #include <utils/qtcassert.h>
 #include <utils/theme/theme.h>
+#include <utils/utilsicons.h>
 
-using namespace ProjectExplorer;
+#include <QApplication>
 
-TaskHub *m_instance = 0;
+using namespace Utils;
+
+namespace ProjectExplorer {
+
+// Task mark categories
+const char TASK_MARK_WARNING[] = "Task.Mark.Warning";
+const char TASK_MARK_ERROR[] = "Task.Mark.Error";
+
+static TaskHub *m_instance = nullptr;
 QVector<Core::Id> TaskHub::m_registeredCategories;
 
 static Core::Id categoryForType(Task::TaskType type)
 {
     switch (type) {
     case Task::Error:
-        return Constants::TASK_MARK_ERROR;
+        return TASK_MARK_ERROR;
     case Task::Warning:
-        return Constants::TASK_MARK_WARNING;
+        return TASK_MARK_WARNING;
     default:
         return Core::Id();
     }
@@ -51,19 +61,27 @@ static Core::Id categoryForType(Task::TaskType type)
 class TaskMark : public TextEditor::TextMark
 {
 public:
-    TaskMark(unsigned int id, const QString &fileName, int lineNumber, Task::TaskType type, bool visible)
-        : TextMark(fileName, lineNumber, categoryForType(type))
-        , m_id(id)
+    TaskMark(const Task &task) :
+        TextMark(task.file, task.line, categoryForType(task.type)),
+        m_id(task.taskId)
     {
-        setVisible(visible);
+        setColor(task.type == Task::Error ? Utils::Theme::ProjectExplorer_TaskError_TextMarkColor
+                                          : Utils::Theme::ProjectExplorer_TaskWarn_TextMarkColor);
+        setDefaultToolTip(task.type == Task::Error ? QApplication::translate("TaskHub", "Error")
+                                                   : QApplication::translate("TaskHub", "Warning"));
+        setPriority(task.type == Task::Error ? TextEditor::TextMark::NormalPriority
+                                             : TextEditor::TextMark::LowPriority);
+        setToolTip(task.description);
+        setIcon(task.icon);
+        setVisible(!task.icon.isNull());
     }
 
-    bool isClickable() const;
-    void clicked();
+    bool isClickable() const override;
+    void clicked() override;
 
-    void updateFileName(const QString &fileName);
-    void updateLineNumber(int lineNumber);
-    void removedFromEditor();
+    void updateFileName(const FilePath &fileName) override;
+    void updateLineNumber(int lineNumber) override;
+    void removedFromEditor() override;
 private:
     unsigned int m_id;
 };
@@ -74,10 +92,10 @@ void TaskMark::updateLineNumber(int lineNumber)
     TextMark::updateLineNumber(lineNumber);
 }
 
-void TaskMark::updateFileName(const QString &fileName)
+void TaskMark::updateFileName(const FilePath &fileName)
 {
-    TaskHub::updateTaskFileName(m_id, fileName);
-    TextMark::updateFileName(fileName);
+    TaskHub::updateTaskFileName(m_id, fileName.toString());
+    TextMark::updateFileName(FilePath::fromString(fileName.toString()));
 }
 
 void TaskMark::removedFromEditor()
@@ -99,16 +117,12 @@ TaskHub::TaskHub()
 {
     m_instance = this;
     qRegisterMetaType<ProjectExplorer::Task>("ProjectExplorer::Task");
-    qRegisterMetaType<QList<ProjectExplorer::Task> >("QList<ProjectExplorer::Task>");
-    TaskMark::setCategoryColor(Constants::TASK_MARK_ERROR,
-                               Utils::Theme::ProjectExplorer_TaskError_TextMarkColor);
-    TaskMark::setCategoryColor(Constants::TASK_MARK_WARNING,
-                               Utils::Theme::ProjectExplorer_TaskWarn_TextMarkColor);
+    qRegisterMetaType<Tasks >("Tasks");
 }
 
 TaskHub::~TaskHub()
 {
-    m_instance = 0;
+    m_instance = nullptr;
 }
 
 void TaskHub::addCategory(Core::Id categoryId, const QString &displayName, bool visible)
@@ -124,7 +138,7 @@ TaskHub *TaskHub::instance()
     return m_instance;
 }
 
-void TaskHub::addTask(Task::TaskType type, const QString &description, Core::Id category, const Utils::FileName &file, int line)
+void TaskHub::addTask(Task::TaskType type, const QString &description, Core::Id category, const Utils::FilePath &file, int line)
 {
     addTask(Task(type, description, file, line, category));
 }
@@ -133,21 +147,15 @@ void TaskHub::addTask(Task task)
 {
     QTC_ASSERT(m_registeredCategories.contains(task.category), return);
     QTC_ASSERT(!task.description.isEmpty(), return);
+    QTC_ASSERT(!task.isNull(), return);
+    QTC_ASSERT(task.m_mark.isNull(), return);
 
-    if (task.file.isEmpty())
-        task.line = -1;
-
-    if (task.line <= 0)
+    if (task.file.isEmpty() || task.line <= 0)
         task.line = -1;
     task.movedLine = task.line;
 
-    if (task.line != -1 && !task.file.isEmpty()) {
-        TaskMark *mark = new TaskMark(task.taskId, task.file.toString(), task.line,
-                                      task.type, !task.icon.isNull());
-        mark->setIcon(task.icon);
-        mark->setPriority(TextEditor::TextMark::LowPriority);
-        task.addMark(mark);
-    }
+    if ((task.options & Task::AddTextMark) && task.line != -1)
+        task.setMark(new TaskMark(task));
     emit m_instance->taskAdded(task);
 }
 
@@ -193,3 +201,4 @@ void TaskHub::requestPopup()
     emit m_instance->popupRequested(Core::IOutputPane::NoModeSwitch);
 }
 
+} // namespace ProjectExplorer
