@@ -31,13 +31,18 @@
 #include "qmakeprojectconfigwidget.h"
 #include "qmakeprojectmanagerconstants.h"
 #include "qmakenodes.h"
+#include "qmakesettings.h"
 #include "qmakestep.h"
 #include "qmakemakestep.h"
 #include "makefileparse.h"
+#include "qmakebuildconfiguration.h"
+
+#include <android/androidconstants.h>
 
 #include <coreplugin/documentmanager.h>
 #include <coreplugin/icore.h>
 
+#include <projectexplorer/buildinfo.h>
 #include <projectexplorer/buildmanager.h>
 #include <projectexplorer/buildsteplist.h>
 #include <projectexplorer/kit.h>
@@ -82,17 +87,17 @@ QString QmakeBuildConfiguration::shadowBuildDirectory(const QString &proFilePath
 
     const QString projectName = QFileInfo(proFilePath).completeBaseName();
     ProjectMacroExpander expander(proFilePath, projectName, k, suffix, buildType);
-    QString projectDir = Project::projectDirectory(FileName::fromString(proFilePath)).toString();
-    QString buildPath = expander.expand(Core::DocumentManager::buildDirectory());
+    QString projectDir = Project::projectDirectory(FilePath::fromString(proFilePath)).toString();
+    QString buildPath = expander.expand(ProjectExplorerPlugin::buildDirectoryTemplate());
     return FileUtils::resolvePath(projectDir, buildPath);
 }
 
-static FileName defaultBuildDirectory(const QString &projectPath,
+static FilePath defaultBuildDirectory(const QString &projectPath,
                                       const Kit *k,
                                       const QString &suffix,
                                       BuildConfiguration::BuildType type)
 {
-    return FileName::fromString(QmakeBuildConfiguration::shadowBuildDirectory(projectPath, k,
+    return FilePath::fromString(QmakeBuildConfiguration::shadowBuildDirectory(projectPath, k,
                                                                               suffix, type));
 }
 
@@ -123,7 +128,7 @@ QmakeBuildConfiguration::QmakeBuildConfiguration(Target *target, Core::Id id)
             this, &QmakeBuildConfiguration::qtVersionsChanged);
 }
 
-void QmakeBuildConfiguration::initialize(const BuildInfo *info)
+void QmakeBuildConfiguration::initialize(const BuildInfo &info)
 {
     BuildConfiguration::initialize(info);
 
@@ -135,31 +140,38 @@ void QmakeBuildConfiguration::initialize(const BuildInfo *info)
     BuildStepList *cleanSteps = stepList(ProjectExplorer::Constants::BUILDSTEPS_CLEAN);
     cleanSteps->appendStep(new QmakeMakeStep(cleanSteps));
 
-    const auto *qmakeInfo = static_cast<const QmakeBuildInfo *>(info);
-    BaseQtVersion *version = QtKitInformation::qtVersion(target()->kit());
+    const QmakeExtraBuildInfo qmakeExtra = info.extraInfo.value<QmakeExtraBuildInfo>();
+    BaseQtVersion *version = QtKitAspect::qtVersion(target()->kit());
 
     BaseQtVersion::QmakeBuildConfigs config = version->defaultBuildConfig();
-    if (qmakeInfo->buildType == BuildConfiguration::Debug)
+    if (info.buildType == BuildConfiguration::Debug)
         config |= BaseQtVersion::DebugBuild;
     else
         config &= ~BaseQtVersion::DebugBuild;
 
-    QString additionalArguments = qmakeInfo->additionalArguments;
+    QString additionalArguments = qmakeExtra.additionalArguments;
     if (!additionalArguments.isEmpty())
         qmakeStep->setUserArguments(additionalArguments);
-    qmakeStep->setLinkQmlDebuggingLibrary(qmakeInfo->config.linkQmlDebuggingQQ2);
-    qmakeStep->setSeparateDebugInfo(qmakeInfo->config.separateDebugInfo);
-    qmakeStep->setUseQtQuickCompiler(qmakeInfo->config.useQtQuickCompiler);
+    qmakeStep->setLinkQmlDebuggingLibrary(qmakeExtra.config.linkQmlDebuggingQQ2);
+    qmakeStep->setSeparateDebugInfo(qmakeExtra.config.separateDebugInfo);
+    qmakeStep->setUseQtQuickCompiler(qmakeExtra.config.useQtQuickCompiler);
 
     setQMakeBuildConfiguration(config);
 
-    FileName directory = qmakeInfo->buildDirectory;
+    FilePath directory = info.buildDirectory;
     if (directory.isEmpty()) {
         directory = defaultBuildDirectory(target()->project()->projectFilePath().toString(),
-                                          target()->kit(), qmakeInfo->displayName, buildType());
+                                          target()->kit(), info.displayName, buildType());
     }
 
     setBuildDirectory(directory);
+
+    if (DeviceTypeKitAspect::deviceTypeId(target()->kit())
+            == Android::Constants::ANDROID_DEVICE_TYPE) {
+        buildSteps->appendStep(Android::Constants::ANDROID_PACKAGE_INSTALLATION_STEP_ID);
+        buildSteps->appendStep(Android::Constants::ANDROID_BUILD_APK_ID);
+    }
+
     updateCacheAndEmitEnvironmentChanged();
 }
 
@@ -199,13 +211,13 @@ void QmakeBuildConfiguration::kitChanged()
 
 void QmakeBuildConfiguration::toolChainUpdated(ToolChain *tc)
 {
-    if (ToolChainKitInformation::toolChain(target()->kit(), ProjectExplorer::Constants::CXX_LANGUAGE_ID) == tc)
+    if (ToolChainKitAspect::toolChain(target()->kit(), ProjectExplorer::Constants::CXX_LANGUAGE_ID) == tc)
         emitProFileEvaluateNeeded();
 }
 
 void QmakeBuildConfiguration::qtVersionsChanged(const QList<int> &,const QList<int> &, const QList<int> &changed)
 {
-    if (changed.contains(QtKitInformation::qtVersionId(target()->kit())))
+    if (changed.contains(QtKitAspect::qtVersionId(target()->kit())))
         emitProFileEvaluateNeeded();
 }
 
@@ -254,7 +266,8 @@ bool QmakeBuildConfiguration::isShadowBuild() const
 
 QString QmakeBuildConfiguration::makefile() const
 {
-    return static_cast<QmakeProject *>(target()->project())->rootProFile()->makefile();
+    auto rootNode = dynamic_cast<QmakeProFileNode *>(target()->project()->rootProjectNode());
+    return rootNode ? rootNode->makefile() : QString();
 }
 
 BaseQtVersion::QmakeBuildConfigs QmakeBuildConfiguration::qmakeBuildConfiguration() const
@@ -281,6 +294,23 @@ void QmakeBuildConfiguration::emitProFileEvaluateNeeded()
         static_cast<QmakeProject *>(p)->scheduleAsyncUpdate();
 }
 
+QString QmakeBuildConfiguration::unalignedBuildDirWarning()
+{
+    return tr("The build directory should be at the same level as the source directory.");
+}
+
+bool QmakeBuildConfiguration::isBuildDirAtSafeLocation(const QString &sourceDir,
+                                                       const QString &buildDir)
+{
+    return buildDir.count('/') == sourceDir.count('/');
+}
+
+bool QmakeBuildConfiguration::isBuildDirAtSafeLocation() const
+{
+    return isBuildDirAtSafeLocation(project()->projectDirectory().toString(),
+                                    buildDirectory().toString());
+}
+
 void QmakeBuildConfiguration::emitQMakeBuildConfigurationChanged()
 {
     emit qmakeBuildConfigurationChanged();
@@ -289,7 +319,7 @@ void QmakeBuildConfiguration::emitQMakeBuildConfigurationChanged()
 QStringList QmakeBuildConfiguration::configCommandLineArguments() const
 {
     QStringList result;
-    BaseQtVersion *version = QtKitInformation::qtVersion(target()->kit());
+    BaseQtVersion *version = QtKitAspect::qtVersion(target()->kit());
     BaseQtVersion::QmakeBuildConfigs defaultBuildConfiguration =
             version ? version->defaultBuildConfig() : BaseQtVersion::QmakeBuildConfigs(BaseQtVersion::DebugBuild | BaseQtVersion::BuildAll);
     BaseQtVersion::QmakeBuildConfigs userBuildConfiguration = m_qmakeBuildConfiguration;
@@ -352,13 +382,13 @@ QmakeBuildConfiguration::MakefileState QmakeBuildConfiguration::compareToImportF
         return MakefileMissing;
     }
 
-    BaseQtVersion *version = QtKitInformation::qtVersion(target()->kit());
+    BaseQtVersion *version = QtKitAspect::qtVersion(target()->kit());
     if (!version) {
         qCDebug(logs) << "**No qt version in kit";
         return MakefileForWrongProject;
     }
 
-    const Utils::FileName projectPath =
+    const Utils::FilePath projectPath =
             m_subNodeBuild ? m_subNodeBuild->filePath() : qs->project()->projectFilePath();
     if (parse.srcProFile() != projectPath.toString()) {
         qCDebug(logs) << "**Different profile used to generate the Makefile:"
@@ -394,16 +424,17 @@ QmakeBuildConfiguration::MakefileState QmakeBuildConfiguration::compareToImportF
     // This copies the settings from userArgs to actualArgs (minus some we
     // are not interested in), splitting them up into individual strings:
     extractSpecFromArguments(&userArgs, workingDirectory, version, &actualArgs);
-    FileName actualSpec = qs->mkspec();
+    const QString actualSpec = qs->mkspec();
 
     QString qmakeArgs = parse.unparsedArguments();
     QStringList parsedArgs;
-    FileName parsedSpec = extractSpecFromArguments(&qmakeArgs, workingDirectory, version, &parsedArgs);
+    QString parsedSpec =
+            extractSpecFromArguments(&qmakeArgs, workingDirectory, version, &parsedArgs);
 
     qCDebug(logs) << "  Actual args:" << actualArgs;
     qCDebug(logs) << "  Parsed args:" << parsedArgs;
-    qCDebug(logs) << "  Actual spec:" << actualSpec.toString();
-    qCDebug(logs) << "  Parsed spec:" << parsedSpec.toString();
+    qCDebug(logs) << "  Actual spec:" << actualSpec;
+    qCDebug(logs) << "  Parsed spec:" << parsedSpec;
     qCDebug(logs) << "  Actual config:" << qs->deducedArguments();
     qCDebug(logs) << "  Parsed config:" << parse.config();
 
@@ -444,8 +475,8 @@ QmakeBuildConfiguration::MakefileState QmakeBuildConfiguration::compareToImportF
     }
     // Actual spec is the default one
 //                    qDebug() << "AS vs VS" << actualSpec << version->mkspec();
-    if ((actualSpec == version->mkspec() || actualSpec == FileName::fromLatin1("default"))
-            && (parsedSpec == version->mkspec() || parsedSpec == FileName::fromLatin1("default") || parsedSpec.isEmpty())) {
+    if ((actualSpec == version->mkspec() || actualSpec == "default")
+            && (parsedSpec == version->mkspec() || parsedSpec == "default" || parsedSpec.isEmpty())) {
         qCDebug(logs) << "**Matched specs (2)";
         return MakefileMatches;
     }
@@ -456,11 +487,11 @@ QmakeBuildConfiguration::MakefileState QmakeBuildConfiguration::compareToImportF
     return MakefileIncompatible;
 }
 
-FileName QmakeBuildConfiguration::extractSpecFromArguments(QString *args,
+QString QmakeBuildConfiguration::extractSpecFromArguments(QString *args,
                                                          const QString &directory, const BaseQtVersion *version,
                                                          QStringList *outArgs)
 {
-    FileName parsedSpec;
+    FilePath parsedSpec;
 
     bool ignoreNext = false;
     bool nextIsSpec = false;
@@ -470,7 +501,7 @@ FileName QmakeBuildConfiguration::extractSpecFromArguments(QString *args,
             ait.deleteArg();
         } else if (nextIsSpec) {
             nextIsSpec = false;
-            parsedSpec = FileName::fromUserInput(ait.value());
+            parsedSpec = FilePath::fromUserInput(ait.value());
             ait.deleteArg();
         } else if (ait.value() == QLatin1String("-spec") || ait.value() == QLatin1String("-platform")) {
             nextIsSpec = true;
@@ -490,11 +521,11 @@ FileName QmakeBuildConfiguration::extractSpecFromArguments(QString *args,
     }
 
     if (parsedSpec.isEmpty())
-        return FileName();
+        return {};
 
-    FileName baseMkspecDir = FileName::fromUserInput(
+    FilePath baseMkspecDir = FilePath::fromUserInput(
             version->qmakeProperty("QT_HOST_DATA") + QLatin1String("/mkspecs"));
-    baseMkspecDir = FileName::fromString(baseMkspecDir.toFileInfo().canonicalFilePath());
+    baseMkspecDir = FilePath::fromString(baseMkspecDir.toFileInfo().canonicalFilePath());
 
     // if the path is relative it can be
     // relative to the working directory (as found in the Makefiles)
@@ -503,26 +534,26 @@ FileName QmakeBuildConfiguration::extractSpecFromArguments(QString *args,
     // for the other one we don't need to do anything
     if (parsedSpec.toFileInfo().isRelative()) {
         if (QFileInfo::exists(directory + QLatin1Char('/') + parsedSpec.toString()))
-            parsedSpec = FileName::fromUserInput(directory + QLatin1Char('/') + parsedSpec.toString());
+            parsedSpec = FilePath::fromUserInput(directory + QLatin1Char('/') + parsedSpec.toString());
         else
-            parsedSpec = FileName::fromUserInput(baseMkspecDir.toString() + QLatin1Char('/') + parsedSpec.toString());
+            parsedSpec = FilePath::fromUserInput(baseMkspecDir.toString() + QLatin1Char('/') + parsedSpec.toString());
     }
 
     QFileInfo f2 = parsedSpec.toFileInfo();
     while (f2.isSymLink()) {
-        parsedSpec = FileName::fromString(f2.symLinkTarget());
+        parsedSpec = FilePath::fromString(f2.symLinkTarget());
         f2.setFile(parsedSpec.toString());
     }
 
     if (parsedSpec.isChildOf(baseMkspecDir)) {
         parsedSpec = parsedSpec.relativeChildPath(baseMkspecDir);
     } else {
-        FileName sourceMkSpecPath = FileName::fromString(version->sourcePath().toString()
+        FilePath sourceMkSpecPath = FilePath::fromString(version->sourcePath().toString()
                                                          + QLatin1String("/mkspecs"));
         if (parsedSpec.isChildOf(sourceMkSpecPath))
             parsedSpec = parsedSpec.relativeChildPath(sourceMkSpecPath);
     }
-    return parsedSpec;
+    return parsedSpec.toString();
 }
 
 bool QmakeBuildConfiguration::isEnabled() const
@@ -554,46 +585,61 @@ QmakeBuildConfigurationFactory::QmakeBuildConfigurationFactory()
     registerBuildConfiguration<QmakeBuildConfiguration>(Constants::QMAKE_BC_ID);
     setSupportedProjectType(Constants::QMAKEPROJECT_ID);
     setSupportedProjectMimeTypeName(Constants::PROFILE_MIMETYPE);
+    setIssueReporter([](Kit *k, const QString &projectPath, const QString &buildDir) {
+        QtSupport::BaseQtVersion *version = QtSupport::QtKitAspect::qtVersion(k);
+        Tasks issues;
+        if (version)
+            issues << version->reportIssues(projectPath, buildDir);
+        if (QmakeSettings::warnAgainstUnalignedBuildDir()
+                && !QmakeBuildConfiguration::isBuildDirAtSafeLocation(
+                    QFileInfo(projectPath).absoluteDir().path(), QDir(buildDir).absolutePath())) {
+            issues.append(Task(Task::Warning, QmakeBuildConfiguration::unalignedBuildDirWarning(),
+                               Utils::FilePath(), -1,
+                               ProjectExplorer::Constants::TASK_CATEGORY_BUILDSYSTEM));
+        }
+        return issues;
+    });
 }
 
-QmakeBuildInfo *QmakeBuildConfigurationFactory::createBuildInfo(const Kit *k,
-                                                              const QString &projectPath,
-                                                              BuildConfiguration::BuildType type) const
+BuildInfo QmakeBuildConfigurationFactory::createBuildInfo(const Kit *k,
+                                                          const QString &projectPath,
+                                                          BuildConfiguration::BuildType type) const
 {
-    BaseQtVersion *version = QtKitInformation::qtVersion(k);
-    auto *info = new QmakeBuildInfo(this);
+    BaseQtVersion *version = QtKitAspect::qtVersion(k);
+    QmakeExtraBuildInfo extraInfo;
+    BuildInfo info(this);
     QString suffix;
     if (type == BuildConfiguration::Release) {
         //: The name of the release build configuration created by default for a qmake project.
-        info->displayName = tr("Release");
+        info.displayName = tr("Release");
         //: Non-ASCII characters in directory suffix may cause build issues.
         suffix = tr("Release", "Shadow build directory suffix");
         if (version && version->isQtQuickCompilerSupported())
-            info->config.useQtQuickCompiler = true;
+            extraInfo.config.useQtQuickCompiler = true;
     } else {
         if (type == BuildConfiguration::Debug) {
             //: The name of the debug build configuration created by default for a qmake project.
-            info->displayName = tr("Debug");
+            info.displayName = tr("Debug");
             //: Non-ASCII characters in directory suffix may cause build issues.
             suffix = tr("Debug", "Shadow build directory suffix");
         } else if (type == BuildConfiguration::Profile) {
             //: The name of the profile build configuration created by default for a qmake project.
-            info->displayName = tr("Profile");
+            info.displayName = tr("Profile");
             //: Non-ASCII characters in directory suffix may cause build issues.
             suffix = tr("Profile", "Shadow build directory suffix");
-            info->config.separateDebugInfo = true;
+            extraInfo.config.separateDebugInfo = true;
             if (version && version->isQtQuickCompilerSupported())
-                info->config.useQtQuickCompiler = true;
+                extraInfo.config.useQtQuickCompiler = true;
         }
         if (version && version->isQmlDebuggingSupported())
-            info->config.linkQmlDebuggingQQ2 = true;
+            extraInfo.config.linkQmlDebuggingQQ2 = true;
     }
-    info->typeName = info->displayName;
-    // Leave info->buildDirectory unset;
-    info->kitId = k->id();
+    info.typeName = info.displayName;
+    // Leave info.buildDirectory unset;
+    info.kitId = k->id();
 
     // check if this project is in the source directory:
-    FileName projectFilePath = FileName::fromString(projectPath);
+    FilePath projectFilePath = FilePath::fromString(projectPath);
     if (version && version->isInSourceDirectory(projectFilePath)) {
         // assemble build directory
         QString projectDirectory = projectFilePath.toFileInfo().absolutePath();
@@ -602,11 +648,12 @@ QmakeBuildInfo *QmakeBuildConfigurationFactory::createBuildInfo(const Kit *k,
         QString qtBuildDir = version->qmakeProperty("QT_INSTALL_PREFIX");
         QString absoluteBuildPath = QDir::cleanPath(qtBuildDir + QLatin1Char('/') + relativeProjectPath);
 
-        info->buildDirectory = FileName::fromString(absoluteBuildPath);
+        info.buildDirectory = FilePath::fromString(absoluteBuildPath);
     } else {
-        info->buildDirectory = defaultBuildDirectory(projectPath, k, suffix, type);
+        info.buildDirectory = defaultBuildDirectory(projectPath, k, suffix, type);
     }
-    info->buildType = type;
+    info.buildType = type;
+    info.extraInfo = QVariant::fromValue(extraInfo);
     return info;
 }
 
@@ -619,27 +666,27 @@ static QList<BuildConfiguration::BuildType> availableBuildTypes(const BaseQtVers
     return types;
 }
 
-QList<BuildInfo *> QmakeBuildConfigurationFactory::availableBuilds(const Target *parent) const
+QList<BuildInfo> QmakeBuildConfigurationFactory::availableBuilds(const Target *parent) const
 {
-    QList<BuildInfo *> result;
+    QList<BuildInfo> result;
 
     const QString projectFilePath = parent->project()->projectFilePath().toString();
 
     foreach (BuildConfiguration::BuildType buildType,
-             availableBuildTypes(QtKitInformation::qtVersion(parent->kit()))) {
-        QmakeBuildInfo *info = createBuildInfo(parent->kit(), projectFilePath, buildType);
-        info->displayName.clear(); // ask for a name
-        info->buildDirectory.clear(); // This depends on the displayName
+             availableBuildTypes(QtKitAspect::qtVersion(parent->kit()))) {
+        BuildInfo info = createBuildInfo(parent->kit(), projectFilePath, buildType);
+        info.displayName.clear(); // ask for a name
+        info.buildDirectory.clear(); // This depends on the displayName
         result << info;
     }
 
     return result;
 }
 
-QList<BuildInfo *> QmakeBuildConfigurationFactory::availableSetups(const Kit *k, const QString &projectPath) const
+QList<BuildInfo> QmakeBuildConfigurationFactory::availableSetups(const Kit *k, const QString &projectPath) const
 {
-    QList<BuildInfo *> result;
-    BaseQtVersion *qtVersion = QtKitInformation::qtVersion(k);
+    QList<BuildInfo> result;
+    BaseQtVersion *qtVersion = QtKitAspect::qtVersion(k);
     if (!qtVersion || !qtVersion->isValid())
         return result;
 
@@ -668,7 +715,7 @@ void QmakeBuildConfiguration::addToEnvironment(Environment &env) const
 void QmakeBuildConfiguration::setupBuildEnvironment(Kit *k, Environment &env)
 {
     prependCompilerPathToEnvironment(k, env);
-    const BaseQtVersion *qt = QtKitInformation::qtVersion(k);
+    const BaseQtVersion *qt = QtKitAspect::qtVersion(k);
     if (qt && !qt->binPath().isEmpty())
         env.prependOrSetPath(qt->binPath().toString());
 }
@@ -676,11 +723,11 @@ void QmakeBuildConfiguration::setupBuildEnvironment(Kit *k, Environment &env)
 QmakeBuildConfiguration::LastKitState::LastKitState() = default;
 
 QmakeBuildConfiguration::LastKitState::LastKitState(Kit *k)
-    : m_qtVersion(QtKitInformation::qtVersionId(k)),
-      m_sysroot(SysRootKitInformation::sysRoot(k).toString()),
-      m_mkspec(QmakeKitInformation::mkspec(k).toString())
+    : m_qtVersion(QtKitAspect::qtVersionId(k)),
+      m_sysroot(SysRootKitAspect::sysRoot(k).toString()),
+      m_mkspec(QmakeKitAspect::mkspec(k))
 {
-    ToolChain *tc = ToolChainKitInformation::toolChain(k, ProjectExplorer::Constants::CXX_LANGUAGE_ID);
+    ToolChain *tc = ToolChainKitAspect::toolChain(k, ProjectExplorer::Constants::CXX_LANGUAGE_ID);
     m_toolchain = tc ? tc->id() : QByteArray();
 }
 
