@@ -35,6 +35,13 @@
 #include <utils/qtcassert.h>
 #include <utils/stringutils.h>
 
+#include <android/androidconstants.h>
+#include <ios/iosconstants.h>
+
+#include <QJsonDocument>
+#include <QJsonObject>
+#include <QJsonParseError>
+
 using namespace ProjectExplorer;
 using namespace Utils;
 
@@ -46,7 +53,7 @@ namespace QmakeProjectManager {
   */
 
 QmakePriFileNode::QmakePriFileNode(QmakeProject *project, QmakeProFileNode *qmakeProFileNode,
-                                   const FileName &filePath, QmakePriFile *pf) :
+                                   const FilePath &filePath, QmakePriFile *pf) :
     ProjectNode(filePath),
     m_project(project),
     m_qmakeProFileNode(qmakeProFileNode),
@@ -106,7 +113,7 @@ bool QmakePriFileNode::supportsAction(ProjectAction action, const Node *node) co
             return !(pro && pro->knowsFile(node->filePath()));
 
         bool addExistingFiles = true;
-        if (node->nodeType() == NodeType::VirtualFolder) {
+        if (node->isVirtualFolderType()) {
             // A virtual folder, we do what the projectexplorer does
             const FolderNode *folder = node->asFolderNode();
             if (folder) {
@@ -126,21 +133,13 @@ bool QmakePriFileNode::supportsAction(ProjectAction action, const Node *node) co
         break;
     }
     case ProjectType::SubDirsTemplate:
-        if (action == AddSubProject || action == RemoveSubProject)
+        if (action == AddSubProject || action == AddExistingProject)
             return true;
         break;
     default:
         break;
     }
 
-    if (action == HasSubProjectRunConfigurations) {
-        if (Target *t = m_project->activeTarget())  {
-            auto canRunForNode = [node](RunConfiguration *rc) { return rc->canRunForNode(node); };
-            if (Utils::anyOf(t->runConfigurations(), canRunForNode))
-                return true;
-        }
-    }
-
     return false;
 }
 
@@ -162,10 +161,36 @@ bool QmakePriFileNode::removeSubProject(const QString &proFilePath)
     return pri ? pri->removeSubProjects(proFilePath) : false;
 }
 
+QStringList QmakePriFileNode::subProjectFileNamePatterns() const
+{
+    return QStringList("*.pro");
+}
+
 bool QmakePriFileNode::addFiles(const QStringList &filePaths, QStringList *notAdded)
 {
     QmakePriFile *pri = priFile();
-    return pri ? pri->addFiles(filePaths, notAdded) : false;
+    if (!pri)
+        return false;
+    QList<Node *> matchingNodes = findNodes([filePaths](const Node *n) {
+        return n->asFileNode() && filePaths.contains(n->filePath().toString());
+    });
+    matchingNodes = filtered(matchingNodes, [](const Node *n) {
+        for (const Node *parent = n->parentFolderNode(); parent;
+             parent = parent->parentFolderNode()) {
+            if (dynamic_cast<const ResourceEditor::ResourceTopLevelNode *>(parent))
+                return false;
+        }
+        return true;
+    });
+    QStringList alreadyPresentFiles = transform<QStringList>(matchingNodes,
+            [](const Node *n) { return n->filePath().toString(); });
+    alreadyPresentFiles.removeDuplicates();
+    QStringList actualFilePaths = filePaths;
+    for (const QString &e : alreadyPresentFiles)
+        actualFilePaths.removeOne(e);
+    if (notAdded)
+        *notAdded = alreadyPresentFiles;
+    return pri->addFiles(actualFilePaths, notAdded);
 }
 
 bool QmakePriFileNode::removeFiles(const QStringList &filePaths, QStringList *notRemoved)
@@ -198,36 +223,153 @@ FolderNode::AddNewInformation QmakePriFileNode::addNewInformation(const QStringL
     return FolderNode::AddNewInformation(filePath().fileName(), context && context->parentProjectNode() == this ? 120 : 90);
 }
 
-QmakeProFileNode *QmakeProFileNode::findProFileFor(const FileName &fileName) const
-{
-    if (fileName == filePath())
-        return const_cast<QmakeProFileNode *>(this);
-    for (Node *node : nodes()) {
-        if (QmakeProFileNode *qmakeProFileNode = dynamic_cast<QmakeProFileNode *>(node))
-            if (QmakeProFileNode *result = qmakeProFileNode->findProFileFor(fileName))
-                return result;
-    }
-    return nullptr;
-}
-
 /*!
   \class QmakeProFileNode
   Implements abstract ProjectNode class
   */
-QmakeProFileNode::QmakeProFileNode(QmakeProject *project, const FileName &filePath, QmakeProFile *pf) :
+QmakeProFileNode::QmakeProFileNode(QmakeProject *project, const FilePath &filePath, QmakeProFile *pf) :
     QmakePriFileNode(project, this, filePath, pf)
-{ }
+{
+    setIsProduct();
+}
 
 bool QmakeProFileNode::showInSimpleTree() const
 {
     return showInSimpleTree(projectType()) || m_project->rootProjectNode() == this;
 }
 
+QString QmakeProFileNode::buildKey() const
+{
+    return filePath().toString();
+}
+
+bool QmakeProFileNode::parseInProgress() const
+{
+    QmakeProjectManager::QmakeProFile *pro = proFile();
+    return !pro || pro->parseInProgress();
+}
+
+bool QmakeProFileNode::validParse() const
+{
+    QmakeProjectManager::QmakeProFile *pro = proFile();
+    return pro && pro->validParse();
+}
+
+QStringList QmakeProFileNode::targetApplications() const
+{
+    QStringList apps;
+    if (includedInExactParse() && projectType() == ProjectType::ApplicationTemplate) {
+        const QString target = targetInformation().target;
+        if (target.startsWith("lib") && target.endsWith(".so"))
+            apps << target.mid(3, target.lastIndexOf('.') - 3);
+        else
+            apps << target;
+    }
+    return apps;
+}
+
+QVariant QmakeProFileNode::data(Core::Id role) const
+{
+    if (role == Android::Constants::AndroidPackageSourceDir)
+        return singleVariableValue(Variable::AndroidPackageSourceDir);
+    if (role == Android::Constants::AndroidDeploySettingsFile)
+        return singleVariableValue(Variable::AndroidDeploySettingsFile);
+    if (role == Android::Constants::AndroidExtraLibs)
+        return variableValue(Variable::AndroidExtraLibs);
+    if (role == Android::Constants::AndroidArch)
+        return singleVariableValue(Variable::AndroidArch);
+    if (role == Android::Constants::AndroidSoLibPath) {
+        TargetInformation info = targetInformation();
+        QStringList res = {info.buildDir.toString()};
+        Utils::FilePath destDir = info.destDir;
+        if (!destDir.isEmpty()) {
+            if (destDir.toFileInfo().isRelative())
+                destDir = Utils::FilePath::fromString(QDir::cleanPath(info.buildDir.toString()
+                                                                      + '/' + destDir.toString()));
+            res.append(destDir.toString());
+        }
+        res.removeDuplicates();
+        return res;
+    }
+
+    if (role == Android::Constants::AndroidTargets)
+        return {};
+
+    if (role == Ios::Constants::IosTarget) {
+        const TargetInformation info = targetInformation();
+        if (info.valid)
+            return info.target;
+    }
+
+    if (role == Ios::Constants::IosBuildDir) {
+        const TargetInformation info = targetInformation();
+        if (info.valid)
+            return info.buildDir.toString();
+    }
+
+    QTC_CHECK(false);
+    return {};
+}
+
+bool QmakeProFileNode::setData(Core::Id role, const QVariant &value) const
+{
+    QmakeProFile *pro = proFile();
+    if (!pro)
+        return false;
+
+    const QString arch = pro->singleVariableValue(Variable::AndroidArch);
+    const QString scope = "contains(ANDROID_TARGET_ARCH," + arch + ')';
+    auto flags = QmakeProjectManager::Internal::ProWriter::ReplaceValues
+               | QmakeProjectManager::Internal::ProWriter::MultiLine;
+
+    if (role == Android::Constants::AndroidExtraLibs)
+        return pro->setProVariable("ANDROID_EXTRA_LIBS", value.toStringList(), scope, flags);
+    if (role == Android::Constants::AndroidPackageSourceDir)
+        return pro->setProVariable("ANDROID_PACKAGE_SOURCE_DIR", {value.toString()}, scope, flags);
+
+    return false;
+}
+
 QmakeProFile *QmakeProFileNode::proFile() const
 {
     return static_cast<QmakeProFile*>(QmakePriFileNode::priFile());
 }
 
+QString QmakeProFileNode::makefile() const
+{
+    return singleVariableValue(Variable::Makefile);
+}
+
+QString QmakeProFileNode::objectsDirectory() const
+{
+    return singleVariableValue(Variable::ObjectsDir);
+}
+
+bool QmakeProFileNode::isDebugAndRelease() const
+{
+    const QStringList configValues = variableValue(Variable::Config);
+    return configValues.contains(QLatin1String("debug_and_release"));
+}
+
+bool QmakeProFileNode::isQtcRunnable() const
+{
+    const QStringList configValues = variableValue(Variable::Config);
+    return configValues.contains(QLatin1String("qtc_runnable"));
+}
+
+bool QmakeProFileNode::includedInExactParse() const
+{
+    const QmakeProFile *pro = proFile();
+    return pro && pro->includedInExactParse();
+}
+
+bool QmakeProFileNode::supportsAction(ProjectAction action, const Node *node) const
+{
+    if (action == RemoveSubProject)
+        return parentProjectNode() && !parentProjectNode()->asContainerNode();
+    return QmakePriFileNode::supportsAction(action, node);
+}
+
 FolderNode::AddNewInformation QmakeProFileNode::addNewInformation(const QStringList &files, Node *context) const
 {
     Q_UNUSED(files)
@@ -271,4 +413,23 @@ QString QmakeProFileNode::buildDir() const
     return QString();
 }
 
+FilePath QmakeProFileNode::buildDir(QmakeBuildConfiguration *bc) const
+{
+    const QmakeProFile *pro = proFile();
+    return pro ? pro->buildDir(bc) : FilePath();
+}
+
+QString QmakeProFileNode::objectExtension() const
+{
+    QStringList exts = variableValue(Variable::ObjectExt);
+    if (exts.isEmpty())
+        return HostOsInfo::isWindowsHost() ? QLatin1String(".obj") : QLatin1String(".o");
+    return exts.first();
+}
+
+TargetInformation QmakeProFileNode::targetInformation() const
+{
+    return proFile() ? proFile()->targetInformation() : TargetInformation();
+}
+
 } // namespace QmakeProjectManager
