@@ -28,12 +28,11 @@
 #include "profileeditor.h"
 #include "qmakeprojectmanager.h"
 #include "qmakenodes.h"
+#include "qmakesettings.h"
 #include "qmakestep.h"
 #include "qmakemakestep.h"
 #include "qmakebuildconfiguration.h"
 #include "desktopqmakerunconfiguration.h"
-#include "wizards/guiappwizard.h"
-#include "wizards/librarywizard.h"
 #include "wizards/simpleprojectwizard.h"
 #include "wizards/subdirsprojectwizard.h"
 #include "customwidgetwizard/customwidgetwizard.h"
@@ -54,6 +53,7 @@
 #include <projectexplorer/buildmanager.h>
 #include <projectexplorer/projectmanager.h>
 #include <projectexplorer/projecttree.h>
+#include <projectexplorer/runcontrol.h>
 #include <projectexplorer/session.h>
 #include <projectexplorer/target.h>
 
@@ -86,7 +86,7 @@ public:
     void buildStateChanged(Project *pro);
     void updateBuildFileAction();
     void disableBuildFileMenus();
-    void enableBuildFileMenus(const Utils::FileName &file);
+    void enableBuildFileMenus(const Utils::FilePath &file);
 
     QmakeManager qmakeProjectManager;
     Core::Context projectContext;
@@ -99,9 +99,13 @@ public:
 
     QmakeBuildConfigurationFactory buildConfigFactory;
     DesktopQmakeRunConfigurationFactory runConfigFactory;
+    SimpleRunWorkerFactory<SimpleTargetRunner, DesktopQmakeRunConfiguration>
+        runWorkerFactory;
 
     ProFileEditorFactory profileEditorFactory;
 
+    QmakeSettingsPage settingsPage;
+
     ExternalQtEditor *m_designerEditor{ExternalQtEditor::createDesignerEditor()};
     ExternalQtEditor *m_linguistEditor{ExternalQtEditor::createLinguistEditor()};
 
@@ -121,6 +125,8 @@ public:
     Utils::ParameterAction *m_buildFileAction = nullptr;
     QAction *m_addLibraryAction = nullptr;
     QAction *m_addLibraryActionContextMenu = nullptr;
+
+    QmakeKitAspect qmakeKitAspect;
 };
 
 QmakeProjectManagerPlugin::~QmakeProjectManagerPlugin()
@@ -140,13 +146,9 @@ bool QmakeProjectManagerPlugin::initialize(const QStringList &arguments, QString
     //create and register objects
     ProjectManager::registerProjectType<QmakeProject>(QmakeProjectManager::Constants::PROFILE_MIMETYPE);
 
-    ProjectExplorer::KitManager::registerKitInformation<QmakeKitInformation>();
-
     IWizardFactory::registerFactoryCreator([] {
         return QList<IWizardFactory *> {
             new SubdirsProjectWizard,
-            new GuiAppWizard,
-            new LibraryWizard,
             new CustomWidgetWizard,
             new SimpleProjectWizard
         };
@@ -367,14 +369,14 @@ void QmakeProjectManagerPluginPrivate::updateRunQMakeAction()
 
 void QmakeProjectManagerPluginPrivate::updateContextActions()
 {
-    const Node *node = ProjectTree::findCurrentNode();
+    const Node *node = ProjectTree::currentNode();
     Project *project = ProjectTree::currentProject();
 
     const ContainerNode *containerNode = node ? node->asContainerNode() : nullptr;
     const auto *proFileNode = dynamic_cast<const QmakeProFileNode *>(containerNode ? containerNode->rootProjectNode() : node);
 
     m_addLibraryActionContextMenu->setEnabled(proFileNode);
-    auto *qmakeProject = qobject_cast<QmakeProject *>(QmakeManager::contextProject());
+    auto *qmakeProject = qobject_cast<QmakeProject *>(project);
     QmakeProFileNode *subProjectNode = nullptr;
     disableBuildFileMenus();
     if (node) {
@@ -450,7 +452,7 @@ void QmakeProjectManagerPluginPrivate::disableBuildFileMenus()
     m_buildFileContextMenu->setEnabled(false);
 }
 
-void QmakeProjectManagerPluginPrivate::enableBuildFileMenus(const Utils::FileName &file)
+void QmakeProjectManagerPluginPrivate::enableBuildFileMenus(const Utils::FilePath &file)
 {
     bool visible = false;
     bool enabled = false;
