@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -35,10 +30,9 @@
 #include "model.h"
 #include "metainforeader.h"
 
-#include <qmldesignerwarning.h>
-
 #include <utils/algorithm.h>
 #include <utils/hostosinfo.h>
+#include <coreplugin/messagebox.h>
 
 #include <QDir>
 #include <QMessageBox>
@@ -46,6 +40,7 @@
 
 #include <qmljs/qmljslink.h>
 #include <qmljs/parser/qmljsast_p.h>
+#include <qmljs/parser/qmljsengine_p.h>
 #include <qmljs/qmljsmodelmanagerinterface.h>
 
 enum { debug = false };
@@ -59,67 +54,8 @@ static bool operator<(const QFileInfo &file1, const QFileInfo &file2)
     return file1.filePath() < file2.filePath();
 }
 
-
 QT_END_NAMESPACE
 
-
-static inline bool checkIfDerivedFromItem(const QString &fileName)
-{
-    return true;
-
-    QmlJS::Snapshot snapshot;
-
-
-    QmlJS::ModelManagerInterface *modelManager = QmlJS::ModelManagerInterface::instance();
-    if (modelManager)
-        snapshot =  modelManager->snapshot();
-
-    QFile file(fileName);
-    if (!file.open(QIODevice::ReadOnly))
-        return false;
-
-    QByteArray source = file.readAll();
-    file.close();
-
-    QmlJS::Document::MutablePtr document =
-            QmlJS::Document::create(fileName.isEmpty() ?
-                                        QStringLiteral("<internal>") : fileName, QmlJS::Language::Qml);
-    document->setSource(source);
-    document->parseQml();
-
-
-    if (!document->isParsedCorrectly())
-        return false;
-
-    snapshot.insert(document);
-
-    QmlJS::Link link(snapshot, modelManager->defaultVContext(document->language(), document), QmlJS::ModelManagerInterface::instance()->builtins(document));
-
-    QList<QmlJS::DiagnosticMessage> diagnosticLinkMessages;
-    QmlJS::ContextPtr context = link(document, &diagnosticLinkMessages);
-
-    QmlJS::AST::UiObjectMember *astRootNode = 0;
-    if (QmlJS::AST::UiProgram *program = document->qmlProgram())
-        if (program->members)
-            astRootNode = program->members->member;
-
-    QmlJS::AST::UiObjectDefinition *definition = QmlJS::AST::cast<QmlJS::AST::UiObjectDefinition *>(astRootNode);
-
-    if (!definition)
-        return false;
-
-    const QmlJS::ObjectValue *objectValue = context->lookupType(document.data(), definition->qualifiedTypeNameId);
-
-    QList<const QmlJS::ObjectValue *> prototypes = QmlJS::PrototypeIterator(objectValue, context).all();
-
-    foreach (const QmlJS::ObjectValue *prototype, prototypes) {
-        if (prototype->className() == "Item")
-            return true;
-    }
-
-    return false;
-}
-
 namespace QmlDesigner {
 static const QString s_qmlFilePattern = QStringLiteral("*.qml");
 
@@ -127,7 +63,8 @@ SubComponentManager::SubComponentManager(Model *model, QObject *parent)
     : QObject(parent),
       m_model(model)
 {
-    connect(&m_watcher, SIGNAL(directoryChanged(QString)), this, SLOT(parseDirectory(QString)));
+    connect(&m_watcher, &QFileSystemWatcher::directoryChanged,
+            this, [this](const QString &path) { parseDirectory(path); });
 }
 
 void SubComponentManager::addImport(int pos, const Import &import)
@@ -193,7 +130,7 @@ void SubComponentManager::parseDirectories()
             parseDirectory(dirInfo.canonicalFilePath());
 
         foreach (const QString &subDir, QDir(QFileInfo(file).path()).entryList(QDir::Dirs | QDir::NoDot | QDir::NoDotDot)) {
-            parseDirectory(dirInfo.canonicalFilePath() + "/" + subDir, true, subDir.toUtf8());
+            parseDirectory(dirInfo.canonicalFilePath() + QLatin1String("/") + subDir, true, subDir.toUtf8());
         }
     }
 
@@ -209,6 +146,15 @@ void SubComponentManager::parseDirectories()
             foreach (const QString &path, importPaths()) {
                 QString fullUrl  = path + QLatin1Char('/') + url;
                 dirInfo = QFileInfo(fullUrl);
+
+                if (dirInfo.exists() && dirInfo.isDir()) {
+                    //### todo full qualified names QString nameSpace = import.uri();
+                    parseDirectory(dirInfo.canonicalFilePath(), false);
+                }
+
+                QString fullUrlVersion = path + QLatin1Char('/') + url + QLatin1Char('.') + import.version().split(".").constFirst();
+                dirInfo = QFileInfo(fullUrlVersion);
+
                 if (dirInfo.exists() && dirInfo.isDir()) {
                     //### todo full qualified names QString nameSpace = import.uri();
                     parseDirectory(dirInfo.canonicalFilePath(), false);
@@ -223,10 +169,10 @@ void SubComponentManager::parseDirectory(const QString &canonicalDirPath, bool a
     if (!model() || !model()->rewriterView())
         return;
 
-    QDir designerDir(canonicalDirPath + Constants::QML_DESIGNER_SUBFOLDER);
+    QDir designerDir(canonicalDirPath + QLatin1String(Constants::QML_DESIGNER_SUBFOLDER));
     if (designerDir.exists()) {
         QStringList filter;
-        filter << "*.metainfo";
+        filter << QLatin1String("*.metainfo");
         designerDir.setNameFilters(filter);
 
         QStringList metaFiles = designerDir.entryList(QDir::Files);
@@ -236,11 +182,11 @@ void SubComponentManager::parseDirectory(const QString &canonicalDirPath, bool a
                 reader.setQualifcation(qualification);
                 try {
                     reader.readMetaInfoFile(metaInfoFile.absoluteFilePath(), true);
-                } catch (InvalidMetaInfoException &e) {
+                } catch (const InvalidMetaInfoException &e) {
                     qWarning() << e.description();
-                    const QString errorMessage = metaInfoFile.absoluteFilePath() + QLatin1Char('\n') + QLatin1Char('\n') + reader.errors().join(QStringLiteral("\n"));
-                    QmlDesignerWarning::show(QCoreApplication::translate("SubComponentManager::parseDirectory", "Invalid meta info"),
-                                          errorMessage);
+                    const QString errorMessage = metaInfoFile.absoluteFilePath() + QLatin1Char('\n') + QLatin1Char('\n') + reader.errors().join(QLatin1Char('\n'));
+                    Core::AsynchronousMessageBox::warning(QCoreApplication::translate("SubComponentManager::parseDirectory", "Invalid meta info"),
+                                                           errorMessage);
                 }
             }
         }
@@ -256,8 +202,8 @@ void SubComponentManager::parseDirectory(const QString &canonicalDirPath, bool a
     dir.setNameFilters(QStringList(s_qmlFilePattern));
     dir.setFilter(QDir::Files | QDir::Readable | QDir::CaseSensitive);
 
-    QList<QFileInfo> monitoredList = watchedFiles(canonicalDirPath);
-    QList<QFileInfo> newList;
+    QFileInfoList monitoredList = watchedFiles(canonicalDirPath);
+    QFileInfoList newList;
     foreach (const QFileInfo &qmlFile, dir.entryInfoList()) {
         if (QFileInfo(m_filePath.toLocalFile()) == qmlFile) {
             // do not parse main file
@@ -294,7 +240,7 @@ void SubComponentManager::parseDirectory(const QString &canonicalDirPath, bool a
             continue;
         }
         // oldFileInfo > newFileInfo
-        parseFile(newFileInfo.filePath(), addToLibrary, qualification);
+        parseFile(newFileInfo.filePath(), addToLibrary, QString::fromUtf8(qualification));
         ++newIter;
     }
 
@@ -305,7 +251,7 @@ void SubComponentManager::parseDirectory(const QString &canonicalDirPath, bool a
     }
 
     while (newIter != newList.constEnd()) {
-        parseFile(newIter->filePath(), addToLibrary, qualification);
+        parseFile(newIter->filePath(), addToLibrary, QString::fromUtf8(qualification));
         if (debug)
             qDebug() << "m_watcher.addPath(" << newIter->filePath() << ')';
         ++newIter;
@@ -334,9 +280,9 @@ void SubComponentManager::parseFile(const QString &canonicalFilePath)
 }
 
 // dirInfo must already contain a canonical path
-QList<QFileInfo> SubComponentManager::watchedFiles(const QString &canonicalDirPath)
+QFileInfoList SubComponentManager::watchedFiles(const QString &canonicalDirPath)
 {
-    QList<QFileInfo> files;
+    QFileInfoList files;
 
     foreach (const QString &monitoredFile, m_watcher.files()) {
         QFileInfo fileInfo(monitoredFile);
@@ -350,7 +296,7 @@ void SubComponentManager::unregisterQmlFile(const QFileInfo &fileInfo, const QSt
 {
     QString componentName = fileInfo.baseName();
     if (!qualifier.isEmpty())
-        componentName = qualifier + '.' + componentName;
+        componentName = qualifier + QLatin1Char('.') + componentName;
 }
 
 
@@ -360,9 +306,6 @@ void SubComponentManager::registerQmlFile(const QFileInfo &fileInfo, const QStri
     if (!model())
         return;
 
-    if (!checkIfDerivedFromItem(fileInfo.absoluteFilePath()))
-        return;
-
     QString componentName = fileInfo.baseName();
     const QString baseComponentName = componentName;
 
@@ -371,7 +314,7 @@ void SubComponentManager::registerQmlFile(const QFileInfo &fileInfo, const QStri
         fixedQualifier = qualifier;
         if (qualifier.right(1) == QStringLiteral("."))
             fixedQualifier.chop(1); //remove last char if it is a dot
-        componentName = fixedQualifier + '.' + componentName;
+        componentName = fixedQualifier + QLatin1Char('.') + componentName;
     }
 
     if (debug)
@@ -380,18 +323,15 @@ void SubComponentManager::registerQmlFile(const QFileInfo &fileInfo, const QStri
     if (addToLibrary) {
         // Add file components to the library
         ItemLibraryEntry itemLibraryEntry;
-        itemLibraryEntry.setType(componentName.toUtf8(), -1, -1);
+        itemLibraryEntry.setType(componentName.toUtf8());
         itemLibraryEntry.setName(baseComponentName);
-        itemLibraryEntry.setCategory("QML Components");
+        itemLibraryEntry.setCategory(QLatin1String("My QML Components"));
         if (!qualifier.isEmpty()) {
             itemLibraryEntry.setRequiredImport(fixedQualifier);
         }
 
-
-        if (!model()->metaInfo().itemLibraryInfo()->containsEntry(itemLibraryEntry)) {
-
-            model()->metaInfo().itemLibraryInfo()->addEntry(itemLibraryEntry);
-        }
+        if (!model()->metaInfo().itemLibraryInfo()->containsEntry(itemLibraryEntry))
+            model()->metaInfo().itemLibraryInfo()->addEntries({itemLibraryEntry});
     }
 }
 
@@ -477,6 +417,8 @@ void SubComponentManager::update(const QUrl &filePath, const QList<Import> &impo
         addImport(ii, imports.at(ii));
     }
 
+    m_watcher.addPath(newDir.absoluteFilePath());
+
     parseDirectories();
 }
 
