@@ -51,10 +51,7 @@ bool QmlItemNode::isItemOrWindow(const ModelNode &modelNode)
     if (modelNode.metaInfo().isSubclassOf("QtQuick.Item"))
         return true;
 
-    if (modelNode.metaInfo().isSubclassOf("QtQuick.Window.Window") && modelNode.isRootNode())
-        return true;
-
-    if (modelNode.metaInfo().isSubclassOf("QtQuick.Controls.Popup"))
+    if (modelNode.metaInfo().isGraphicalItem()  && modelNode.isRootNode())
         return true;
 
     return false;
@@ -88,8 +85,15 @@ static QmlItemNode createQmlItemNodeFromSource(AbstractView *view, const QString
     return QmlItemNode();
 }
 
+QmlItemNode QmlItemNode::createQmlItemNode(AbstractView *view,
+                                           const ItemLibraryEntry &itemLibraryEntry,
+                                           const QPointF &position,
+                                           QmlItemNode parentQmlItemNode)
+{
+    return QmlItemNode(createQmlObjectNode(view, itemLibraryEntry, position, parentQmlItemNode));
+}
 
-QmlItemNode QmlItemNode::createQmlItemNode(AbstractView *view, const ItemLibraryEntry &itemLibraryEntry, const QPointF &position, QmlItemNode parentQmlItemNode)
+QmlObjectNode QmlItemNode::createQmlObjectNode(AbstractView *view, const ItemLibraryEntry &itemLibraryEntry, const QPointF &position, QmlItemNode parentQmlItemNode)
 {
     if (!parentQmlItemNode.isValid())
         parentQmlItemNode = QmlItemNode(view->rootModelNode());
@@ -98,16 +102,14 @@ QmlItemNode QmlItemNode::createQmlItemNode(AbstractView *view, const ItemLibrary
 
     NodeAbstractProperty parentProperty = parentQmlItemNode.defaultNodeAbstractProperty();
 
-    return QmlItemNode::createQmlItemNode(view, itemLibraryEntry, position, parentProperty);
+    return QmlItemNode::createQmlObjectNode(view, itemLibraryEntry, position, parentProperty);
 }
 
-QmlItemNode QmlItemNode::createQmlItemNode(AbstractView *view, const ItemLibraryEntry &itemLibraryEntry, const QPointF &position, NodeAbstractProperty parentproperty)
+QmlObjectNode QmlItemNode::createQmlObjectNode(AbstractView *view, const ItemLibraryEntry &itemLibraryEntry, const QPointF &position, NodeAbstractProperty parentproperty)
 {
-    QmlItemNode newQmlItemNode;
-
-    try {
-        RewriterTransaction transaction = view->beginRewriterTransaction(QByteArrayLiteral("QmlItemNode::createQmlItemNode"));
+    QmlObjectNode newQmlObjectNode;
 
+    view->executeInTransaction("QmlItemNode::createQmlItemNode", [=, &newQmlObjectNode, &parentproperty](){
         NodeMetaInfo metaInfo = view->model()->metaInfo(itemLibraryEntry.typeName());
 
         int minorVersion = metaInfo.minorVersion();
@@ -133,34 +135,31 @@ QmlItemNode QmlItemNode::createQmlItemNode(AbstractView *view, const ItemLibrary
                 }
             }
 
-            newQmlItemNode = QmlItemNode(view->createModelNode(itemLibraryEntry.typeName(), majorVersion, minorVersion, propertyPairList));
+            newQmlObjectNode = QmlItemNode(view->createModelNode(itemLibraryEntry.typeName(), majorVersion, minorVersion, propertyPairList));
         } else {
-            newQmlItemNode = createQmlItemNodeFromSource(view, itemLibraryEntry.qmlSource(), position);
+            newQmlObjectNode = createQmlItemNodeFromSource(view, itemLibraryEntry.qmlSource(), position);
         }
 
         if (parentproperty.isValid())
-            parentproperty.reparentHere(newQmlItemNode);
+            parentproperty.reparentHere(newQmlObjectNode);
 
-        if (!newQmlItemNode.isValid())
-            return newQmlItemNode;
+        if (!newQmlObjectNode.isValid())
+            return;
 
-        newQmlItemNode.setId(view->generateNewId(itemLibraryEntry.name()));
+        newQmlObjectNode.setId(view->generateNewId(itemLibraryEntry.name()));
 
         for (const auto &propertyBindingEntry : propertyBindingList)
-            newQmlItemNode.modelNode().bindingProperty(propertyBindingEntry.first).setExpression(propertyBindingEntry.second);
+            newQmlObjectNode.modelNode().bindingProperty(propertyBindingEntry.first).setExpression(propertyBindingEntry.second);
 
         for (const auto &propertyBindingEntry : propertyEnumList)
-            newQmlItemNode.modelNode().variantProperty(propertyBindingEntry.first).setEnumeration(propertyBindingEntry.second.toUtf8());
+            newQmlObjectNode.modelNode().variantProperty(propertyBindingEntry.first).setEnumeration(propertyBindingEntry.second.toUtf8());
 
-        Q_ASSERT(newQmlItemNode.isValid());
-    }
-    catch (const RewritingException &e) {
-        e.showException();
-    }
+        Q_ASSERT(newQmlObjectNode.isValid());
+    });
 
-    Q_ASSERT(newQmlItemNode.isValid());
+    Q_ASSERT(newQmlObjectNode.isValid());
 
-    return newQmlItemNode;
+    return newQmlObjectNode;
 }
 
 QmlItemNode QmlItemNode::createQmlItemNodeFromImage(AbstractView *view, const QString &imageName, const QPointF &position, QmlItemNode parentQmlItemNode)
@@ -177,10 +176,8 @@ QmlItemNode QmlItemNode::createQmlItemNodeFromImage(AbstractView *view, const QS
 {
     QmlItemNode newQmlItemNode;
 
-    if (parentproperty.isValid()) {
-        RewriterTransaction transaction = view->beginRewriterTransaction(QByteArrayLiteral("QmlItemNode::createQmlItemNodeFromImage"));
-
-        if (view->model()->hasNodeMetaInfo("QtQuick.Image")) {
+    if (parentproperty.isValid() && view->model()->hasNodeMetaInfo("QtQuick.Image")) {
+        view->executeInTransaction("QmlItemNode::createQmlItemNodeFromImage", [=, &newQmlItemNode, &parentproperty](){
             NodeMetaInfo metaInfo = view->model()->metaInfo("QtQuick.Image");
             QList<QPair<PropertyName, QVariant> > propertyPairList;
             propertyPairList.append({PropertyName("x"), QVariant(qRound(position.x()))});
@@ -203,8 +200,7 @@ QmlItemNode QmlItemNode::createQmlItemNodeFromImage(AbstractView *view, const QS
             newQmlItemNode.modelNode().variantProperty("fillMode").setEnumeration("Image.PreserveAspectFit");
 
             Q_ASSERT(newQmlItemNode.isValid());
-        }
-        Q_ASSERT(newQmlItemNode.isValid());
+        });
     }
 
     return newQmlItemNode;
@@ -512,11 +508,6 @@ QList<QmlModelState> QmlModelStateGroup::allStates() const
     return returnList;
 }
 
-QString QmlItemNode::simplifiedTypeName() const
-{
-    return modelNode().simplifiedTypeName();
-}
-
 uint qHash(const QmlItemNode &node)
 {
     return qHash(node.modelNode());
