@@ -26,6 +26,7 @@
 #include "qmlobjectnode.h"
 #include "qmlitemnode.h"
 #include "qmlstate.h"
+#include "qmltimelinekeyframegroup.h"
 #include "variantproperty.h"
 #include "nodeproperty.h"
 #include <invalidmodelnodeexception.h>
@@ -35,6 +36,9 @@
 #include "bindingproperty.h"
 #include "nodelistproperty.h"
 #include "nodeinstanceview.h"
+
+#include <qmltimeline.h>
+
 #ifndef QMLDESIGNER_TEST
 #include <qmldesignerplugin.h>
 #endif
@@ -48,6 +52,30 @@ void QmlObjectNode::setVariantProperty(const PropertyName &name, const QVariant
     if (!isValid())
         throw new InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
 
+    if (timelineIsActive() && currentTimeline().isRecording()) {
+        modelNode().validId();
+
+        QmlTimelineKeyframeGroup timelineFrames(currentTimeline().keyframeGroup(modelNode(), name));
+
+        Q_ASSERT(timelineFrames.isValid());
+
+        qreal frame = currentTimeline().modelNode().auxiliaryData("currentFrame@NodeInstance").toReal();
+        timelineFrames.setValue(value, frame);
+
+        return;
+    } else if (modelNode().hasId() && timelineIsActive() && currentTimeline().hasKeyframeGroup(modelNode(), name)) {
+        QmlTimelineKeyframeGroup timelineFrames(currentTimeline().keyframeGroup(modelNode(), name));
+
+        Q_ASSERT(timelineFrames.isValid());
+
+        if (timelineFrames.isRecording()) {
+            qreal frame = currentTimeline().modelNode().auxiliaryData("currentFrame@NodeInstance").toReal();
+            timelineFrames.setValue(value, frame);
+
+            return;
+        }
+    }
+
     if (isInBaseState()) {
         modelNode().variantProperty(name).setValue(value); //basestate
     } else {
@@ -83,6 +111,14 @@ QmlModelState QmlObjectNode::currentState() const
         return QmlModelState();
 }
 
+QmlTimeline QmlObjectNode::currentTimeline() const
+{
+    if (isValid())
+        return view()->currentTimeline();
+    else
+        return QmlTimeline();
+}
+
 bool QmlObjectNode::isRootModelNode() const
 {
     return modelNode().isRootNode();
@@ -161,6 +197,9 @@ bool QmlObjectNode::propertyAffectedByCurrentState(const PropertyName &name) con
     if (currentState().isBaseState())
         return modelNode().hasProperty(name);
 
+    if (timelineIsActive() && currentTimeline().hasTimeline(modelNode(), name))
+        return true;
+
     if (!currentState().hasPropertyChanges(modelNode()))
         return false;
 
@@ -172,6 +211,21 @@ QVariant QmlObjectNode::modelValue(const PropertyName &name) const
     if (!isValid())
         throw new InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
 
+    if (timelineIsActive() && currentTimeline().hasTimeline(modelNode(), name)) {
+        QmlTimelineKeyframeGroup timelineFrames(currentTimeline().keyframeGroup(modelNode(), name));
+
+        Q_ASSERT(timelineFrames.isValid());
+
+        qreal frame = currentTimeline().modelNode().auxiliaryData("currentFrame@NodeInstance").toReal();
+
+        QVariant value = timelineFrames.value(frame);
+
+        if (!value.isValid()) //interpolation is not done in the model
+            value = instanceValue(name);
+
+        return value;
+    }
+
     if (currentState().isBaseState())
         return modelNode().variantProperty(name).value();
 
@@ -209,7 +263,7 @@ QString QmlObjectNode::stripedTranslatableText(const PropertyName &name) const
             return regularExpressionPatter.cap(2);
         return instanceValue(name).toString();
     }
-    return modelNode().variantProperty(name).value().toString();
+    return instanceValue(name).toString();
 }
 
 QString QmlObjectNode::expression(const PropertyName &name) const
@@ -239,6 +293,11 @@ bool QmlObjectNode::isInBaseState() const
     return currentState().isBaseState();
 }
 
+bool QmlObjectNode::timelineIsActive() const
+{
+    return currentTimeline().isValid();
+}
+
 bool QmlObjectNode::instanceCanReparent() const
 {
     return isInBaseState();
@@ -303,9 +362,17 @@ void QmlObjectNode::destroy()
 
     removeAliasExports(modelNode());
 
-    foreach (QmlModelStateOperation stateOperation, allAffectingStatesOperations()) {
+    for (QmlModelStateOperation stateOperation : allAffectingStatesOperations()) {
         stateOperation.modelNode().destroy(); //remove of belonging StatesOperations
     }
+
+    for (const ModelNode &timelineNode : view()->allModelNodes()) {
+        if (QmlTimeline::isValidQmlTimeline(timelineNode)) {
+            QmlTimeline timeline(timelineNode);
+            timeline.destroyKeyframesForTarget(modelNode());
+        }
+    }
+
     removeStateOperationsForChildren(modelNode());
     modelNode().destroy();
 }
@@ -512,7 +579,6 @@ QmlItemNode QmlObjectNode::itemForInstance(const NodeInstance &instance) const
 }
 
 QmlObjectNode::QmlObjectNode()
-    : QmlModelNodeFacade()
 {
 }
 
@@ -559,9 +625,10 @@ bool QmlObjectNode::hasInstanceParent() const
 
 bool QmlObjectNode::hasInstanceParentItem() const
 {
-    return nodeInstance().parentId() >= 0
-            && nodeInstanceView()->hasInstanceForId(nodeInstance().parentId())
-            && QmlItemNode::isItemOrWindow(view()->modelNodeForInternalId(nodeInstance().parentId()));
+    return isValid()
+           && nodeInstance().parentId() >= 0
+           && nodeInstanceView()->hasInstanceForId(nodeInstance().parentId())
+           && QmlItemNode::isItemOrWindow(view()->modelNodeForInternalId(nodeInstance().parentId()));
 }
 
 
@@ -626,4 +693,10 @@ uint qHash(const QmlObjectNode &node)
 {
     return qHash(node.modelNode());
 }
+
+QString QmlObjectNode::simplifiedTypeName() const
+{
+    return modelNode().simplifiedTypeName();
+}
+
 } //QmlDesigner
