@@ -36,6 +36,7 @@
 #include <sourcetool/sourcetool.h>
 #include <colortool/colortool.h>
 #include <texttool/texttool.h>
+#include <timelineeditor/timelineview.h>
 #include <pathtool/pathtool.h>
 
 #include <qmljseditor/qmljseditorconstants.h>
@@ -70,7 +71,7 @@
 #include <QScreen>
 #include <QWindow>
 
-Q_LOGGING_CATEGORY(qmldesignerLog, "qtc.qmldesigner")
+Q_LOGGING_CATEGORY(qmldesignerLog, "qtc.qmldesigner", QtWarningMsg)
 
 using namespace QmlDesigner::Internal;
 
@@ -93,7 +94,7 @@ QmlDesignerPlugin *QmlDesignerPlugin::m_instance = nullptr;
 
 static bool isInDesignerMode()
 {
-    return Core::ModeManager::currentMode() == Core::Constants::MODE_DESIGN;
+    return Core::ModeManager::currentModeId() == Core::Constants::MODE_DESIGN;
 }
 
 static bool checkIfEditorIsQtQuick(Core::IEditor *editor)
@@ -106,7 +107,7 @@ static bool checkIfEditorIsQtQuick(Core::IEditor *editor)
                     || document->language() == QmlJS::Dialect::QmlQtQuick2Ui
                     || document->language() == QmlJS::Dialect::Qml;
 
-        if (Core::ModeManager::currentMode() == Core::Constants::MODE_DESIGN) {
+        if (Core::ModeManager::currentModeId() == Core::Constants::MODE_DESIGN) {
             Core::AsynchronousMessageBox::warning(QmlDesignerPlugin::tr("Cannot Open Design Mode"),
                                                   QmlDesignerPlugin::tr("The QML file is not currently opened in a QML Editor."));
             Core::ModeManager::activateMode(Core::Constants::MODE_EDIT);
@@ -192,7 +193,13 @@ bool QmlDesignerPlugin::delayedInitialize()
 
     d->settings.fromSettings(Core::ICore::settings());
 
-    d->viewManager.registerViewTakingOwnership(new QmlDesigner::Internal::ConnectionView());
+    d->viewManager.registerViewTakingOwnership(new QmlDesigner::Internal::ConnectionView);
+    if (DesignerSettings::getValue(DesignerSettingsKey::ENABLE_TIMELINEVIEW).toBool()) {
+        auto timelineView = new QmlDesigner::TimelineView;
+        d->viewManager.registerViewTakingOwnership(timelineView);
+        timelineView->registerActions();
+    }
+
     d->viewManager.registerFormEditorToolTakingOwnership(new QmlDesigner::SourceTool);
     d->viewManager.registerFormEditorToolTakingOwnership(new QmlDesigner::ColorTool);
     d->viewManager.registerFormEditorToolTakingOwnership(new QmlDesigner::TextTool);
@@ -210,13 +217,13 @@ void QmlDesignerPlugin::extensionsInitialized()
     });
 }
 
-static QStringList allUiQmlFilesforCurrentProject(const Utils::FileName &fileName)
+static QStringList allUiQmlFilesforCurrentProject(const Utils::FilePath &fileName)
 {
     QStringList list;
     ProjectExplorer::Project *currentProject = ProjectExplorer::SessionManager::projectForFile(fileName);
 
     if (currentProject) {
-        foreach (const Utils::FileName &fileName, currentProject->files(ProjectExplorer::Project::SourceFiles)) {
+        foreach (const Utils::FilePath &fileName, currentProject->files(ProjectExplorer::Project::SourceFiles)) {
             if (fileName.endsWith(".ui.qml"))
                 list.append(fileName.toString());
         }
@@ -225,7 +232,7 @@ static QStringList allUiQmlFilesforCurrentProject(const Utils::FileName &fileNam
     return list;
 }
 
-static QString projectPath(const Utils::FileName &fileName)
+static QString projectPath(const Utils::FilePath &fileName)
 {
     QString path;
     ProjectExplorer::Project *currentProject = ProjectExplorer::SessionManager::projectForFile(fileName);
@@ -297,7 +304,7 @@ void QmlDesignerPlugin::showDesigner()
 
     d->mainWidget.initialize();
 
-    const Utils::FileName fileName = Core::EditorManager::currentEditor()->document()->filePath();
+    const Utils::FilePath fileName = Core::EditorManager::currentEditor()->document()->filePath();
     const QStringList allUiQmlFiles = allUiQmlFilesforCurrentProject(fileName);
     if (warningsForQmlFilesInsteadOfUiQmlEnabled() && !fileName.endsWith(".ui.qml") && !allUiQmlFiles.isEmpty()) {
         OpenUiQmlFileDialog dialog(&d->mainWidget);
@@ -374,7 +381,8 @@ void QmlDesignerPlugin::jumpTextCursorToSelectedModelNode()
             if (currentSelectedNode != selectedNode) {
                 int line, column;
                 currentDesignDocument()->textEditor()->convertPosition(nodeOffset, &line, &column);
-                currentDesignDocument()->textEditor()->gotoLine(line, column);
+                // line has to be 1 based, column 0 based!
+                currentDesignDocument()->textEditor()->gotoLine(line, column - 1);
             }
         }
     }
@@ -473,7 +481,7 @@ void QmlDesignerPlugin::switchToTextModeDeferred()
 void QmlDesignerPlugin::emitCurrentTextEditorChanged(Core::IEditor *editor)
 {
     d->blockEditorChange = true;
-    Core::EditorManager::instance()->currentEditorChanged(editor);
+    emit Core::EditorManager::instance()->currentEditorChanged(editor);
     d->blockEditorChange = false;
 }
 
