@@ -25,23 +25,23 @@
 
 #include "linuxdeviceprocess.h"
 
-#include <projectexplorer/runnables.h>
+#include <projectexplorer/runcontrol.h>
 
 #include <utils/environment.h>
-#include <utils/qtcprocess.h>
 
 using namespace ProjectExplorer;
 using namespace Utils;
 
 namespace RemoteLinux {
 
-static QString quote(const QString &s) { return Utils::QtcProcess::quoteArgUnix(s); }
-
 LinuxDeviceProcess::LinuxDeviceProcess(const QSharedPointer<const ProjectExplorer::IDevice> &device,
-        QObject *parent)
+                                       QObject *parent)
     : ProjectExplorer::SshDeviceProcess(device, parent), m_processId(0)
 {
     connect(this, &DeviceProcess::finished, this, [this]() {
+        m_processId = -1;
+    });
+    connect(this, &DeviceProcess::started, this, [this]() {
         m_processId = 0;
     });
 }
@@ -54,7 +54,7 @@ void LinuxDeviceProcess::setRcFilesToSource(const QStringList &filePaths)
 QByteArray LinuxDeviceProcess::readAllStandardOutput()
 {
     QByteArray output = SshDeviceProcess::readAllStandardOutput();
-    if (m_processId != 0)
+    if (m_processId != 0 || runInTerminal())
         return output;
 
     m_processIdString.append(output);
@@ -70,44 +70,46 @@ QByteArray LinuxDeviceProcess::readAllStandardOutput()
 
 qint64 LinuxDeviceProcess::processId() const
 {
-    return m_processId;
+    return m_processId < 0 ? 0 : m_processId;
 }
 
-QString LinuxDeviceProcess::fullCommandLine(const StandardRunnable &runnable) const
+QString LinuxDeviceProcess::fullCommandLine(const Runnable &runnable) const
 {
-    const Environment env = runnable.environment;
+    CommandLine cmd;
+
+    for (const QString &filePath : rcFilesToSource()) {
+        cmd.addArgs({"test", "-f", filePath});
+        cmd.addArgs("&&");
+        cmd.addArgs({".", filePath});
+        cmd.addArgs(";");
+    }
 
-    QString fullCommandLine;
-    foreach (const QString &filePath, rcFilesToSource())
-        fullCommandLine += QString::fromLatin1("test -f %1 && . %1;").arg(filePath);
     if (!runnable.workingDirectory.isEmpty()) {
-        fullCommandLine.append(QLatin1String("cd ")).append(quote(runnable.workingDirectory))
-                .append(QLatin1String(" && "));
+        cmd.addArgs({"cd", runnable.workingDirectory});
+        cmd.addArgs("&&");
     }
-    QString envString;
-    for (auto it = env.constBegin(); it != env.constEnd(); ++it) {
-        if (!envString.isEmpty())
-            envString += QLatin1Char(' ');
-        envString.append(env.key(it)).append(QLatin1String("='")).append(env.value(it))
-                .append(QLatin1Char('\''));
-    }
-    fullCommandLine.append("echo $$ && ");
-    if (!envString.isEmpty())
-        fullCommandLine.append(envString);
-    fullCommandLine.append(" exec ");
-    fullCommandLine.append(quote(runnable.executable));
-    if (!runnable.commandLineArguments.isEmpty()) {
-        fullCommandLine.append(QLatin1Char(' '));
-        fullCommandLine.append(runnable.commandLineArguments);
-    }
-    return fullCommandLine;
+
+    if (!runInTerminal())
+        cmd.addArgs("echo $$ && ");
+
+    const Environment &env = runnable.environment;
+    for (auto it = env.constBegin(); it != env.constEnd(); ++it)
+        cmd.addArgs(env.key(it) + "='" + env.value(it) + '\'');
+
+    if (!runInTerminal())
+        cmd.addArg("exec");
+
+    cmd.addArg(runnable.executable);
+    cmd.addArgs(runnable.commandLineArguments);
+
+    return cmd.arguments();
 }
 
-QStringList LinuxDeviceProcess::rcFilesToSource() const
+const QStringList LinuxDeviceProcess::rcFilesToSource() const
 {
     if (!m_rcFilesToSource.isEmpty())
         return m_rcFilesToSource;
-    return QStringList() << QLatin1String("/etc/profile") << QLatin1String("$HOME/.profile");
+    return {"/etc/profile", "$HOME/.profile"};
 }
 
 } // namespace RemoteLinux
