@@ -37,7 +37,6 @@
 #include <utils/reloadpromptutils.h>
 #include <utils/fileutils.h>
 
-#include <QTemporaryFile>
 #include <QFileInfo>
 #include <QDir>
 #include <qdebug.h>
@@ -86,23 +85,25 @@ ResourceEditorW::ResourceEditorW(const Core::Context &context,
 
     Core::CommandButton *refreshButton = new Core::CommandButton(Constants::REFRESH, m_toolBar);
     refreshButton->setIcon(QIcon(QLatin1String(":/texteditor/images/finddocuments.png")));
-    connect(refreshButton, SIGNAL(clicked()), this, SLOT(onRefresh()));
+    connect(refreshButton, &QAbstractButton::clicked, this, &ResourceEditorW::onRefresh);
     m_toolBar->addWidget(refreshButton);
 
     m_resourceEditor->setResourceDragEnabled(true);
-    m_contextMenu->addAction(tr("Open File"), this, SLOT(openCurrentFile()));
+    m_contextMenu->addAction(tr("Open File"), this, &ResourceEditorW::openCurrentFile);
     m_openWithMenu = m_contextMenu->addMenu(tr("Open With"));
-    m_renameAction = m_contextMenu->addAction(tr("Rename File..."), this, SLOT(renameCurrentFile()));
-    m_copyFileNameAction = m_contextMenu->addAction(tr("Copy Resource Path to Clipboard"), this, SLOT(copyCurrentResourcePath()));
+    m_renameAction = m_contextMenu->addAction(tr("Rename File..."), this,
+                                              &ResourceEditorW::renameCurrentFile);
+    m_copyFileNameAction = m_contextMenu->addAction(tr("Copy Resource Path to Clipboard"),
+                                                    this, &ResourceEditorW::copyCurrentResourcePath);
 
     connect(m_resourceDocument, &ResourceEditorDocument::loaded,
             m_resourceEditor, &QrcEditor::loaded);
-    connect(m_resourceEditor, SIGNAL(undoStackChanged(bool,bool)),
-            this, SLOT(onUndoStackChanged(bool,bool)));
-    connect(m_resourceEditor, SIGNAL(showContextMenu(QPoint,QString)),
-            this, SLOT(showContextMenu(QPoint,QString)));
-    connect(m_resourceEditor, SIGNAL(itemActivated(QString)),
-            this, SLOT(openFile(QString)));
+    connect(m_resourceEditor, &QrcEditor::undoStackChanged,
+            this, &ResourceEditorW::onUndoStackChanged);
+    connect(m_resourceEditor, &QrcEditor::showContextMenu,
+            this, &ResourceEditorW::showContextMenu);
+    connect(m_resourceEditor, &QrcEditor::itemActivated,
+            this, &ResourceEditorW::openFile);
     connect(m_resourceEditor->commandHistory(), &QUndoStack::indexChanged,
             m_resourceDocument, [this]() { m_resourceDocument->setShouldAutoSave(true); });
     if (debugResourceEditorW)
@@ -136,7 +137,7 @@ Core::IDocument::OpenResult ResourceEditorDocument::open(QString *errorString,
         return openResult;
     }
 
-    setFilePath(FileName::fromString(fileName));
+    setFilePath(FilePath::fromString(fileName));
     setBlockDirtyChanged(false);
     m_model->setDirty(fileName != realFileName);
     m_shouldAutoSave = false;
@@ -150,8 +151,8 @@ bool ResourceEditorDocument::save(QString *errorString, const QString &name, boo
     if (debugResourceEditorW)
         qDebug(">ResourceEditorW::save: %s", qPrintable(name));
 
-    const FileName oldFileName = filePath();
-    const FileName actualName = name.isEmpty() ? oldFileName : FileName::fromString(name);
+    const FilePath oldFileName = filePath();
+    const FilePath actualName = name.isEmpty() ? oldFileName : FilePath::fromString(name);
     if (actualName.isEmpty())
         return false;
 
@@ -207,7 +208,7 @@ bool ResourceEditorDocument::setContents(const QByteArray &contents)
     return success;
 }
 
-void ResourceEditorDocument::setFilePath(const FileName &newName)
+void ResourceEditorDocument::setFilePath(const FilePath &newName)
 {
     m_model->setFileName(newName.toString());
     IDocument::setFilePath(newName);
