@@ -38,6 +38,7 @@
 
 #include <projectexplorer/projectexplorericons.h>
 #include <projectexplorer/runconfiguration.h>
+#include <projectexplorer/runconfigurationaspects.h>
 
 #include <QApplication>
 
@@ -57,11 +58,11 @@ ValgrindToolRunner::ValgrindToolRunner(RunControl *runControl)
     runControl->setIcon(ProjectExplorer::Icons::ANALYZER_START_SMALL_TOOLBAR);
     setSupportsReRunning(false);
 
-    if (IRunConfigurationAspect *aspect = runControl->runConfiguration()->extraAspect(ANALYZER_VALGRIND_SETTINGS))
-        m_settings = qobject_cast<ValgrindBaseSettings *>(aspect->currentSettings());
+    m_settings =
+            qobject_cast<ValgrindBaseSettings *>(runControl->settings(ANALYZER_VALGRIND_SETTINGS));
 
     if (!m_settings)
-        m_settings = ValgrindPlugin::globalSettings();
+        m_settings = ValgrindGlobalSettings::instance();
 }
 
 void ValgrindToolRunner::start()
@@ -75,7 +76,7 @@ void ValgrindToolRunner::start()
     m_progress.reportStarted();
 
 #if VALGRIND_DEBUG_OUTPUT
-    emit outputReceived(tr("Valgrind options: %1").arg(toolArguments().join(QLatin1Char(' '))), DebugFormat);
+    emit outputReceived(tr("Valgrind options: %1").arg(toolArguments().join(' ')), DebugFormat);
     emit outputReceived(tr("Working directory: %1").arg(runnable().workingDirectory), DebugFormat);
     emit outputReceived(tr("Command line arguments: %1").arg(runnable().debuggeeArgs), DebugFormat);
 #endif
@@ -83,11 +84,17 @@ void ValgrindToolRunner::start()
     m_runner.setValgrindExecutable(m_settings->valgrindExecutable());
     m_runner.setValgrindArguments(genericToolArguments() + toolArguments());
     m_runner.setDevice(device());
-    QTC_ASSERT(runnable().is<StandardRunnable>(), reportFailure());
-    m_runner.setDebuggee(runnable().as<StandardRunnable>());
+    m_runner.setDebuggee(runnable());
+
+    if (auto aspect = runControl()->aspect<TerminalAspect>())
+        m_runner.setUseTerminal(aspect->useTerminal());
 
     connect(&m_runner, &ValgrindRunner::processOutputReceived,
             this, &ValgrindToolRunner::receiveProcessOutput);
+    connect(&m_runner, &ValgrindRunner::valgrindExecuted,
+            this, [this](const QString &commandLine) {
+        appendMessage(commandLine, NormalMessageFormat);
+    });
     connect(&m_runner, &ValgrindRunner::processErrorReceived,
             this, &ValgrindToolRunner::receiveProcessError);
     connect(&m_runner, &ValgrindRunner::finished,
@@ -110,8 +117,7 @@ void ValgrindToolRunner::stop()
 
 QString ValgrindToolRunner::executable() const
 {
-    QTC_ASSERT(runnable().is<StandardRunnable>(), return QString());
-    return runnable().as<StandardRunnable>().executable;
+    return runnable().executable;
 }
 
 QStringList ValgrindToolRunner::genericToolArguments() const
@@ -120,17 +126,17 @@ QStringList ValgrindToolRunner::genericToolArguments() const
     QString smcCheckValue;
     switch (m_settings->selfModifyingCodeDetection()) {
     case ValgrindBaseSettings::DetectSmcNo:
-        smcCheckValue = QLatin1String("none");
+        smcCheckValue = "none";
         break;
     case ValgrindBaseSettings::DetectSmcEverywhere:
-        smcCheckValue = QLatin1String("all");
+        smcCheckValue = "all";
         break;
     case ValgrindBaseSettings::DetectSmcEverywhereButFile:
-        smcCheckValue = QLatin1String("all-non-file");
+        smcCheckValue = "all-non-file";
         break;
     case ValgrindBaseSettings::DetectSmcStackOnly:
     default:
-        smcCheckValue = QLatin1String("stack");
+        smcCheckValue = "stack";
         break;
     }
     return {"--smc-check=" + smcCheckValue};
@@ -183,8 +189,8 @@ void ValgrindToolRunner::receiveProcessError(const QString &message, QProcess::P
     if (m_isStopping)
         return;
 
-    QObject *obj = ExtensionSystem::PluginManager::getObjectByName(QLatin1String("AppOutputPane"));
-    if (IOutputPane *pane = qobject_cast<IOutputPane *>(obj))
+    QObject *obj = ExtensionSystem::PluginManager::getObjectByName("AppOutputPane");
+    if (auto pane = qobject_cast<IOutputPane *>(obj))
         pane->popup(IOutputPane::NoModeSwitch);
 }
 
