@@ -54,7 +54,7 @@ QString testDataDir = QLatin1String(SRCDIR "/data");
 static QmlProjectItem *loadQmlProject(QString name, QString *error)
 {
     return QmlProjectFileFormat::parseProjectFile(
-                Utils::FileName::fromString(testDataDir).appendPath(name).appendString(".qmlproject"), error);
+                Utils::FilePath::fromString(testDataDir).pathAppended(name + ".qmlproject"), error);
 }
 
 void tst_FileFormat::testFileFilter()
@@ -223,8 +223,9 @@ void tst_FileFormat::testLibraryPaths()
 
     project->setSourceDirectory(testDataDir);
 
-    QStringList expectedPaths(QStringList() << SRCDIR "/otherLibrary"
-                                            << SRCDIR "/data/library");
+    const QDir base(testDataDir);
+    const QStringList expectedPaths({base.relativeFilePath(SRCDIR "/otherLibrary"),
+                                     base.relativeFilePath(SRCDIR "/data/library")});
     qDebug() << expectedPaths << project->importPaths();
     QCOMPARE(project->importPaths().toSet(), expectedPaths.toSet());
     delete project;
